/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

// sample of xdo BLOB for DB2
/*
 * Filename       : TxdoCRUDFDB2.cpp
 * Description    : test add, update, delete, retrieve of BLOB for DB2 - thru fileName.
 *                  (use the DB2 SAMPLE database, table EMP_PHOTO)   
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDB2.hpp>
#include <DKBlobDB2.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TxdoCRUDFDB2 <database> <userid> <pw>" << endl;
     return;
    }
    //### get input parameters
    DKString database = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    //### setup table, colume, predicate, picture file00
    DKString fileName1 =  "e:\\cmbroot\\samples\\java\\dl\\images\\eagle.gif"; //for add
    DKString fileName2 =  "e:\\cmbroot\\samples\\java\\dl\\images\\vir2.gif";   //for update
    DKString table = "EMP_PHOTO";                           //from SAMPLE database of DB2
    DKString column = "PICTURE";                            //blob column 
    DKString predicate0 = "EMPNO = '000130' and PHOTO_FORMAT = 'gif'" ; //primary key of exist blob
    DKString predicate1 = "EMPNO = '000210' and PHOTO_FORMAT = 'gif'" ; //primary key for null blob
     
   try {
      DKDatastoreDB2 dsDB2;
      cout<< "Datastore DB2 created"<<endl;
      cout<<"connecting to datastore"<<endl;
      dsDB2.connect(database,userid,pw);
      cout<<"datastore connected "<<database<<" userid - "<< userid << endl;
      
      //### setup XDO
      DKBlobDB2* axdo = new DKBlobDB2(&dsDB2);
      DKPidXDODB2*  apid = new DKPidXDODB2;
      apid->setTable(table);
      apid->setColumn(column);
      apid->setDataPredicate(predicate0);   //exist blob
      axdo->setPidObject(apid);
      cout<<"table= "<<axdo->getTable()<<endl;
      cout<<"column= "<<((DKPidXDODB2*)(axdo->getPid()))->getColumn()<<endl;
      cout<<"dataPredicate= "<<((DKPidXDODB2*)(axdo->getPid()))->getDataPredicate()<<endl;

      //### retrieve the exist blob - 000130 gif
      cout<<"length="<<axdo->length()<<endl;   //in memory
      cout<<"getSize="<<axdo->getSize()<<endl; //in datastore
      cout<<"about to do retrieve(fileName)..."<<endl;
      axdo->retrieve("F000130.gif");
      cout<<"retrieve successfully..."<<endl;
      cout <<"about to do open()"<<endl;
      axdo->setInstanceOpenHandler("lviewpro", TRUE);   //use lviewpro in NT
      axdo->open("000130.gif");     //iexplore requires the file extension  

      //### add with fileName1 to the null blob - 000210 gif
      apid->setDataPredicate(predicate1);  //null blob
      axdo->setPidObject(apid);
      cout<<"table= "<<axdo->getTable()<<endl;
      cout<<"column="<<axdo->getColumn()<<endl;
      cout<<"dataPredicate="<<axdo->getDataPredicate()<<endl;
      cout<<"before add getSize="<<axdo->getSize()<<endl; //size from database 
      cout<<"about to do add(fileName)...."<<endl;
      axdo->add(fileName1);       
      cout<<"add successfully.... "<<endl;
      cout<<"after add getSize="<<axdo->getSize()<<endl; //size from database 

      //### retrieve to fileName and open - 000210 gif
      cout<<"about to do retrieve() of added blob..."<<endl;
      axdo->retrieve();
      cout<<"retrieve successfully..."<<endl;
      cout <<"about to do open()"<<endl;
      axdo->setInstanceOpenHandler("lviewpro", TRUE);   //use lviewpro in NT
      axdo->open();     //iexplore requires the file extension 
 
      //### update with fileName2 the added blob - 00210 gif
      //== before update, the length of memory is the same
      cout<<"before update, length = "<<axdo->length()<<endl;
      cout<<"about to do update(fileName2)..."<<endl;
      axdo->update(fileName2);  
      cout<<"After update, do retrieve and open() to verify it is updated."<<endl;
      axdo->retrieve();
      cout<<"after update & retrieve, length = "<<axdo->length()<<endl;
      axdo->open();     //iexplore requires the file extension 

      //### delete the added blob - 000210 gif 
      cout<<"Delete the above added blob ..."<<endl;
      axdo->del();
      cout<<"delete successfully!"<<endl;
      dsDB2.commit();
      delete axdo;
      delete apid;
      dsDB2.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

