/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolDL.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKBlobDL.hpp>
#include <DKDatastoreDL.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefDL.hpp>
#include <DKAttrDefDL.hpp>
#include <DKServerDefDL.hpp>
#include <DKIndexClassDefDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryDLLName = "libcmbdlfac815";

   try {
      DKDatastorePool* dsrDL = new DKDatastorePool("DL", factoryDLLName);
      dsrDL->setDatastoreName(libsrv);
      dsrDL->setMaxPoolSize(10);
      dsrDL->setMinPoolSize(3);
      dsrDL->setTimeOut(1); 

      dsrDL->initConnections(userid, pw, 2);
      //dsrDL->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrDL->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrDL->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrDL->getTimeOut()<<endl;

      dkDatastore *dsDL1, *dsDL2, *dsDL3, *dsDL4;;
      dsDL1 = dsrDL->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsDL1(ds1) =  " << dsDL1 << endl;
      dsDL2 = dsrDL->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsDL2(ds2) =  " << dsDL2 << endl;

      DKDatastoreSession *dsIdDL1, *dsIdDL2, *dsIdDL3, *dsIdDL4;
      dsIdDL1 = dsrDL->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdDL1(dsid1)="<< dsIdDL1->getDatastore()
        <<", sessionId=" << dsIdDL1->getId()<<endl;
      dsIdDL2 = dsrDL->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdDL2(dsid2)="<<dsIdDL2->getDatastore()
        <<", sessionId=" << dsIdDL2->getId()<<endl;

      dsrDL->returnConnection(dsIdDL2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdDL2(dsid2) =  " << 
        dsIdDL2->getDatastore() << ", sessionId=" << dsIdDL2->getId()<<endl;
      dsrDL->returnConnection(dsDL2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsDL2(ds2) =  " << dsDL2 <<endl; 

      dsIdDL3 = dsrDL->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdDL3(ds2)=" <<
        dsIdDL3->getDatastore() << ", sessionId=" << dsIdDL3->getId()<<endl;
      dsIdDL4 = dsrDL->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdDL4(dsid3)=" <<
        dsIdDL4->getDatastore() << ", sessionId=" << dsIdDL4->getId() <<endl;


      dsIdDL4 = dsrDL->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdDL4(dsid4)=" <<
        dsIdDL4->getDatastore() << ", sessionId=" << dsIdDL4->getId()<<endl;

      dsDL1 = dsrDL->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsDL1(ds3) =  " << dsDL1 <<endl;
      dsDL2 = dsrDL->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsDL2(ds4) =  " << dsDL2 <<endl;
 
      dsDL3 = dsrDL->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsDL3(ds5) =  " << dsDL3 <<endl;
      dsDL4 = dsrDL->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsDL4(ds6) =  " << dsDL4 <<endl;


      dsIdDL3 = dsrDL->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdDL3(dsid2)=" <<
      dsIdDL3->getDatastore() << ", sessionId=" << dsIdDL3->getId()<<endl;


/**/
      //**** use dsDL2 to do ListCatalog
      cout<<"||**** start: use dsDL2 to do ListCatalog ****||"<<endl;
      DKSequentialCollection *pCol = 0;
      dkIterator *pIter = 0;
      DKSequentialCollection *pCol2 = 0;
      dkIterator *pIter2 = 0;
      DKServerDefDL *pSV = 0;
      DKIndexClassDefDL *pEnt = 0;
      DKString strServerName;
      DKString strServerType;
      DKString strIndexClass;
      DKAttrDefDL *pAttr = 0;
      DKAny a;
      DKAny *pA = 0;
      long i = 0;
      long j = 0;
      cout << "list servers" << endl;
      a = dsDL2->listDataSources();
      pCol = (DKSequentialCollection*)((dkCollection*)a);
      pIter = pCol->createIterator();
      while (pIter->more() == TRUE)
      {
       i++;
       pSV = (DKServerDefDL*)((void*)(*pIter->next()));
       strServerName = pSV->getName();
       strServerType = pSV->getServerType();
       cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl; 
       delete pSV;
      }
      delete pIter;
      delete pCol;
      cout << i << " servers listed" << endl;
      cout << "list index classes" << endl;
      pCol = (DKSequentialCollection*)((dkCollection*)dsDL2->listEntities());
      pIter = pCol->createIterator();
      i = 0;
      while (pIter->more() == TRUE)
      {
       i++;
       pEnt = (DKIndexClassDefDL*)((void*)(*pIter->next()));
       strIndexClass = pEnt->getName();
       cout << "index class name [" << i << "] - " << strIndexClass << endl;
       cout << "  list attributes for " << strIndexClass << " index class" << endl;
       pCol2 = (DKSequentialCollection*)((dkCollection*)dsDL2->listEntityAttrs(strIndexClass));
       pIter2 = pCol2->createIterator();
       j = 0;
       while (pIter2->more() == TRUE)
       {
        j++;
        pA = pIter2->next();
        pAttr = (DKAttrDefDL*) pA->value();
        cout << "    Attribute name [" << j << "] - " << pAttr->getName() << endl;
        cout << "      datastoreName " << pAttr->datastoreName() << endl;
        cout << "      datastoreType " << pAttr->datastoreType() << endl;
        cout << "      attributeOf   " << pAttr->getEntityName() << endl;
        cout << "      type          " << pAttr->getType() << endl;
        cout << "      size          " << pAttr->getSize() << endl;
        cout << "      id            " << pAttr->getId() << endl;
        cout << "      nullable      " << pAttr->isNullable() << endl;
        cout << "      precision     " << pAttr->getPrecision() << endl;
        cout << "      scale         " << pAttr->getScale() << endl;
        cout << "      string type   " << pAttr->getStringType() << endl;
        delete pAttr;
       }
       cout << "  " << j << " attributes listed for " << strIndexClass << " index class" << endl;
       delete pIter2;
       delete pCol2;
       delete pEnt;
      }
      delete pIter;
      delete pCol;
      cout << i << " index classes listed" << endl;
      cout<<"||**** end: use dsDL2 to do ListCatalog ****||"<<endl;
      //****end of use dsDL2 to do ListCatalog 
/**/
      dsrDL->returnConnection(dsDL2);
      cout<<" #returnConnection, dsDL2(ds4) =  "<< dsDL2 <<endl;
      dsDL4 = dsrDL->getConnection(userid, pw);
      cout<<" #getConnection, dsDL4 =  "<< dsDL4 <<endl;
      cout<<"    dsDL4 == dsDL2 =  " <<(dsDL4 == dsDL2)<<endl;
/**
      //**** use dsDL4 to retrieve XDO - need to change the itemId and partId  
      cout<<"||**** start: use dsDL4 to retrieve XDO ****||"<<endl;
       DKString itemId="V02MP2@#TOH9II$O";
       int partId = 1;
       DKBlobDL* axdo = new DKBlobDL((DKDatastoreDL*)dsDL4);
       DKPidXDODL*  apid = new DKPidXDODL;
       apid ->setPartId(partId);
       apid ->setPrimaryId(itemId);
       axdo ->setPidObject(apid);
       cout<<"itemId= "<<((DKPidXDODL*)axdo->getPidObject())->getPrimaryId()<<endl;
       cout<<"partId= "<<((DKPidXDODL*)axdo->getPidObject())->getPartId()<<endl;
       cout<<"before retrieve..."<<endl;
       cout <<" length of lobdata = "<<axdo->length()<<endl;
       cout<<"about to call retrieve()..."<<endl;
       axdo->retrieve();
       cout<<"after retrieve..."<<endl;
       cout <<" length of lobdata = "<<axdo->length()<<endl;
       cout <<" mimeType = "<<axdo->mimeType()<<endl;
       cout <<" size of lobdata = "<<axdo->getSize()<<endl;
       cout<<" createdTimestamp="<<axdo->getCreatedTimestamp()<<endl;
       cout<<" updatedTimestamp="<<axdo->getUpdatedTimestamp()<<endl;
       cout<<"about to do open()..."<<endl;
       axdo->setInstanceOpenHandler("notepad", TRUE);
       axdo->open();
       delete axdo;
       delete apid;   
      cout<<"||**** end: use dsDL4 to retrieve XDO ****||"<<endl;
      //****end of use dsDL4 to retrieve XDO
**/
      dsrDL->returnConnection(dsDL1);
      cout<<" #returnConnection, dsDL1(ds3) =  " << dsDL1 <<endl;
      dsrDL->returnConnection(dsDL3);
      cout<<" #returnConnection, dsDL3(ds5) =  " << dsDL3 <<endl;
      dsrDL->returnConnection(dsIdDL4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdDL4(dsid4) =  " <<
        dsIdDL4->getDatastore() << ", sessionId=" << dsIdDL4->getId() <<endl;
     
/***
      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;
***/

      dsDL2 = (DKDatastoreDL*)dsrDL->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsDL2(ds7) =  " << dsDL2<<endl;
      cout<<" #call delete dsrDL..." <<endl;
      delete dsrDL;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

