/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

// sample List ACL information
/*
 * Filename       : TListACLDL.cpp
 * Description    : List ACL information
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKAccessControlDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TListACLDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKDatastoreDL dsDL; // must be outside try/catch block
   try {
      cout << "Datastore DL created" << endl;
      cout << "connecting to datastore" << endl;
      dsDL.connect(libsrv,userid,pw);
      cout << "datastore connected " << libsrv << " userid - " << userid << endl;
      
      cout << "datastore disconnected" << endl;
      
      DKAccessControlDL acDL(&dsDL);
            
      DKSequentialCollection * aclList = (DKSequentialCollection *)acDL.listAccessControlLists();
      DKAccessControlListDL * pACL;
      
      if (aclList != NULL)
      {
         dkIterator * pIter = aclList->createIterator();
                         
	 cout << "There are " << aclList->cardinality() << " ACL(s) defined in the system." << endl;
         while (pIter->more()) 
         {
            pACL = (DKAccessControlListDL *)((void*)(*pIter->next())); 
            cout << "ACL Name = " << pACL->getName() << endl;           
            cout << "ACL Description = " << pACL->getDescription() << endl;           
            DKSequentialCollection * aclData = (DKSequentialCollection *)pACL->getAccessData();

            if (aclData != NULL) 
            {
               dkIterator * pIter1 = aclData->createIterator();
               DKAccessDataDL * pData;
               cout << "There are " << aclData->cardinality() << " Access Data defined in this ACL." << endl;               
               while (pIter1->more()) 
               {      
                 pData = (DKAccessDataDL *)((void*)(*pIter1->next())); 
                 cout << "   Command = " << pData->getCommand() << endl;
                 cout << "   Patron Type = " << pData->getPatronType() << endl;
                 cout << "   Patron = " << pData->getPatron() << endl;
                 cout << "   Privilege = " << pData->getPrivilegeName() << endl;  
                 cout << "   --------------------" << endl;
                 //delete pData;  
               }
               delete pIter1;                          
            }           
            delete pACL; 
         }
         delete pIter;  
         delete aclList;
      }
      
      dsDL.disconnect();
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

