/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   *
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample connect/execute/retrieve/disconnect to OD
/*
 * Filename       : TRetrieveFolderOD.cpp
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/folders.
 *                  Retrieve data for a given number of hits and store the
 *                   data from each hit to a disk file
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

#ifdef WIN32
#include <io.h>
#include <conio.h>
#endif
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKDatastoreExtOD.hpp>
#include <DKEnvironment.hpp>
//#include <arscmsrv.h>

static DKBoolean CompareIgnoreCase(DKString str1, DKString str2);

static void displayHit(int cnt, DKDDO* p);

static void displayDDO(int            cnt,
                       DKDatastoreOD* dsOD,
                       DKDDO*         p,
                       DKBoolean      fAfp2Html);

static DKString Op2Str(short Op);

static void setFolderQueryCriteria(int testcase, DKString libsrv);

static DKString folderName;

static long      NumCriteria = 0;
static DKString* CriteriaNames = 0;
static DKString* CriteriaValues = 0;
static short*    CriteriaOperators = 0;

static UINT MaxCount = 5;

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;

  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TRetrieveFolderOD <libsrv> <userid> <pw> <connect_string> <configuration string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];

  DKDatastoreOD* dsOD; // must be outside try/catch block

  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;

  if (configuration_string != "")
  {
    configuration_string += ";ENTITY_TYPE=TEMPLATES";
    dsOD  = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD  = new DKDatastoreOD("ENTITY_TYPE=TEMPLATES");
  }
  cout << "Datastore OD created" << endl;

  DKFolderOD* folder = NULL;
  DKSearchCriteria** criteria = NULL;
  try
  {
    dkResultSetCursor* pCur = 0;

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    // Select a testcase by changing this number!!!
    int testcase = 3;
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    setFolderQueryCriteria(testcase, libsrv);

    cout << "\nUsing folder =>" << folderName << "<=" << endl;

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    cout << "connect_string=" << connect_string << endl;

    DKNVPair parms[3];
    parms[0] = DKNVPair("MAX_RESULTS", DKString(MaxCount));
    parms[1] = DKNVPair("CONTENT", DKString("ATTRONLY"));
    parms[2] = DKNVPair( DK_CM_PARM_END, DKAny((long)0) );

    // List folder names
    DKDatastoreDefOD* dsDef = (DKDatastoreDefOD*)dsOD->datastoreDef();
    dkCollection* folders = dsDef->listSearchTemplates();
    dkIterator* pIter = folders->createIterator();
    DKFolderOD* fldOD;
    int i = 0;
    DKBoolean foundIt = FALSE;
    while (pIter->more() == TRUE)
    {
      i++;
      fldOD = (DKFolderOD*)(void*)(*pIter->next());
      DKString thisFolderName = fldOD->getName();
      cout << "folder[" << i << "]: " << thisFolderName << endl;
      if (thisFolderName == folderName)
      {
        folder = fldOD;
        foundIt = TRUE;
        break;
      }
    }
    cout << "\nUsing folder =>" << folderName << "<=" << endl;
    if (foundIt)
    {
      // Set the search criteria
      criteria = folder->getCritInfo();
      int ii = 0;
      int numQueryCrit = folder->getNumQueryCrit();
      for (int k = 0; k < numQueryCrit; k++)
      {
        DKString critName = criteria[k]->getName();
        if (critName == CriteriaNames[ii])
        {
          criteria[k]->setValue(CriteriaValues[ii]);
          criteria[k]->setOperator(CriteriaOperators[ii]);
          cout << "  Name: " << critName << endl;
          cout << "  =>value: " << CriteriaValues[ii] << endl;
          cout << "  =>operator: " << Op2Str(CriteriaOperators[ii]) << endl;
          ii++;
        }
        if (ii >= NumCriteria)
        {
          break;
        }
      }

      DKTimestamp timeBefore;
      cout << "Before query, time stamp is " << timeBefore.asString(NULL) << endl;
      cout << "executing query" << endl;
      pCur = dsOD->execute(folder,parms,NULL);
      cout << "query executed" << endl;

      DKTimestamp timeAfter;
      cout << "After query, time stamp is " << timeAfter.asString(NULL) << endl;
      cout << "cardinality: " << pCur->cardinality() << endl;
      cout << "datastore executed query" << endl;

      if (pCur == NULL)
        cout << "cursor is NULL!!!" << endl;

      USHORT cnt = 0;
      DKDDO *p = 0;

      DKSequentialCollection* pColHitList = new DKSequentialCollection();
      while (pCur->isValid())
      {
        p = pCur->fetchNext();
        if (p != NULL)
        {
          cnt++;
          i = pCur->getPosition();
          displayHit(cnt, p);
          // save the ddo to the end of the hitlist
          pColHitList->addElement(DKAny(p));
        }
      }
      cout << endl;

      dkIterator* pIter = 0;
      cnt = 0;
      if (pColHitList->cardinality() > 0)
      {
        pIter = pColHitList->createIterator();
        while (pIter->more() == TRUE)
        {
          cnt++;
          DKDDO* ddoold = (DKDDO*)(pIter->next()->value());

          DKString pidStr = ((DKPid*)ddoold->getPidObject())->pidString();
          //cout << " pidString : " << pidStr << endl;

          // the retrieval action
          DKPid* pid = new DKPid(pidStr);

          short id, docType = 0;
          DKAny a;
          if ((id = ddoold->propertyId(DK_CM_PROPERTY_ITEM_TYPE)) > 0)
          {
            a = ddoold->getProperty(id);
            if (a.typeCode() == DKAny::tc_ushort)
              docType = (short)(USHORT)a;
            else
              docType = a;
          }
          if (docType == DK_CM_DOCUMENT)
          {
            cout << "create the DDO from the pidstring..." << endl;
            try
            {
              p = dsOD->createDDO(ddoold->getObjectType(), DK_CM_DOCUMENT);
              p->setPidObject(pid);

              dsOD->retrieveObject((dkDataObject*)p);
              displayDDO(cnt, dsOD, p, FALSE);
            }
            catch (DKException &exc)
            {
              cout << "Error id " << exc.errorId() << endl;
              cout << "Exception id " << exc.exceptionId() << endl;
              for (unsigned long i = 0; i< exc.textCount();i++)
              {
                cout << "Error text: " << exc.text(i) << endl;
              }
              for (unsigned long g=0; g< exc.locationCount();g++)
              {
                const DKExceptionLocation* p = exc.locationAtIndex(g);
                cout << "Filename: " << p->fileName() << endl;
                cout << "Function: " << p->functionName() << endl;
                cout << "LineNumber: " << p->lineNumber() << endl;
              }
              cout << "Exception Class Name: " << exc.name() << endl;
            }
          }
          delete pid;
          delete ddoold;
          delete p;
        }
        delete pIter;
      }
      delete pColHitList;

      DKTimestamp timeAfter2;
      cout << "\nAfter results processed " << timeAfter2.asString(NULL) << endl;
      cout << "\nquery results processed" << endl;
      cout << "Number of items retrieved: " << cnt << endl;
      if (pCur != 0)
        delete pCur;
    }
    else
    {
      cout <<"Folder "<<folderName<<" not found on "<<libsrv<<"!!!"<<endl;
    }
    dsOD->disconnect();
    cout << "\ndatastore disconnected" << endl;
    delete dsOD;
    delete folders;
    delete pIter;
  }

  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
  cout << "done ..." << endl;

  if (CriteriaNames)
    delete [] CriteriaNames;
  if (CriteriaValues)
    delete [] CriteriaValues;
  if (CriteriaOperators)
    delete [] CriteriaOperators;
}

//***********************************************************************
//   Op2Str
//***********************************************************************
static DKString Op2Str(short Op)
{
  DKString OpStr = "";
  switch (Op)
  {
    case DK_CM_OPCODE_EQ         : OpStr = "EQ"         ; break;
    case DK_CM_OPCODE_NEQ        : OpStr = "NEQ"        ; break;
    case DK_CM_OPCODE_GT         : OpStr = "GT"         ; break;
    case DK_CM_OPCODE_LT         : OpStr = "LT"         ; break;
    case DK_CM_OPCODE_LIKE       : OpStr = "LIKE"       ; break;
    case DK_CM_OPCODE_NOTLIKE    : OpStr = "NOTLIKE"    ; break;
    case DK_CM_OPCODE_BETWEEN    : OpStr = "BETWEEN"    ; break;
    case DK_CM_OPCODE_IN         : OpStr = "IN"         ; break;
    case DK_CM_OPCODE_NOTIN      : OpStr = "NOTIN"      ; break;
    case DK_CM_OPCODE_GEQ        : OpStr = "GEQ"        ; break;
    case DK_CM_OPCODE_LEQ        : OpStr = "LEQ"        ; break;
    case DK_CM_OPCODE_NOTBETWEEN : OpStr = "NOTBETWEEN" ; break;
    case DK_CM_OPCODE_AND        : OpStr = "AND"        ; break;
    case DK_CM_OPCODE_OR         : OpStr = "OR"         ; break;
    case DK_CM_OPCODE_NOT        : OpStr = "NOT"        ; break;
    case DK_CM_OPCODE_UNDEFINED  : OpStr = "UNDEFINED"  ; break;
    default                      : OpStr = "Unknown!!!" ; break;
  }
  return OpStr;
}

// --------------------------------------------------------------------
// display hits
// --------------------------------------------------------------------
static void displayHit(int cnt, DKDDO* p)
{
  DKAny    a;
  short    j = 0;

  if (p != NULL)
  {
    try
    {
      int numDataItems = numDataItems = p->dataCount();
      // display attribute (field) names (1st time only)
      if (cnt == 1)
      {
        cout << "" << endl;
        for (j = 1; j <= numDataItems; j++)
        {
          DKString strDataName = p->getDataName(j);
          if (strDataName != DK_CM_DKPARTS   &&
              strDataName != "DKResource"    &&
              strDataName != "DKViews"       &&
              strDataName != "DKLargeObject" &&
              strDataName != "DKPermissions" &&
              strDataName != "DKFixedView"   &&
              strDataName != "DKAnnotations")
          {
            cout << "  \t\"" << strDataName << "\"";
          }
        }
        cout << "" << endl;
      }
      // display attribute (field) values
      for (j = 1; j <= numDataItems; j++)
      {
        a = p->getData(j);
        DKString strDataName = p->getDataName(j);
        if (strDataName != DK_CM_DKPARTS   &&
            strDataName != "DKResource"    &&
            strDataName != "DKViews"       &&
            strDataName != "DKLargeObject" &&
            strDataName != "DKPermissions" &&
            strDataName != "DKFixedView"   &&
            strDataName != "DKAnnotations")
        {
          if (j == 1)
            cout << " " << cnt << ".\t";
          else
            cout << "\t";

          if ((a.typeCode() == DKAny::tc_string) ||
              (a.typeCode() == DKAny::tc_long)   ||
              (a.typeCode() == DKAny::tc_short)  ||
              (a.typeCode() == DKAny::tc_date)   ||
              (a.typeCode() == DKAny::tc_time)   ||
              (a.typeCode() == DKAny::tc_timestamp))
          {
            cout << "\"" << a << "\"";
          }
          else
            cout << "";
        }
      }
      cout << "" << endl;
    }
    catch (DKException exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for (unsigned long i = 0; i< exc.textCount();i++)
      {
        cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
        const DKExceptionLocation* p = exc.locationAtIndex(g);
        cout << "Filename: " << p->fileName() << endl;
        cout << "Function: " << p->functionName() << endl;
        cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
  }
}

// --------------------------------------------------------------------
// Retrieve and display results
// --------------------------------------------------------------------
static void displayDDO(int            cnt,
                       DKDatastoreOD* dsOD,
                       DKDDO*         p,
                       DKBoolean      fAfp2Html)
{
  DKString DocFile, DataFile, ResourceFile, TocFile, LogViewFile;

  // create the output directory if it does not exist
#ifdef WIN32
  DKString OutDir = DKString("c:\\cmbuild\\output\\");
#else
  DKString OutDir = DKString("/archive/bulldog/eip/samples/output/");
#endif
  int rc = access( OutDir, 0 );
  if ( rc == -1 )  // does not exist
  {
    // create the directory
    rc = DKDatastoreOD::createDirectory(OutDir);
    // if we can't create the directory, quit now!
    if (rc == FALSE)
    {
      cout << "\n***Cannot create output directory \"" << OutDir << "\"!!!\n" << endl;
      return;
    }
  }

  dkDataObjectBase * pDO = NULL;
  dkCollection* pCol = NULL;
  dkIterator* pIter = NULL;
  DKAnnotationOD* annotation = NULL;

  DKDDO *pDDO = 0;
  DKBlobOD *pXDO = 0;

  DKPidXDOOD *pidXDO = 0;
  DKPid *pid = 0;

  DKAny a;
  int i = 0;
  short k = 0;
  short j = 0;
  short val = 0;
  int numDataItems = 0;
  long pidIdCnt = 0;
  long pidIndex = 0;

  DKString strData = "";
  DKString strDataName = "";
  DKString strPid;

  DKString DataString;
  FILE *myLogViewFile,
       *myResourceFile,
       *myTocFile,
       *myDataFile,
       *myDocFile;
  char* DataBuffer = 0;
  //CsvNote * pODNote;
  int numBytes = 0;
  int numNotes = 0;
  DKString noteString;
  DKString extension("");
  DKString MimeType("");
  DKString segmentFile;
  DKString cnt_str;

  cout << "\n\n==========> " << "Item " << cnt << " <=========\n" << endl;
  numDataItems = p->dataCount();
  pid = (DKPid*)p->getPidObject();
  strPid = pid->pidString();
  cout << "pid string " << strPid << endl;
  cout << "pid id string " << pid->getId() << endl;
  strPid = pid->getIdString();
  cout << "pid idString " << strPid << endl;
  pidIdCnt = pid->getIdStringCount();
  cout << "pid idString cnt " << pidIdCnt << endl;
  strPid = pid->getPrimaryId();
  cout << "pid primary id " << strPid << endl;
  pidIndex = 0;
  strPid = pid->getIdString(pidIndex);
  cout << "pid item id " << strPid << endl;

  // Retrieve privileges
  DKDatastoreExtOD* dsODExt = (DKDatastoreExtOD*)dsOD->getExtension(DK_CM_DATASTORE_EXT);
  DKCommonPrivilege* priv = dsODExt->getCommonPrivilege(p);
  if (priv != NULL)
  {
    cout << "privileges are:" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_PRINT) == DK_CM_PRIV_YES)
       cout << " +++can print document" << endl;
    else
       cout << " ---cannot print document" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_VIEW_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can view annotations" << endl;
    else
       cout << " ---cannot view annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_CREATE_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can create annotations" << endl;
    else
       cout << " ---cannot create annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_DELETE_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can delete annotations" << endl;
    else
       cout << " ---cannot delete annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_EDIT_TEXT_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can edit annotations" << endl;
    else
       cout << " ---cannot edit annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_MODIFY_MULTI_VIEWDATA) == DK_CM_PRIV_YES)
       cout << " +++can change logical views" << endl;
    else
       cout << " ---cannot change logical views" << endl;
  }

  k = p->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
  if (k > 0)
  {
    a = p->getProperty(k);
    if (a.typeCode() == DKAny::tc_ushort)
      val = (short)(USHORT)a;
    else
      val = a;
    cout << "******************************" << endl;
    switch (val)
    {
      case DK_CM_DOCUMENT :
        cout << "Item is a document " << endl;
        break;
      case DK_CM_ITEM :
        cout << "Item is an item " << endl;
        break;
    }
    cout << "******************************" << endl;
  }

  cnt_str = DKString(cnt);
  DataFile     = OutDir + DKString("docdata")  + cnt_str;
  ResourceFile = OutDir + DKString("resource") + cnt_str + DKString(".dat");
  TocFile      = OutDir + DKString("Toc") + cnt_str + DKString(".dat");
  LogViewFile  = OutDir + DKString("LogView")  + cnt_str + DKString(".dat");

  // Removed existing output files
  if (access( LPCTSTR(ResourceFile), 0 ) != -1 )
    remove(LPCTSTR(ResourceFile));
  if (access( LPCTSTR(TocFile), 0 ) != -1 )
    remove(LPCTSTR(TocFile));
  if (access( LPCTSTR(LogViewFile), 0 ) != -1 )
    remove(LPCTSTR(LogViewFile));

  cout << "  Number of Data Items: " << numDataItems << endl;
  for (j = 1; j <= numDataItems; j++)
  {
    a = p->getData(j);
    strDataName = p->getDataName(j);

    cout << "    " << j << ". Attribute Name: " << strDataName << endl;
    cout << "       type: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_TYPE) << endl;
    cout << "       nullable: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE) << endl;

    if (strDataName != DK_CM_DKPARTS         &&
        strDataName != "DKResource"    &&
        strDataName != "DKViews"       &&
        strDataName != "DKLargeObject" &&
        strDataName != "DKPermissions" &&
        strDataName != "DKFixedView"   &&
        strDataName != "DKAnnotations")
    {
      cout << "       attribute id: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID) << endl;
    }

    if (a.typeCode() == DKAny::tc_string)
    {
      DKString astring = a;
      cout << "       Attribute Value (string): =>" << astring << "<=" << endl;
    }
    else if (a.typeCode() == DKAny::tc_long /*Integer*/)
    {
      long along = a;
      cout << "       Attribute Value (long): " << along << endl;
    }
    else if (a.typeCode() == DKAny::tc_short)
    {
      short ashort = a;
      cout << "       Attribute Value (short): " << ashort << endl;
    }
    else if (a.typeCode() == DKAny::tc_date)
    {
      DKDate adate = a;
      cout << "      Attribute Value (date): " << adate << endl;
    }
    else if (a.typeCode() == DKAny::tc_time)
    {
      DKTime atime = a;
      cout << "       Attribute Value (time): " << atime << endl;
    }
    else if (a.typeCode() == DKAny::tc_timestamp)
    {
      DKTimestamp atimestamp = a;
      cout << "       Attribute Value (timestamp): " << atimestamp << endl;
    }
    else if (a.typeCode() == DKAny::tc_collection)
    {
      cout << "       Attribute Value is collection" << endl;
      pCol = a;
      pIter = pCol->createIterator();
      i = 0;

      while (pIter->more() == TRUE)
      {
        i++;
        a = *(pIter->next());
        pDO = a;

        if (pDO->protocol() == DK_CM_XDO)
        {
          cout << "         dkXDO object " << i << " in collection" << endl;
          pXDO = (DKBlobOD*)pDO;
          pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
          cout << "           XDO pid string: " << pidXDO->pidString() << endl;

          MimeType = pXDO->getMimeType();
          cout << "MimeType = " << MimeType << endl;

          // Set the EBCDIC->ASCII conversion
          if ( MimeType == "application/ondemand line" )
          {
            cout << "Setting the EBCDIC->ASCII conversion option..." << endl;
            a = (long)DK_CM_TEXT_ASCII;
            pXDO->setOption( DK_CM_OPT_XDO_TEXT_CONVERSION, a);
          }
          else if (fAfp2Html == TRUE && MimeType == "application/afp" )
          {
            cout << "Setting the AFP->HTML conversion option..." << endl;
            a = (long)DK_CM_AFP_HTML;
            pXDO->setOption(DK_CM_OPT_XDO_AFP_CONVERSION, a);
            a = (long)DK_CM_APPLET;
            pXDO->setOption(DK_CM_OPT_XDO_VIEWER, a);
          }

          // Retrieve and open instance handler for an XDO
          pXDO->retrieve();
          cout << "Content Begin:" << endl;

          DataString = pXDO->getContent();
          long DataSize = DataString.length();
          DataBuffer = (char*)LPCTSTR(DataString);
          long item_type = ((DKPidXDOOD*)pXDO->getPidObject())->getType();
          long part_id = ((DKPidXDOOD*)pXDO->getPidObject())->getPartId();
          switch (item_type)
          {
            case DK_CM_ANNOTATION:
              // binary
              cout << "======DK_CM_ANNOTATION (Notes)======" << endl;
              annotation = (DKAnnotationOD*)((DKBlobOD*)pXDO)->getExtension("dkAnnotationExt");
              noteString  = "NOTE: " + annotation->getTimestamp();
              noteString += " by " + annotation->getUserID();
              noteString += " for page " + DKString(annotation->getPageNumber());
              noteString += "\n";
              noteString += annotation->isPublic() ? "Public" : "Private";
              noteString += "--";
              noteString += annotation->isOkToCopy() ? "Can" : "Cannot";
              noteString += " be copied to another server";
              noteString += DKString("\n") + annotation->getText();
              cout << noteString << endl;

              // ======== method #2 ========
              //cout << "====Note====" << endl;
              //cout << "isPublic     = " << annotation->isPublic()      << endl;
              //cout << "isOkToCopy   = " << annotation->isOkToCopy()    << endl;
              //cout << "Color        = " << annotation->getColor()      << endl;
              //cout << "Page Number  = " << annotation->getPageNumber() << endl;
              //cout << "X coordinate = " << annotation->getX()          << endl;
              //cout << "Y coordinate = " << annotation->getY()          << endl;
              //cout << "UserID       = " << annotation->getUserID()     << endl;
              //cout << "Timestamp    = " << annotation->getTimestamp()  << endl;
              //cout << "GroupName    = " << annotation->getGroupName()  << endl;
              //cout << "NoteID       = " << annotation->getPart()       << endl;
              //cout << "Text         = " << annotation->getText()       << endl;
              break;
            case DK_OD_VIEW:
              // binary
              cout << "======DK_OD_VIEW (Logical Views)======" << endl;
              myLogViewFile = fopen(LogViewFile,"ab+");
              numBytes = fwrite(DataBuffer,1,DataSize,myLogViewFile);
              fclose(myLogViewFile);
              break;
            case DK_OD_SEGMENT:
              // binary
              if ( extension == "" )
              {
                if ( MimeType == "application/afp" )
                  extension = "afp";
                else if ( MimeType == "application/ondemand line" )
                  extension = "lin";
                else if ( MimeType == "text/plain" )
                  extension = "txt";
                else if ( MimeType == "image/bmp" )
                  extension = "bmp";
                else if ( MimeType == "image/gif" )
                  extension = "gif";
                else if ( MimeType == "image/jpeg" )
                  extension = "jpg";
                else if ( MimeType == "image/pcx" )
                  extension = "pcx";
                else if ( MimeType == "image/tif" )
                  extension = "tif";
                else if ( MimeType == "application/pdf" )
                  extension = "pdf";
                else
                  extension = "dat";
              }
              cout << "======DK_OD_SEGMENT (Doc Data) part_id=" << part_id << "======" << endl;
              if (fAfp2Html == TRUE && extension == "afp")
                extension = "htm";
              segmentFile = DataFile + "." + extension;
              if ( part_id == 0 )
              {
                if (access( LPCTSTR(segmentFile), 0 ) != -1 )
                  remove(LPCTSTR(segmentFile));
              }
              // The first segment starts at part_id 0
              if ( part_id >= 0)
              {
                myDataFile = fopen(segmentFile,"ab+");
                numBytes = fwrite(DataBuffer,1,DataSize,myDataFile);
                fclose(myDataFile);
              }
              break;
          }
          cout << endl << "Content End" << endl;
          //pXDO->open();
        }
      }
      if (pIter != 0)
        delete pIter;
    }
    else if (a.typeCode() != DKAny::tc_null)
    {
      cout << "       Attribute Value (non NULL): " << a << endl;

      if (strDataName == "DKResource"    ||
          strDataName == "DKFixedView"   ||
          strDataName == "DKLargeObject")
      {
        pDO = (dkDataObjectBase*)a;

        if (pDO->protocol() == DK_CM_XDO)
        {
          cout << "         dkXDO object " << endl;
          pXDO = (DKBlobOD*)pDO;
          pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
          cout << "           XDO pid string: " << pidXDO->pidString() << endl;

          // Retrieve and open instance handler for an XDO
          pXDO->retrieve();
          //pXDO->open();

          DataString = pXDO->getContent();
          long DataSize = DataString.length();
          DataBuffer = (char*)LPCTSTR(DataString);
          long item_type = ((DKPidXDOOD*)pXDO->getPidObject())->getType();

          if (strDataName == "DKResource" )
          {
            // binary
            cout << "======DK_OD_RESOURCE (AFP Resource Group)======" << endl;
            // I don't think resource is needed for html files!
            if (fAfp2Html != TRUE)
            {
              myResourceFile = fopen(ResourceFile,"ab+");
              numBytes = fwrite(DataBuffer,1,DataSize,myResourceFile);
              fclose(myResourceFile);
            }
          }
          else if (strDataName == "DKFixedView")
          {
            // string
            cout << "======DK_OD_FIXED_VIEW (Fixed View)======" << endl;
            cout << DataString << endl;
          }
          else if (item_type == DK_OD_TOC)
          {
            // binary
            cout << "======DK_OD_TOC (Large Object table of contents)======" << endl;
            myTocFile = fopen(TocFile,"ab+");
            numBytes = fwrite(DataBuffer,1,DataSize,myTocFile);
            fclose(myTocFile);
          }
        }
      }
    }
    else cout << "       Attribute Value is NULL" << endl;
  }

  DocFile = OutDir + DKString("sample") + cnt_str + DKString(".") + extension;
  if (access( LPCTSTR(DocFile), 0 ) != -1 )
    remove(LPCTSTR(DocFile));


  // If the resource file exists, concatenate the resource with data
  BOOL ResourceExists = ( access( LPCTSTR(ResourceFile), 0 ) != -1 );
  if ( ResourceExists )
  {
    char * buffer = new char[100000];
    myDocFile = fopen(DocFile, "wb");
    myResourceFile = fopen(ResourceFile,"rb");
    if ( myResourceFile )
    {
      while ( !feof( myResourceFile ) )
      {
        numBytes = fread( buffer, sizeof( char ), 100000, myResourceFile );
        if ( ferror( myResourceFile ) )
        {
          perror( "Resource read error" );
          break;
        }
        numBytes = fwrite(buffer,1,numBytes,myDocFile);
      }
      fclose(myResourceFile);
      remove(LPCTSTR(ResourceFile));
    }
    segmentFile = DataFile + "." + extension;
    myDataFile = fopen(segmentFile,"rb");
    // copy data
    if ( myDataFile )
    {
      while ( !feof( myDataFile ) )
      {
        numBytes = fread( buffer, sizeof( char ), 100000, myDataFile );
        if ( ferror( myDataFile ) )
        {
          perror( "Data read error" );
          break;
        }
        numBytes = fwrite(buffer,1,numBytes,myDocFile);
      }
      fclose(myDataFile);
      remove(LPCTSTR(segmentFile));
    }
    fclose(myDocFile);
    delete [] buffer;
  }
}

// --------------------------------------------------------------------
// Compare two strings case insensitively
// --------------------------------------------------------------------
static DKBoolean CompareIgnoreCase(DKString str1, DKString str2)
{
#ifdef WIN32
  return (str1.CompareNoCase(str2) == 0);
#else
  return (IString::lowerCase(str1) == IString::lowerCase(str2));
#endif
}

// --------------------------------------------------------------------
// Set the folder query criteria
// --------------------------------------------------------------------
static void setFolderQueryCriteria(int testcase, DKString libsrv)
{
  // Folder and search criteria
  // OD/390 v2.1
  // Folder and search criteria
  if (CompareIgnoreCase(libsrv, "9.91.121.31"))
  {
    NumCriteria = 3;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "CHKSTMN1";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 2:
        folderName = "COMPAT2";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 3:
        folderName = "COMPATD";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 4:
        folderName = "DELINQCY";
        CriteriaNames[0] = "LOAN NUMBER";
        break;
      case 5:
        folderName = "FSINV";
        CriteriaNames[0] = "INVOICE#";
        break;
      case 6:
        folderName = "INSURE";
        CriteriaNames[0] = "POLICY NUMBER";
        break;
      case 7:
        folderName = "INSURTST";
        CriteriaNames[0] = "POLICY NUMBER";
        break;
      case 8:
        folderName = "K0560";
        CriteriaNames[0] = "FIL#";
        break;
      case 9:
        folderName = "KEYBANK";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
    }

    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_EQ;
    CriteriaNames    [1] = "FROM (YYYY-MM-DD)";
    CriteriaValues   [1] = "1991-05-26";
    CriteriaOperators[1] = DK_CM_OPCODE_GEQ;
    CriteriaNames    [2] = "TO   (YYYY-MM-DD)";
    CriteriaValues   [2] = "2001-05-23";
    CriteriaOperators[2] = DK_CM_OPCODE_LEQ;
  }
  // OD/390 v7.1
  else if (CompareIgnoreCase(libsrv, "9.91.121.34"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP data
        folderName = "Credit Card Statements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account";
        CriteriaValues   [0] = "%"; //"000-000-010";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 2:
        folderName = "CHKSTMNT";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "ACCOUNT NUM";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 3:  // line data
        folderName = "Delinquency";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Page Number";
        CriteriaValues   [0] = "0";
        CriteriaOperators[0] = DK_CM_OPCODE_NEQ;
        break;
      case 4:  // line data large object
        folderName = "LOAN LO";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Date";
        CriteriaValues   [0] = "10/01/94";
        CriteriaOperators[0] = DK_CM_OPCODE_EQ;
        break;
      case 5:  // pdf
        folderName = "PDF Landscape";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Region";
        CriteriaValues   [0] = "'%'";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
      case 6:  // AFP
        folderName = "afpdata";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Tax Year";
        CriteriaValues   [0] = "1995";
        CriteriaOperators[0] = DK_CM_OPCODE_EQ;
        break;
    }
  }
  // OD/400 v4.5
  else if (libsrv.length() >= 7 &&
           CompareIgnoreCase(libsrv.subString(1,7),"rdr400g"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "CheckStatements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Customer Name";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 8 &&
           CompareIgnoreCase(libsrv.subString(1,8),"stealth5"))
  {
    folderName = "Credit Card Statements";

    NumCriteria = 1;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    CriteriaNames    [0] = "Name";
    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
  }
  else if (libsrv.length() >= 6 &&
           CompareIgnoreCase(libsrv.subString(1,6),"reagan"))
  {
    folderName = "load-db2-CRD";

    NumCriteria = 1;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    CriteriaNames    [0] = "Account Name";
    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
  }
  else if (libsrv.length() >= 6 &&
           CompareIgnoreCase(libsrv.subString(1,6),"reagan"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "load-db2-CRD";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account Name";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 7 &&
           CompareIgnoreCase(libsrv.subString(1,8),"everest"))
  {
    switch (testcase)
    {
      default:
      case 1:  // PDF
        folderName = "43MBPDF";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Report Title";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 5 &&
           CompareIgnoreCase(libsrv.subString(1,5),"akira"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "rl-DFAC";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "nomcli"; //"exploit";
        CriteriaValues   [0] = "MR PEREZ ERIC"; //"%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 2:
        folderName = "LARpmr94348";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Customer Id";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 3:
        folderName = "Credit Card Statements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account"; ;
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else //if (CompareIgnoreCase(libsrv.subString(1,6),"gunnar"))
  {
    // Single object AFP
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "Credit Card Statements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account";
        CriteriaValues   [0] = "%"; //"000-000-010";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
}
