/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Attributes must first be defined before they can be added to an Item Type definition.  
Some attributes require more information than others, such as the maximum length of a varchar.
Attribute definitions may be re-used under multiple item and component types.

	Some of the possible types of attributes are:

	Description Constant           Object               DKAny Type Code Format (C++ Only)
	----------- ------------------ -------------------- --------------- --------------------------
	Blob        DK_CM_BLOB         DKAny(DKByteArray)   tc_bytearray    n/a
	Char        DK_CM_CHAR         DKAny(char)          tc_char         n/a
	Clob        DK_CM_CLOB         DKAny(DKString)      tc_string       n/a
	Date        DK_CM_DATE         DKAny(DKDate)        tc_date         yyyy-mm-dd
	Decimal     DK_CM_DECIMAL      DKAny(DKDecimal)     tc_decimal      n/a
	Double      DK_CM_DOUBLE       DKAny(double)        tc_double       n/a
	Integer     DK_CM_INTEGER      DKAny(long)          tc_long         n/a
	Short       DK_CM_SHORT        DKAny(short)         tc_short        n/a
	Time        DK_CM_TIME         DKAny(DKTime)        tc_time         hh.mm.ss
	Timestamp   DK_CM_TIMESTAMP    DKAny(DKTimestamp)   tc_timestamp    yyyy-mm-dd-hh.mm.ss.nnnnnn
	Varchar     DK_CM_VARCHAR      DKAny(DKString)      tc_string       n/a
    ----------------------------------------------------------------------------------------------
    Note:  Constants shown are defined in DKConstant.h

        Char / Fixed-Length Character String
            The fixed-length character string (Char) may be used to store a
            string value that is expected to be a specific length, such as a
            single character.  If a value is specified using less than the
            specified size, extra space will be padded on the end.
            
        Varchar / Variable-Length String
            In contrast to the fixed-length character string (Char), the
            variable-length string (Varchar) stores strings of any length
            up to the specified maximum size.  The string's length will
            remain the same when retrieving at a later time.

	    Integer
		    The "Integer" attribute is also referred to as "Long Integer".
			The C++ APIs use a long value with the DKAny with these
			attributes.

    Names
        Attribute names are limited to 15 characters.  Use the description
        field to add more detailed information.

        Names only need to be unique among those of its own namespace.  The 
        Attribute namespace is seperate from the Entity Type namespace, Link Type 
        Namespace, etc.

    Attribute Definition Design-Time Properties
        Each attribute definition may be assigned a number of properties, including
        important information such as it's type to optional information such as setting
        a minimum, maximum, or making it required.  A sub-set of these properties are
        saved persistently with the generic attribute definition in the datastore
        definition.  As the generic attribute definitions are re-used and added to any
        number of Item Types, they retain these persistently stored properties, which
        may be modified when added to an individual Item Type.  These are applicable
        to "Attribute Definition Design-Time".  For example, the minimum and maximum
        may be optionally set at this time.
        
        Most properties that are able to be set & stored persistently with the generic
        attribute definition may be modified when added to each individual Item Type.
        For example, the minimum & maximum value may be specified when the attribute
        definition is first created & added to the datastore definition, and later 
        modified for the specific Item Type when added to it.
        
        Minimum Value / String Length
            Short & Integer
                Short and Integer attributes may have a minimum value, to be enforced
                by the API.  If both the minimum & maximum are set to '0' 
                (min = max = 0), the two properties will be considered "not set" and
                enformcement will be turned off.  If either a minimum or maximum is
                specified, both min & max enforcement will be turned on, using defaults
                if the other is not specified.  For example, if maximum is set to '5'
                and minimum is not altered, the default of '0' will be enforced as the
                minimum.  To specify no minimum and a maximum of '5', specify the
                minimum as the lowest possible integer or short value that will fit
                into a 32 or 64-bit value.

                This property may be adjusted when added to an item type or 
                component type definition.

            Varchar
                Varchar attributes may have a minimum required string length, to be
                enforced by the API.  Any positive value will be enforced. If both
                the minimum & maximum are set to '0' (min = max = 0), the two
                properties will be considered "not set" and enformcement will be
                turned off.  Additionally, min & max string length enforcment may
                be individually turned off by setting the individual property to 
                a negative value.  For example, if the minimum is set to '-1', no
                minimum will be enforced, regardless of what the maximum is set to.

                This property may be adjusted when added to an item type or 
                component type definition.

            Other Types
                Other attribute types are not currently enforced by the API.

        Maximum Value / String Length
            Char / Fixed Length String
                The char attribute requires either this or the size property to
                determine the exact fixed length / number of characters that this
                attribute will use in the persistent datastore.  This will allow
                faster performance than Varchar strings, but at the loss of any
                space not used by strings smaller than the full fixed length.

                The maximum property in this case may not be adjusted to enforce
                a smaller value than the actual size.  The Varchar Maximum 
                property provides this functionality.

            Short & Integer
                Short and Integer attributes may have a maximum value, to be enforced
                by the API.  If both the minimum & maximum are set to '0' 
                (min = max = 0), the two properties will be considered "not set" and
                enformcement will be turned off.  If either a minimum or maximum is
                specified, both min & max enforcement will be turned on, using defaults
                if the other is not specified.  For example, if minimum is set to '-5'
                and maximum is not altered, the default of '0' will be enforced as the
                maximum.  To specify no maximum and a minimum of '-5', specify the
                maximum as the largest possible integer or short value that will fit
                into a 32 or 64-bit value.

                This property may be adjusted when added to an item type or 
                component type definition.

            Varchar
                Varchar attributes may have a maximum string length, to be enforced
                by the API.  Any positive value will be enforced. If both the
                minimum & maximum are set to '0' (min = max = 0), the two
                properties will be considered "not set" and enformcement will be
                turned off.  Additionally, min & max string length enforcment may
                be individually turned off by setting the individual property to 
                a negative value.  For example, if the minimum is set to '-1', no
                minimum will be enforced, regardless of what the maximum is set to.
                
                Size is not the same as maximum (See "Size" section below).  The
                maximum can not be set to a value larger than the actual size.
                However, if the maximum is less than the size, the difference
                results in wasted space.

                This property may be adjusted when added to an item type or 
                component type definition.

            Other Types
                Other attribute types are not currently enforced by the API.

        Size
            Blob & Clob
                Size in this case pertains to the number of bytes allocated for 
                the attribute value.  This is a required property.

            Char / Fixed Length String
                Size in this case is the same as the maximum.  The Size property is
                required, is currently not updateable at a later time, and can not
                be adjusted when adding to an item type or component type definition.
                Although Maximum may be updateable, it may not be used as an
                alternative to adjusting size in this case because it will not be
                enforced by the system.  
            
            Varchar
                Size in this case is the actual maximum number of characters that
                will be allowed for this attribute.  The Size property is required,
                is currently not updateable at a later time, and can not be
                adjusted when adding to an item type or component type definition.
                Maximum can be updated and can be used to enforce a smaller size
                than the actual size allocated.  However, if the maximum is less
                than the size, the difference results in wasted space.

    Item Type Design-Time Properties
        Other properties are not persistently stored with the generic Attribute
        Definitions, but instead only stored in memory with the attribute definition
        object, applicable to the specific Item Type or Component Type that it is
        applied to.  These are considered to be applicable to "Item Type Design-Time". 
        For example, the default value, default type, and is nullable may only be
        specified at this time.

        A number of these will be demonstrated in SItemTypeCreationICM, which include:
        
            - Nullable
            - Unique
            - Text Searchable
            - Default Value
            - Default Type
            - Representative

        IMPORTANT:  The setNullable(), setUnique(), setTextSearchable(), setDefaultValue()
                    setDefaultType(), and setRepresentative() operations only apply to the
                    attribute in memory to be used when adding to an Item or Component Type.
                    It will not be saved persistently within the System's Definition object
                    and will no longer be set when retrieved at a later time.  One would not
                    specify any of the listed attribute properties in this sample because
                    the attribute definition object in local memory is not being added to a
                    Component Type at this time.
            
        Text Searchable
            Attributes in a Data Model may be searched by a text search engine, such as
            IBM DB2 Text Information Extender (TIE) for DB2 v7 or IBM DB2 Net Search
            Extender (NSE) for DB2 v8.  Regardless of this property setting, it may
            still be searched using the standard search mechanismns.
            
        Nullable
            A value does not have to be specified.  No value, or 'null' may be used.
            
        Unique
            Within a Component Type, not other Component Types may exist with the same 
            value.

        Default Value
            The default value property contains the string representation of the 
            default value.  The default / initial value will only be used in accordance
            with the specified "Default Type".  Please refer to the "Default Type" for
            information on turning on "Default Value".  By default, this property
            will be ignored until the "Default Type" is changed from off to on based on
            the rules outlined in "Default Type".
        
            Initial Value
                Currently, default values are assigned as initial values when a new
                Item / DDO is created by the system.
        
            Current Date
                The current date may be specified as the default, to be set 
                automatically by the system to the current date when a new Item 
                / DDO is created.  Enable this for any Date attribute by setting
                the "Default Value" to an empty String, "", and turning Default
                Values on through one of the applicable "Default Type" values.
            
            Current Time
                The current time may be specified as the default, to be set 
                automatically by the system to the current time when a new Item 
                / DDO is created.  Enable this for any Time attribute by setting
                the "Default Value" to an empty String, "", and turning Default
                Values on through one of the applicable "Default Type" values.
            
            Current Timestamp
                The current timestamp may be specified as the default, to be set 
                automatically by the system to the current time when a new Item 
                / DDO is created.  Enable this for any Timestamp attribute by setting
                the "Default Value" to an empty String, "", and turning Default
                Values on through one of the applicable "Default Type" values.
        
            Zero Number
                Any numeric field may be set to '0' by default.  Enable this for
                any numeric attribute by setting the "Default Value" to an empty
                string, "", and turning Default Values on through one of the 
                applicable "Default Type" values.
        
        Default Type
            "Default Type" determines how the "Default Value" will be applied.  By
            default, it will turn off Default Values.  The following "Default Type"
            settings explain how each applies.
            
            Simplified Explanation (RECOMMENDED)
                Simply speaking, default values may be enabled or disabled in the
                following manner:
                
                Default Value Enablement  Constant
                ------------------------  ----------------------------------
                Off (default)             DK_ATTR_TYPE_DEFAULT_UNDEFINED
                On                        DK_ATTR_TYPE_DEFAULT_STRING
                ------------------------------------------------------------
                Note: Constants defined in DKAttrDefICM.h
  
            Advanced Options
                The technical advanced options allow specific uses & applications
                of "Default Value".  For most users, the "Simplified Explanation"
                above should be used.
        
            Default Type  Constant                        Application of Default Value
            ------------  ------------------------------  ----------------------------------------
            Undefined     DK_ATTR_TYPE_DEFAULT_UNDEFINED  Default Value Ignored.  Regardless of
            (OFF)                                         what the Default Value is set to, the
                                                          attribute is considered to have no
                                                          default value based on default type.
            ......................................................................................
            String        DK_ATTR_TYPE_DEFAULT_STRING     Regardless of whether the attribute is
            (ON)                                          defined as nullable or not, the default
                                                          value will be considered as a String
                                                          representation of the value and will be
                                                          converted to the correct type, depending
                                                          on the attribute type and set as the
                                                          default value.
           ......................................................................................
           Float         DK_ATTR_TYPE_DEFAULT_FLOAT      If attribute type is a float, the
                                                         default value will be converted to a
                                                         float object and set as the default
                                                         value.  If the attribute type takes
                                                         a String, it will be set as the string.
                                                         Otherwise it will be ignored if not 
                                                         applicable for that attribute type.
           ......................................................................................
           Decimal       DK_ATTR_TYPE_DEFAULT_DECIMAL    If attribute type is a float, the
                                                         default value will be converted to a
                                                         decimal object and set as the default
                                                         value.  If the attribute type takes
                                                         a String, it will be set as the string.
                                                         Otherwise it will be ignored if not 
                                                         applicable for that attribute type.
           ......................................................................................
           Integar       DK_ATTR_TYPE_DEFAULT_INTEGER    If attribute type is numeric, the
                                                         default value will be converted to that
                                                         type and set as the default
                                                         value.  If the attribute type takes
                                                         a String, it will be set as the string.
                                                         Otherwise it will be ignored if not 
                                                         applicable for that attribute type.
           ......................................................................................
           Hex String    DK_ATTR_TYPE_DEFAULT_HEXSTRING  This will be considered a string
                                                         containint a hexidecimal numeric value.
                                                         If attribute type is numeric, the
                                                         default value will be converted to that
                                                         type and set as the default
                                                         value.  If the attribute type takes
                                                         a String, it will be set as the string.
                                                         Otherwise it will be ignored if not 
                                                         applicable for that attribute type.
           ......................................................................................
           Other         DK_ATTR_TYPE_DEFAULT_OTHER      If the attribute is not nullable 
                                                         (required), it will be converted to
                                                         the correct type depending on the
                                                         attribute type.  If the attribute is
                                                         nullable, the Default Value will be
                                                         ignored & not set.
           --------------------------------------------------------------------------------------
           Note: Constants defined in DKAttrDefICM.h
                    
Attribute Groups
    Groups of attributes can be created and used to easily add, retrieve,
    and access entire groups of attributes to items and sub-components.
    
    Please consult the SAttributeGroupDefCreationICM Sample for more information.

Reference Attribute (Group):
    Please consult the SReferenceAttrDefCreationICM Sample.

Definition Process
1.  Create a new Attribute Definition object.
2.  Describe it, setting its name, description, type, size, etc.
3.  Add the new definition to the persistent datastore.

---------------------------
Insurance Model Attributes:
---------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the attribute
      definition is used by and was create by the samples.  This helps avoid using
      attribute names that may already be in use.

NOTE: Since attribute definitions can be re-used under multiple item and
      component types, only the following list contains new attributes needed
      and which piece of the data model it was first needed in.  Multiple pieces
      of the Insurance Model re-use the same attributes.

    Attribute Name  Type          Part of Insurance Model it may be seen in
    --------------- ------------- -----------------------------------------
    S_firstName     Varchar(40)   Insurance Policy
    S_lastName      Varchar(40)   Insurance Policy
    S_street        Varchar(50)   Insurance Policy
    S_city          Varchar(20)   Insurance Policy
    S_state         Varchar(2)    Insurance Policy
    S_zipCode       Varchar(10)   Insurance Policy
    S_policyNumber  Varchar(11)   Insurance Policy
    S_vin           Varchar(20)   Insurance Policy
    S_reportNumber  Varchar(20)   Police Report
    S_accidentDate  Date          Police Report
    S_licensePlate  Varchar(10)   Police Report
    S_driverLicense Varchar(20)   Police Report
    S_claimNumber   Varchar(8)    Auto Claim
    S_claimantFName Varchar(40)   Auto Claim
    S_claimantLName Varchar(40)   Auto Claim
    S_claimantLicns VarChar(20)   Auto Claim
    S_insuredFName  Varchar(40)   Auto Claim
    S_insuredLName  Varchar(40)   Auto Claim
    S_adjusterFName Varchar(40)   Adjuster Report
    S_adjusterLName Varchar(40)   Adjuster Report
    S_adjustmntDate Date          Adjuster Report

---------------------------------------
Additional Attributes Defined in Sample
---------------------------------------
    S_varchar       Varchar(200)
    S_blob          Blob(100KB)
    S_char          Char(1)    
    S_clob          Clob(200KB)
    S_date          Date
    S_decimal       Decimal
    S_double        Double
    S_integer       Integer(-5000 to 7000)
    S_short         Short
    S_time          Time
    S_timestamp     Timestamp

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeDefinitionCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining & Explaining Attributes, All Attribute Types, Attribute
 *                    Properties, Insurance Model Attributes, +++
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Attribute Types
 *                    Creating a New Attribute Definition
 *                    Saving / Adding Attribute Definition in Datastore Definition
 *                    Defining Varchar Attributes
 *                    Defining Blob Attributes
 *                    Defining Char Attributes
 *                    Defining Clob Attributes
 *                    Defining Date Attributes
 *                    Defining Decimal Attributes
 *                    Defining Double Attributes
 *                    Defining Integer Attributes
 *                    Defining Short Attributes
 *                    Defining Time Attributes
 *                    Defining Timestamp Attributes
 *                    Attribute Definition Property: Size
 *                    Attribute Definition Property: Min Value
 *                    Attribute Definition Property: Max Value
 *                    Attribute Definition Property: Nullable
 *                    Attribute Definition Property: Unique
 *                    Attribute Definition Property: Text Searchable
 *                    Attribute Name Requirements
 *                    Defining attributes used by samples & Insurance Model
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeDefinitionCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes defined here must not exist.  If any have already been created, 
 *                    please run SAttributeDefinitionDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeDefinitionCreationICM.hpp
 *                    SAttributeDefinitionCreationICM_launch.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeDefinitionCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;
        DKAttrDefICM*  attrReturned = NULL;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SAttributeDefinitionCreationICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================"     << endl;
        cout << "IBM Information Integrator for Content      v" << ver << endl;
        cout << "Sample Program:  SAttributeDefinitionCreationICM"     << endl;
        cout << "------------------------------------------------"     << endl;
        cout << " Database: " << database                              << endl;
        cout << " UserName: " << userName                              << endl;
        cout << "================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Define a Varchar Attribute
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using attribute names that may already be in use.
            //
            // For more information on options that can be set for attribute
            // definitions, please consult the Programming Reference.
            //
            // Defining a String / Varchar with a maximum of 200 characters:
            cout << "Defining Varchar Attribute 'S_varchar'..." << endl;

                DKAttrDefICM* attr = new DKAttrDefICM(dsICM); // create a new attribute definition
                attr->setName("S_varchar");                   // Set a name, maximum 15 characters
                attr->setDescription("My Description");       // For more detail than name
                attr->setType(DK_CM_VARCHAR);                 // It will be a varchar attribute
                attr->setSize(200);                           // Set the maximum characters in string
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Varchar Attribute 'S_varchar'." << endl;

            //--------------------------------------------------------------
            // Define a Blob Attribute
            //--------------------------------------------------------------
            // Blob: Binary Large Object.
            // Defining a 100KB Blob Attribute:
            cout << "Defining Blob Attribute 'S_blob'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_blob");                      // Set a name, maximum 15 characters
                attr->setDescription("My Blob");              // For more detail than name
                attr->setType(DK_CM_BLOB);                    // Determines which type of attribute.
                attr->setSize(100000); //(B)                  // Determines the size the attribute, in Bytes (B).
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Blob Attribute 'S_blob'." << endl;

            //--------------------------------------------------------------
            // Define a Char Attribute
            //--------------------------------------------------------------
            // Defining a Char Attribute:
            cout << "Defining Char Attribute 'S_char'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_char");                      // Set a name, maximum 15 characters
                attr->setDescription("My Char");              // For more detail than name
                attr->setType(DK_CM_CHAR);                    // Determines which type of attribute.
                attr->setSize(1);                             // Number of Characters.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Char Attribute 'S_char'." << endl;

            //--------------------------------------------------------------
            // Define a Clob Attribute
            //--------------------------------------------------------------
            // Clob: Character Large Object
            // Defining a 200KB Clob Attribute:
            cout << "Defining Clob Attribute 'S_clob'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_clob");                      // Set a name, maximum 15 characters
                attr->setDescription("My Clob");              // For more detail than name
                attr->setType(DK_CM_CLOB);                    // Determines which type of attribute.
                attr->setSize(200000); //(B)                  // Determines the size the attribute, in Bytes (B).
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Clob Attribute 'S_clob'." << endl;

            //--------------------------------------------------------------
            // Define a Date Attribute
            //--------------------------------------------------------------
            // Defining a Date Attribute:
            cout << "Defining Date Attribute 'S_date'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_date");                      // Set a name, maximum 15 characters
                attr->setDescription("My Date");              // For more detail than name
                attr->setType(DK_CM_DATE);                    // Determines which type of attribute.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Date Attribute 'S_date'." << endl;

            //--------------------------------------------------------------
            // Define a Decimal Attribute
            //--------------------------------------------------------------
            // Defining a Decimal Attribute:  (Example Value: 123.45)
            cout << "Defining Decimal Attribute 'S_decimal'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_decimal");                   // Set a name, maximum 15 characters
                attr->setDescription("My Decimal");           // For more detail than name
                attr->setType(DK_CM_DECIMAL);                 // Determines which type of attribute.
                attr->setScale(3); // Scale                   // Set the scale for the Decimal Value.
                attr->setSize(5);  // Precision               // Set the Precision of the Decimal Value.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Decimal Attribute 'S_decimal'." << endl;

            //--------------------------------------------------------------
            // Define a Double Attribute
            //--------------------------------------------------------------
            // Defining a Double Attribute:
            cout << "Defining Double Attribute 'S_double'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_double");                    // Set a name, maximum 15 characters
                attr->setDescription("My Double");            // For more detail than name
                attr->setType(DK_CM_DOUBLE);                  // Determines which type of attribute.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Double Attribute 'S_double'." << endl;

            //--------------------------------------------------------------
            // Define a Integer Attribute
            //--------------------------------------------------------------
            // Defining a Integer Attribute the cannot be less than -5000 and 
            // not more than 7000:
            cout << "Defining Integer Attribute 'S_integer'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_integer");                   // Set a name, maximum 15 characters
                attr->setDescription("My Integer");           // For more detail than name
                attr->setType(DK_CM_INTEGER);                 // Determines which type of attribute.
                attr->setMin(-5000);                          // Set a minimum possible value.
                attr->setMax(7000);                           // Set a maximum possible value.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Integer Attribute 'S_integer'." << endl;

            //--------------------------------------------------------------
            // Define a Short Attribute
            //--------------------------------------------------------------
            // Defining a Short Attribute:
            cout << "Defining Short Attribute 'S_short'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_short");                     // Set a name, maximum 15 characters
                attr->setDescription("My Short");             // For more detail than name
                attr->setType(DK_CM_SHORT);                   // Determines which type of attribute.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Short Attribute 'S_short'." << endl;

            //--------------------------------------------------------------
            // Define a Time Attribute
            //--------------------------------------------------------------
            // Defining a Time Attribute:
            cout << "Defining Time Attribute 'S_time'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_time");                      // Set a name, maximum 15 characters
                attr->setDescription("My Time");              // For more detail than name
                attr->setType(DK_CM_TIME);                    // Determines which type of attribute.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Time Attribute 'S_time'." << endl;

            //--------------------------------------------------------------
            // Define a Timestamp Attribute
            //--------------------------------------------------------------
            // Defining a Timestamp Attribute:
            cout << "Defining Timestamp Attribute 'S_timestamp'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_timestamp");                 // Set a name, maximum 15 characters
                attr->setDescription("My Timestamp");         // For more detail than name
                attr->setType(DK_CM_TIMESTAMP);               // Determines which type of attribute.
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Timestamp Attribute 'S_timestamp'." << endl;


            // =============================================================
            // =============================================================
            // =============================================================
            // =============================================================


            //--------------------------------------------------------------
            // Defined Attributes for the Insurance Model
            //--------------------------------------------------------------

            // S_firstName: will be a Varchar containing 40 characters that it
            //              not nullable but does not require a unique value.
            cout << "Defining Attribute 'S_firstName'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_firstName");                 // Set a name, maximum 15 characters
                attr->setDescription("First Name");           // For more detail than name
                attr->setType(DK_CM_VARCHAR);                 // It will be a varchar attribute
                attr->setSize(40);                            // Set the maximum character size
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Attribute 'S_firstName'." << endl;

            // S_policyNumber: will be an Integer that must be greater than 0 that it
            //                 not nullable and requires a unique value.
            cout << "Defining Attribute 'S_policyNumber'..." << endl;

                attr = new DKAttrDefICM(dsICM);               // create a new attribute definition
                attr->setName("S_policyNumber");              // Set a name, maximum 15 characters
                attr->setDescription("Policy Number");        // For more detail than name
                attr->setType(DK_CM_VARCHAR);                 // It will be an Varchar attribute
                attr->setSize(11);                            // Set the maximum character size
                attr->add();                                  // add it to the persistent datastore
                delete(attr);                                 // Free up memory.  Delete object in Memory.

            cout << "Defined Attribute 'S_policyNumber'." << endl;

            // The rest of the attributes will be created in the exact same manner. 
            // The wrapper function provided in this sample will be used to define
            // the remaining.

            // S_lastName      Varchar(40)   Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_lastName", "Last Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_street        Varchar(50)   Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_street", "Street", DK_CM_VARCHAR, 50);
            if (attrReturned) delete attrReturned;
            // S_city          Varchar(20)   Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_city", "City", DK_CM_VARCHAR, 20);
            if (attrReturned) delete attrReturned;
            // S_state         Varchar(2)    Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_state", "State", DK_CM_VARCHAR, 2);
            if (attrReturned) delete attrReturned;
            // S_zipCode       Varchar(10)   Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_zipCode", "Zip Code", DK_CM_VARCHAR, 10);
            if (attrReturned) delete attrReturned;
            // S_vin           Varchar(20)   Insurance Policy
            attrReturned = defineAttribute(dsICM, "S_vin", "Vehicle Identification Number", DK_CM_VARCHAR, 20);
            if (attrReturned) delete attrReturned;
            // S_reportNumber  Varchar(20)   Police Report
            attrReturned = defineAttribute(dsICM, "S_reportNumber", "Police Report Number", DK_CM_VARCHAR, 20);
            if (attrReturned) delete attrReturned;
            // S_accidentDate  Date          Police Report
            attrReturned = defineAttribute(dsICM, "S_accidentDate", "Accident Date", DK_CM_DATE, 0);
            if (attrReturned) delete attrReturned;
            // S_licensePlate  Varchar(10)   Police Report
            attrReturned = defineAttribute(dsICM, "S_licensePlate", "Lisence Plate", DK_CM_VARCHAR, 10);
            if (attrReturned) delete attrReturned;
            // S_driverLicense Varchar(20)   Police Report
            attrReturned = defineAttribute(dsICM, "S_driverLicense", "Drivers License", DK_CM_VARCHAR, 20);
            if (attrReturned) delete attrReturned;
            // S_claimNumber   Varchar(8)    Auto Claim
            attrReturned = defineAttribute(dsICM, "S_claimNumber", "Claim Number", DK_CM_VARCHAR, 8);
            if (attrReturned) delete attrReturned;
            // S_claimantFName Varchar(40)   Auto Claim
            attrReturned = defineAttribute(dsICM, "S_claimantFName", "Claimant First Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_claimantLName Varchar(40)   Auto Claim
            attrReturned = defineAttribute(dsICM, "S_claimantLName", "Claimant Last Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_claimantLicns  VarChar(20)   Auto Claim
            attrReturned = defineAttribute(dsICM, "S_claimantLicns", "Claimant Drivers Liscense", DK_CM_VARCHAR, 20);
            if (attrReturned) delete attrReturned;
            // S_insuredFName  Varchar(40)   Auto Claim
            attrReturned = defineAttribute(dsICM, "S_insuredFName", "Insured First Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_insuredLName  Varchar(40)   Auto Claim
            attrReturned = defineAttribute(dsICM, "S_insuredLName", "Insured Last Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_adjusterFName Varchar(40)   Adjuster Report
            attrReturned = defineAttribute(dsICM, "S_adjusterFName", "Adjuster First Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_adjusterLName Varchar(40)   Adjuster Report
            attrReturned = defineAttribute(dsICM, "S_adjusterLName", "Adjuster Last Name", DK_CM_VARCHAR, 40);
            if (attrReturned) delete attrReturned;
            // S_adjustmntDate Date          Adjuster Report
            attrReturned = defineAttribute(dsICM, "S_adjustmntDate", "Adjustment Date", DK_CM_DATE, 0);
            if (attrReturned) delete attrReturned;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Define an attribute, setting options for the specified basic criteria.
    * For more information on options that can be set for attribute definitions,
    * please consult the Programming Reference.  Do not forget to delete the 
    * returned object when it is no longer needed.
    * @param dsICM        Connect DKDatastoreICM object to define the attribute in.
    * @param name         Name of the new attribute.
    * @param description  More detailed description than name.
    * @param type         Type of attribute.  See documentaiton at top of sample for types.
    * @param size         In this sample, it will only be applicable to char, varchar, blob, & clob attributes.  Determines the maximum characters allowed.
    * @return Returns a reference to the newly defined DKAttrDefICM that was added to the datastore.
    **/
    DKAttrDefICM* SAttributeDefinitionCreationICM::defineAttribute(DKDatastoreICM * dsICM, char * name, char * description, short type, int size){
        cout << "Defining Attribute '" << name << "'..." << endl;

            DKAttrDefICM* attr = new DKAttrDefICM(dsICM); // create a new attribute definition
            attr->setName(name);                          // Set a name, maximum 15 characters
            attr->setDescription(description);            // For more detail than name
            attr->setType(type);                          // Determines which type of attribute.

            if(    (type==DK_CM_VARCHAR) || (type==DK_CM_CHAR) 
                || (type==DK_CM_BLOB)    || (type==DK_CM_CLOB) 
              ) // in this sample, only applicable to char, varchar, blob, & clob
                attr->setSize(size);                      // Set the maximum character size (if applicable)

            attr->add();                                  // add it to the persistent datastore

        cout << "Defined Attribute '" << name << "'." << endl;
        return(attr);
    }
