/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Document Routing
    Items may be routed through a process, advance from node to node, selecting any 
    defined path between nodes. 
    
    Items
        Any Document or Folder may be routed between nodes.  Folders can be especially
        useful because they may may contain any number of Items of any type which may
        be added or removed as the process progresses.  Folders may wait at a Collection
        Point for a set number of different types of documents to be filed.  
        
        -> Please refer to SItemCreationICM & SFolderICM Samples for information on Items
           and the Semantic Types available.

    Work Node
        A point in a proces where Items wait for actions to be taken by end users or by
        the User-Application.  There are two types of Work Nodes, Regular & Collection
        Point.
        
        Regular Work Node
            A regular work node may be exited by the User-Application at any time.  The
            User-Application is responsible for validating its own exit criteria.
            
        Collection Point
            A Collection Point is a Work Node with system-defined exit criteria, 
            specifically applicable to routing Folders.  A set of Requirements may
            be specified that must be met before the process can resume / advance
            past this point.  The process will automatically move onto the next
            node after the requirements are met.  If the requirements are already
            met when an item reaches the Collection Point node, it will automatically
            jump to the next node.  There may be only one route exiting a collection
            point, which will be used by the system to automatically advance the
            process.
            
            Resume Requirements
                These Requirements define the number of Items of a particular Item Type
                that must be added to the Folder before the process can resume / advance.
                For example, a process may wait for all documents (Insurance Claim, Policy,
                Police Report, etc) to be submitted before advancing to final approval.
                
                Individual Requirements will be added to a list of requirements.  Each
                individual Resume Requirement is called a "Resume List Entry" that will
                belong to the Resume List.  Each Resume List Entry is managed simply as
                a DKCollectionResumeListEntry.
                
            Resume List
                The collection of Requirements is called a "Resume List" since they are
                the criteria required to resume.  The Resume List is managed simply as
                a DKSequentialCollection containing the individual Requirements called
                Resume List Entries.

        Work Node Types   Constant  Description
        ----------------  --------  -------------------------------------------------------
            Regular          0      Regular Work Node.
        ...................................................................................
        Collection Point     1      A process cannot proceed from a collection point to the
                                    next node in the process route until all the documents
                                    defined by the collection point definition arrive.
        -----------------------------------------------------------------------------------

    Routing Process
        A Routing Process is the defined routes that a package may follow.  Multiple
        Routing Processes may re-use the same nodes and multiple routes between nodes
        may be used.
        
        Route
            A route simply connects two nodes with an implied direction.  Each route
            is given a "Selection Name" which is used to select this route when exiting
            a node / advancing the process.  Selection Names are user-defined.  There
            are no system-defined names that must be used.  For example, "Continue" may
            be used in Samples which is coincidently similar to the continueProcess 
            function name.  "Continue" in this case is a user-defined name of the route
            which will be used to select this route when exiting the node.  Routes are
            managed as DKRouteListEntry objects.
            
            For example, a Work Node for an insurance claim approval may have two routes
            leaving it, "Accept" and "Reject" which may lead to different nodes.

            Collection Points
                Only one route may exit a Collection Point.  The system will use this
                route to automatically advance once the requirements are met.

        Route List
            Individual Routes are managed as a "Route List", which is simply a 
            DKSequentialCollection containing the DKRouteListEntry objects.

        Example Routing Process
            This example will create a process according to the following diagram:
    
        ---------   auto  ---------------               ----------------
        | Start |  -----> | Fill/Submit |  "Continue"   | Investigate/ |   "Continue"
        ---------         |    Claim    |  ---------->  | Gather Docs  |  --------
                          ---------------               ----------------         |
                                                                                 |
                                ---------   "Complete"  ----------------         |
                                |  End  |  <----------  |  Adjustment  |  <-------
                                ---------               |    Report    | 
                                                        ----------------
                                                    
            This sample deminstrates a simple route, using one route between any node.
            Any number of routes may be used, named anything desired.  "Reject" or 
            "Cancel" are two typical route names that may be used with an additional 
            node named "Rejected".

            For Example, it could be expanded upon as the following diagram demonstrates:

        ---------   auto  ---------------               ----------------
        | Start |  -----> | Fill/Submit |  "Continue"   | Investigate/ |   "Continue"
        ---------         |    Claim    |  ---------->  | Gather Docs  |  --------
                          ---------------               ----------------         |
                                |   \                      |                     |
                                |    \                     |"Reject"             |
                                |     \                    |                     |
           -------------        |      \                   |--->  ------------   |
           | Cancelled | <------|       \ "Fast Track"            | Rejected |   |
           -------------  "Cancel"       \----------->\    |--->  ------------   |
                                                       \   |                     |
                                                        \  |"Reject"             |
                                                         \ |                     |
                                ---------   "Complete"  ----------------         |
                                |  End  |  <----------  |  Adjustment  |  <-------
                                ---------               |    Report    | 
                                                        ----------------
    Work Package
        When an instance of a Routing Process is in progress, a Work Package is the vehicle
        through with an Item moves along through the Routing Process.  A Work Package 
        contains all necessary information about the Process and the Item that it is
        transporting.  The Work Package is the object that the User-Application will use
        and manipulate.
        
        Priority
            One of the properties of a Work Package is its priority.  Any number may
            be specified and used.  Using & enforcing priority is the responsibility
            of the User-Program to allow maximum flexability.

    Work List
        One or more Work Nodes from which a user obtains a list of Work Packages or the
        "next" Work Package.  A Work Node may be in more than one Work List.  Using a
        Work List allows an administrator or a User-Application to dynamically change
        work assignments without contacting end users.

        Work List Property   Value*   Description
        ------------------  --------  ------------------------------------------------------
        SelectionOrder      priority  Work packages returned are ordered by priority.
                            date      Work packages returned are ordered by date.
        ....................................................................................
        SelectionFilterOn   yes       Work packages returned are those being suspended.
        Suspend             no        Work packages returned are those not being suspended.
                            either    Work packages returned include either.
        ....................................................................................
        SelectionFilterOn   yes       Work packages returned are those users being notified.
    	Notify		        no        Work packages returned are those not being notified.
    				        either    Work packages returned include both
        ....................................................................................
        SelectionFilterOn   yes       Work packages returned are those belonging to the user
        Owner               no        Work packages returned are those not belonging to the
                                      user.
        ....................................................................................
        MaxResult           all       Return all work packages
                            >0        Return specified number of work packages
        ------------------------------------------------------------------------------------
        *NOTE: Value 'priority' is specified using DK_ICM_DR_SELECTION_ORDER_PRIORITY  (DKConstantICM.h)
               Value 'date'     is specified using DK_ICM_DR_SELECTION_ORDER_TIME
               Value 'yes'      is specified using DK_ICM_DR_SELECTION_FILTER_YES
               Value 'no'       is specified using DK_ICM_DR_SELECTION_FILTER_NO
               Value 'either'   is specified using DK_ICM_DR_SELECTION_FILTER_EITHER
               Value 'all'      is specified using DK_ICM_DR_MAX_RESULT_ALL
       

----------------------------------------
Routing Processes Defined in This Sample
----------------------------------------

     Name             Description
     ---------------  --------------------------------------
     S_claimProcess   Process for an Insurance Claim.

---------------------------------
Work Nodes Defined in This Sample
---------------------------------

     Name             Type     Description
     ---------------  -------  --------------------------------------
     S_fillClaim      Regular  Claimant Fills Out Claim
     S_gatherAll      Coll Pt  Gather Claim, Police Report, Policy, & Photos
     S_adjustment     Regular  Adjuster Fills Out Adjustment Report

---------------------------------
Work Lists Defined in This Sample
---------------------------------

     Name             Nodes Covered  Description
     ---------------  -------------  -------------------------
     S_fillClaimWL    Work List for Fill/Submit Claim Work Node.
     S_agentWL        Work List Covering Gathering Documents & Creating the Adjustment Report.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKConstant.h>
#include <DKDocRoutingServiceMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SDocRoutingDefinitionCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SDocRoutingDefinitionCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Definition & Explanation of Document Routing, Routing Processes, Routing
 *                    Folders, Work Nodes, Collection Points, Resume Requirements, Resume List
 *                    Routes, Route Lists, Work Packages, Work Package Priority, & Accessing
 *                    the Routing Management / Definition Object.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Document Routing
 *                    Routing Folders
 *                    Work Nodes
 *                    Collection Points
 *                    Resume Requirements
 *                    Resume List
 *                    Routing Processes
 *                    Routes
 *                    Route Lists
 *                    Work Packages
 *                    Work Package Priority
 *                    Routing Management / Definition Object
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SDocRoutingDefinitionCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SDocRoutingDefinitionCreationICM_launch.cpp
 *                    SDocRoutingDefinitionCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SDocRoutingDefinitionCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                               << endl;
            cout << "  SDocRoutingDefinitionCreationICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."                << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================="     << endl;
        cout << "IBM Information Integrator for Content       v" << ver << endl;
        cout << "Sample Program:  SDocRoutingDefinitionCreationICM"     << endl;
        cout << "-------------------------------------------------"     << endl;
        cout << " Database: " << database                               << endl;
        cout << " UserName: " << userName                               << endl;
        cout << "=================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //---------------------------------------------------------------------
            // Routing Management / Definition Object
            //---------------------------------------------------------------------
            // DKDocRoutingServiceMgmtICM:
            //                          Provides methods to manage DKProcessICM, 
            //                          DKWorkNodeICM, and DKWorkListICM.
            cout << "Creating Routing Service & Management Objects..." << endl;

                DKDocRoutingServiceMgmtICM* routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM); // Obtain the routing management object.

            cout << "Created Routing Service & Management Objects." << endl;

            //------------------------------------------------------------------------
            // Define Work Node:  Claimant Fills Out Claim
            //------------------------------------------------------------------------
            // A work node may be either a regular work node or a collection point.
            //
            // This sample will create 2 work nodes and one collection point to the
            // process.
            //
            // First, a work node will be defined where the claimant fills out a claim.
            cout << "Defining Work Node: Claimant Fills Out Claim..." << endl;

                DKWorkNodeICM* workNode1 = new DKWorkNodeICM();  // Create new Work Node Object.
                workNode1->setName("S_fillClaim");               // Choose a Name with 15 characters or less.
                workNode1->setDescription("Claimant Fills Out Claim"); // Choose a Description for more information than the name.
                workNode1->setTimeLimit(100);                    // Sets the value of the maximum time that an item can spend at this work node (in minutes).
                workNode1->setOverloadLimit(200);                // Specify max number of work packages that can be at this node
                workNode1->setType(0);                           // Set the type is to be a regular work node
                routingMgmt->add(workNode1);                     // Add the Work Node to Routing Management.

                delete(workNode1);                               // Free Memory.  This object will no longer be needed.
                
            cout << "Defined Work Node: Claimant Fills Out Claim." << endl;

            //------------------------------------------------------------------------
            // Define Collection Point:  Gather Claim, Police Report, Policy, & Photos
            //------------------------------------------------------------------------
            // A process cannot proceed from a collection point to the next node in 
            // the process route until all the documents defined by the collection
            // point definition arrive.
            //
            // In the Insurance Model example, this will be the gathering point for
            // all relevant documentation to be gathered.  An agent will research
            // and gather the claim, police report, policy, & photos before it is
            // moved onto the final adjustment phase.
            cout << "Defining Collection Point: Gather Claim, Police Report, Policy, & Photos..." << endl;

                DKWorkNodeICM* workNode2 = new DKWorkNodeICM();  // Create a new Work Node Object
                workNode2->setName("S_gatherAll");               // Choose a Name with 15 characters or less.
                workNode2->setDescription("Gather Claim, Police Report, Policy, & Photos"); // Choose a Description for more information than the name.
                workNode2->setTimeLimit(100);                    // Sets the value of the maximum time that an item can spend at this work node (in minutes).
                workNode2->setOverloadLimit(200);                // Specify max number of work packages that can be at this node.
                workNode2->setType(1);                           // Set the type of node to be a collection point.

                // Create the "Resume" List, which is the list of document types that it must
                // wait for before moving on to the next node.  A list will be created to
                // hold "resume entries" which are descriptions of requirements that must be
                // met before the process may move on.
                dkCollection* resumeList = new DKSequentialCollection();   // Create a List / Collection to hold all Resume Entries.
	            
                // Create as many requirements, or "Resume List Entries", which specify
                // what Item Types it must wait for.  The process cannot pass this collection
                // point unless the specified number of Item (DDO) of the specified Item Type
                // reaches this collection point.
                
                // Require that a Claim be submitted.
                DKCollectionResumeListEntryICM* resumeRequirement = new DKCollectionResumeListEntryICM();  // Create a requirement / Resume List Entry.
                resumeRequirement->setFolderItemTypeName("S_simple");             // Set the Item Type Name Folder Item that is being routed.
                resumeRequirement->setRequiredItemTypeName("S_autoClaim");        // Make the collection wait for an Item of the specified Item Type to be added to the folder before proceeding.
                resumeRequirement->setQuantityNeeded(1);                          // Specify the number of Items of the specified Item Type that it must wait for.
                resumeList->addElement(resumeRequirement);                        // Add the requirement (Entry) to the List of Requirments (Resume Lise).

                // Require that a Police Report be obtained.  The insurance company may perform this search.
                resumeRequirement = new DKCollectionResumeListEntryICM();         // Create a requirement / Resume List Entry.
                resumeRequirement->setFolderItemTypeName("S_simple");             // Set the Item Type Name Folder Item that is being routed.
                resumeRequirement->setRequiredItemTypeName("S_policeReport");     // Make the collection wait for an Item of the specified Item Type to be added to the folder before proceeding.
                resumeRequirement->setQuantityNeeded(1);                          // Specify the number of Items of the specified Item Type that it must wait for.
                resumeList->addElement(resumeRequirement);                        // Add the requirement (Entry) to the List of Requirments (Resume Lise).

                // Require that a Policy be located.  The insurance company would perform this search.
                resumeRequirement = new DKCollectionResumeListEntryICM();         // Create a requirement / Resume List Entry.
                resumeRequirement->setFolderItemTypeName("S_simple");             // Set the Item Type Name Folder Item that is being routed.
                resumeRequirement->setRequiredItemTypeName("S_insurePolicy");     // Make the collection wait for an Item of the specified Item Type to be added to the folder before proceeding.
                resumeRequirement->setQuantityNeeded(1);                          // Specify the number of Items of the specified Item Type that it must wait for.
                resumeList->addElement(resumeRequirement);                        // Add the requirement (Entry) to the List of Requirments (Resume Lise).

                // Require that a Photo submitted.  The insurance company may obtain this during a preliminary assessment or investigation.
                resumeRequirement = new DKCollectionResumeListEntryICM();         // Create a requirement / Resume List Entry.
                resumeRequirement->setFolderItemTypeName("S_simple");             // Set the Item Type Name Folder Item that is being routed.
                resumeRequirement->setRequiredItemTypeName("S_autoPhoto");        // Make the collection wait for an Item of the specified Item Type to be added to the folder before proceeding.
                resumeRequirement->setQuantityNeeded(1);                          // Specify the number of Items of the specified Item Type that it must wait for.
                resumeList->addElement(resumeRequirement);                        // Add the requirement (Entry) to the List of Requirments (Resume Lise).

                // When all requirements (resume list entries) have been added to the list of 
                // requirements (resume list), set the resume list in the collection point.
                workNode2->setCollectionResumeList(resumeList);
	
                // Add the Collection Point to the Routing Managment.
                routingMgmt->add(workNode2);  

                // Destroy Objects No Longer Needed in Memory
                delete(workNode2);                                                // Free Memory.

            cout << "Defined a Collection Point." << endl;

            //------------------------------------------------------------------------
            // Define Work Node: Adjuster Fills out Adjuster Report
            //------------------------------------------------------------------------
            // A work node will be defined where the adjuster fills out the adjustment
            // report.
            cout << "Defining Work Node: Adjuster Fills out Adjustment Report..." << endl;

                DKWorkNodeICM* workNode3 = new DKWorkNodeICM();  // Create new Work Node Object.
                workNode3->setName("S_adjustment");              // Choose a Name with 15 characters or less.
                workNode3->setDescription("Adjuster Fills Out Adjustment Report"); // Choose a Description for more information than the name.
                workNode3->setTimeLimit(100);                    // Sets the value of the maximum time that an item can spend at this work node (in minutes).
                workNode3->setOverloadLimit(200);                // Specify max number of work packages that can be at this node
                workNode3->setType(0);                           // Set the type is to be a regular work node
                routingMgmt->add(workNode3);                     // Add the Work Node to Routing Management.

                delete(workNode3);                               // Free Memory.

            cout << "Defined Work Node: Adjuster Fills out Adjustment Report." << endl;

            //------------------------------------------------------------------------
            // Define Process & Route
            //------------------------------------------------------------------------
            // Define the Process & Route that the process will follow to completion.
            // 
            // A Route List Entry simply connects two nodes with an implied direction.
            // Multiple routes may exist between nodes.  A specific route may be selected
            // by a user-defined "selection" keyword.  Examples might be "Continue", "Go",
            // "Accept", "Reject", "Complete", etc.
            //
            // This specific example will only use one route between nodes.  This example
            // could be expanded upon by adding "Cancel" or "Reject" routes.
            cout << "Defining Process & Route..." << endl;

                DKProcessICM* process = new DKProcessICM();                // Create a new Process Definition
                process->setName("S_claimProcess");                        // Choose a name with 15 characters or less.
                process->setDescription("Process for an Insurance Claim"); // More description than the 15 character name.

                // Define all possible Routes.
                dkCollection* routes = new DKSequentialCollection();       // Create a list of all possible routes between nodes.

                // Connect the Work Nodes using Route List Entries.  A simple route between
                // two work nodes is specified by associating a 'From' work node and a 'To'
                // work node.

                // Route: Start -> Fill Claim
                DKRouteListEntryICM* nodeRoute = new DKRouteListEntryICM();  // Create a new connection between two nodes.
                nodeRoute->setFrom(DK_ICM_DR_START_NODE);                    // Every process must start with the start node.
                nodeRoute->setTo("S_fillClaim");                             // Start off by going to the Claimant filling or submitting the Claim Form.
                nodeRoute->setSelection("Continue");                         // Choose any user-defined name for an action that will make the transition take place.
                routes->addElement(nodeRoute);                               // Add the individual route to the collection of all possible routes.

                // Route: Fill Claim -> Gather All
                nodeRoute = new DKRouteListEntryICM();                       // Create a new connection between two nodes.
                nodeRoute->setFrom("S_fillClaim");                           // Connect the fill/submit claim node
                nodeRoute->setTo("S_gatherAll");                             // to the gather all documentation node.
                nodeRoute->setSelection("Continue");                         // Choose any user-defined name for an action that will make the transition take place.
                routes->addElement(nodeRoute);                               // Add the individual route to the collection of all possible routes.
                
                // Route: Gather All -> Adjustment
                nodeRoute = new DKRouteListEntryICM();                       // Create a new connection between two nodes.
                nodeRoute->setFrom("S_gatherAll");                           // Connect the gather all documentation node
                nodeRoute->setTo("S_adjustment");                            // to the adjustment node.
                nodeRoute->setSelection("Continue");                         // Choose any user-defined name for an action that will make the transition take place.
                routes->addElement(nodeRoute);                               // Add the individual route to the collection of all possible routes.

                // Route: Adjustment -> End
                nodeRoute = new DKRouteListEntryICM();                       // Create a new connection between two nodes.
                nodeRoute->setFrom("S_adjustment");                          // Connect the adjustement node
                nodeRoute->setTo(DK_ICM_DR_END_NODE);                        // to the end of the process.
                nodeRoute->setSelection("Complete");                         // Choose any user-defined name for an action that will make the transition take place.
                routes->addElement(nodeRoute);                               // Add the individual route to the collection of all possible routes.

                // Set the route in the process.
                process->setRoute(routes);
        
                // Add the process to the routing Management.
                routingMgmt->add(process);

                delete(process);                                                // Free Memory.

            cout << "Defined Process & Route." << endl;

            //------------------------------------------------------------------------
            // Define a Work List: fill Claim Work List
            //------------------------------------------------------------------------
            // Now worklists will be created for the work nodes created earlier.
            // There will be one work list for every work node and collection point.
            // A work list may cover any number of Work Nodes.  In this sample, 
            // an individual Work List will cover a single node.
            cout << "Defining Work List for S_fillClaim Work Node..." << endl;

                DKWorkListICM* workList = new DKWorkListICM();  // Create a new work list.
                workList->setName("S_fillClaimWL");             // Choose a name of 15 characters or less.
                workList->setDescription("Work List Covering Fill/Submit Claim Work Node.");  // Choose a description.
                workList->setSelectionOrder(DK_ICM_DR_SELECTION_ORDER_TIME);                  // See Header Documentaiton for Options
                workList->setSelectionFilterOnSuspend(DK_ICM_DR_SELECTION_FILTER_NO);         // See Header Documentation for Options
                workList->setSelectionFilterOnNotify(DK_ICM_DR_SELECTION_FILTER_NO);          // See Header Documentation for Options  
                workList->setMaxResult(100);                    // Maximum of 100 Listed.
                DKString* wnNames = new DKString[1];
                wnNames[0] = "S_fillClaim";                     // An number of work nodes may be coved.
                workList->setWorkNodeNames(wnNames,1);          // Specify the work node names that this work list applies to.
                routingMgmt->add(workList);                     // Add to Routing Management

                delete(workList);                               // Free Memory

            cout << "Defined Work List for S_fillClaim Work Node." << endl;

            //------------------------------------------------------------------------
            // Define a Work List Covering Multiple Nodes: Insurance Company's Work
            //------------------------------------------------------------------------
            // For example, all work that must be done by the Insurance company may
            // be grouped in one Work List, covering all nodes involving them.  For
            // example, the following will define a Work List covering the Gather
            // All Documents work node and the adjustment work node.
            cout << "Defining Work List Covering Multiple Nodes..." << endl;

                workList = new DKWorkListICM();                 // Create a new work list.
                workList->setName("S_agentWL");                 // Choose a name of 15 characters or less.
                workList->setDescription("Work List Covering Gathering Documents & Creating the Adjustment Report.");
                workList->setSelectionOrder(DK_ICM_DR_SELECTION_ORDER_TIME);          // See Header Documentaiton for Options
                workList->setSelectionFilterOnSuspend(DK_ICM_DR_SELECTION_FILTER_NO); // See Header Documentation for Options
                workList->setSelectionFilterOnNotify(DK_ICM_DR_SELECTION_FILTER_NO);  // See Header Documentation for Options  
                workList->setMaxResult(100);                    // Maximum of 100 Listed.
                DKString* wnNames2 = new DKString[2];
                wnNames2[0] = "S_gatherAll";
                wnNames2[1] = "S_adjustment";
                workList->setWorkNodeNames(wnNames2,2);         // Specify the work node names that this work list applies to.
                routingMgmt->add(workList);                     // Add to Routing Management

                delete(workList);                               // Free Memory
                delete(routingMgmt);

            cout << "Defined Work List Covering Multiple Work Nodes." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
