/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Creating Blank DDOs for Existing Items
    All DDOs must be created through the DKDatastoreICM.createDDO() functions.  If the 
    Item has already been added to the datastore, the DDO can be re-created using its
    PID.  The PID is a Persistent Identification that uniquely identifies the particular
    component / row in the database.  The full pid string must be used in the 
    DKDatastoreICM::createDDO(pidString) function.
    
    It is technically possible to create a retrievable DDO by using the DKDDO constructor
    and manually setting the PID information, but it is easiest to use the Datastore's
    createDDO function.
    
    Query/Search is another way of obtaining DDOs that will be created for you.

Retrieving Items
    Items are retrieved through the DKDDO.retrieve(options) function.  Retrieval
    options may be specified and are recommended to ensure that only what is 
    desired is being retrieved.  This will save performance, avoiding extra
    calls to the library server for children or links if the user program knows
    that there are none in the data model or that they will not be used at this
    time.  Retrieval Options are binary, so they may be added together to set more
    than one option bit.

    Retrieval Option      Constant                     Explanation
    --------------------  ---------------------------  ------------------------------------------
    Pid Information Only  DK_CM_CONTENT_IDONLY         Useful in Query/Search.  Query Will return
                                                       collection with empty DDOs with complete
                                                       PID information.
    .............................................................................................
    Attributes Only       DK_CM_CONTENT_ATTRONLY       Retrieves only Attributes
    .............................................................................................
    Children              DK_CM_CONTENT_CHILDREN       Retrieve the Child Components. 
    .............................................................................................
    One Level             DK_CM_CONTENT_ONELEVEL       Retrieve One-Level of Information, 
                                                       Chilren & Attributes
    .............................................................................................
    Outbound Links        DK_CM_CONTENT_LINKS_OUTBOUND Retrieve Outbound Link information.
    .............................................................................................
    Inbound Links         DK_CM_CONTENT_LINKS_INBOUND  Retrieve Inbound Link information.
    .............................................................................................
    Item Tree             DK_CM_CONTENT_ITEMTREE       Retrieve the entire tree of information,
                                                       including all meta data / attributes, 
                                                       children & sub-children, links, etc.
    .............................................................................................
    Item Tree Without     DK_CM_CONTENT_ITEMTREE...    Retrieve the entire tree of information,
    Links                               ..._NO_LINKS   but save time and increase performance by 
                                                       not checking for links.
    .............................................................................................
    No Resource Content   DK_CM_CONTENT_NO             If it is a Resource Item, do not retrieve
                                                       the Resource object content.
    .............................................................................................
    Resource Content      DK_CM_CONTENT_YES            If it is a Resource Item, retrieve the
                                                       Resource object content.
    .............................................................................................
    Only Resource Content DK_CM_CONTENT_ONLY           If it is a Resource Item, retrieve only 
                                                       the Resource object content.
    .............................................................................................
    Check Out Item        DK_CM_CHECKOUT               In addition, check out / lock the item.
    .............................................................................................
    Latest Version        DK_CM_VERSION_LATEST         If versioning is enabled, retrieve the 
                                                       latest version, despite what the specific
                                                       object the Pid information specifies.
    ---------------------------------------------------------------------------------------------
    Note: Constants shown above are defined in DKConstant.h
  

Retrieving Child Components
    It is possible to retrieve an individual child component, given its PID string, 
    However, Items cannot be updated by retrieving the component and calling the
    DKDDO.update() method.  All Updates should be handled through the Root Item.
    The child component may be modified within the retrieved structure of the 
    Root Item, and then the DKDDO.update() method should be called on the Root Component.

Reading Attributes / Data Items Programmatically
    The user program does not need to know the type of data that each attribute is
    when retrieving it.  All data is accessed through DKAny objects, which contain 
    a type code that describes the content.  Additionally, it can be determined through
    the attribute definition (see SAttributeDefinitionCreation/RetrievalICM),
    or by the data property type.  Valid data property types are listed below.

    Note: The following applies to all connectors.  A subset is used
          by the IBM Content Management Connector (ICM).                             >> More >>
                                                                                                   Data Contained 
    Data Type         Constant                            Object              DKAny Type Code      May Be Cast To
    ----------------  ----------------------------------  ------------------  -------------------  --------------
    Undefined         DK_CM_DATAITEM_TYPE_UNDEFINED       DKAny or Other      -                    -
    String (Varchar)  DK_CM_DATAITEM_TYPE_STRING          DKAny(DKString)     tc_string            DKString
    Short             DK_CM_DATAITEM_TYPE_SHORT           DKAny(short)        tc_short             short
    Long              DK_CM_DATAITEM_TYPE_LONG            DKAny(long)         tc_long              long
    Float             DK_CM_DATAITEM_TYPE_FLOAT           DKAny(float)        tc_float             float
    Decimal           DK_CM_DATAITEM_TYPE_DECIMAL         DKAny(DKDecimal)    tc_decimal           DKDecimal
    Date              DK_CM_DATAITEM_TYPE_DATE            DKAny(DKDate)       tc_date              DKDate
    Time              DK_CM_DATAITEM_TYPE_TIME            DKAny(DKTime)       tc_time              DKTime
    Timestamp         DK_CM_DATAITEM_TYPE_TIMESTAMP       DKAny(DKTimestamp)  tc_timestamp         DKTimestamp
    Double            DK_CM_DATAITEM_TYPE_DOUBLE          DKAny(double)       tc_double            doublt
    Byte Array        DK_CM_DATAITEM_TYPE_BYTEARRAY       DKAny(DKByteArray)  tc_bytearray         DKByteArray
    DDO               DK_CM_DATAITEM_TYPE_DDOOBJECT       DKAny(dkDataObjectBase) tc_dobase        DKDDO or Sub-Class
    XDO               DK_CM_DATAITEM_TYPE_XDOOBJECT       DKAny(dkDataObjectBase) tc_dobase        DKLobICM, dkXDO, or Sub-Class
    Data Object Base  DK_CM_DATAITEM_TYPE_DATAOBJECTBASE  DKAny(dkDataObjectBase) tc_dobase        dkAbstractDataObjectBase
    Collection        DK_CM_DATAITEM_TYPE_COLLECTION      DKAny(dkCollection) tc_collection        DKSequentialCollection or dkCollection
    DDO Collection    DK_CM_DATAITEM_TYPE_COLLECTION_DDO  DKAny(dkCollection) tc_collection        DKChildCollection, DKFolder, DKSequentialCollection, dkCollection, or other.
    XDO Collection    DK_CM_DATAITEM_TYPE_COLLECTION_XDO  DKAny(dkCollection) tc_collection        DKParts, DKSequentialCollection, dkCollection, or other.
    Link Collection   DK_CM_DATAITEM_TYPE_LINKCOLLECTION  DKAny(dkCollection) tc_collection        DKLinkCollection, DKSequentialCollection, dkCollection
    Array             DK_CM_DATAITEM_TYPE_ARRAY           DKAny               -                    - 
    -------------------------------------------------------------------------------------------------------------
    ** Constants shown above are defined in DKConstant.h                             >> More >>

Detecting Nonexistant Data Items
    When obtaining the dataid of any attribute, property, etc. of a DDO, if '0'
    is returned, it indicates that the data item is nonexistant in this ddo object.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKChildCollection.hpp>
#include <DKLinkCollection.hpp>
#include <DKParts.hpp>
#include <DKFolder.hpp>
#include <DKLink.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKComponentTypeDefICM.hpp>
#include <dkResultSetCursor.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemCreationICM.hpp"
#include "SItemRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Items, PID Information, Retrieving & Accessing Child Components
 *                    & Multivalue Attributes, Retrieval Options, Accessing Attributes
 *                    Explicitly & Programmatically, Detecting Nonexistant Attributes & 
 *                    Properties in DDO, Printing DDOs.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: PID Information
 *                    Retrieving Items
 *                    Retrieving Child Components
 *                    Accessing Child Components / Multivalue Attributes
 *                    Retrieval Options
 *                    Accessing Attributes
 *                    Accessing Attributes Programmatically Using the Component Definition
 *                    Accessing All DDO Data Items Programmatically
 *                    Detecting Nonexistant Attributes / Properties in DDO
 *                    Printing DDOs
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SItemRetrievalICM_launch.cpp
 *                    SItemRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        const char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        const char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        const char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemRetrievalICM <database> <userName> <password>"     << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        const char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemRetrievalICM"              << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of Item Property type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            cout << "Creating & Saving Sample Items..." << endl;

                DKDDO* ddoDocument = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_DOCUMENT);
                DKDDO* ddoFolder   = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_FOLDER);   
                DKDDO* ddoItem     = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_ITEM);     

                // Set the reference attribute for ddoItem before it is added, and after the others were added.
                dsICM->checkOut(ddoItem);          // Covered in SItemUpdateICM
                DKDDO* ddoFolderCopy = dsICM->createDDO(((DKPidICM*)ddoFolder->getPidObject())->pidString());
                ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_myRef"),ddoFolderCopy); // Using copy of DKDDO named 'ddoFolder' made persistent above.
                ddoItem->update(DK_CM_CHECKIN);    // Covered in SItemUpdateICM

            cout << "Created & Saved Sample Items." << endl;

            //-------------------------------------------------------------
            // Obtain the Pid Strings so that the DDOs may be recreated.
            //-------------------------------------------------------------
            // First, obtain the DKPidICM object using the getPidObject() operation.
            // If the DDO was created by DKDatastoreICM, it may be cast from dkPid to DKPidICM.
            // Next, call the pidString() function for a String representation.
            cout << "Obtaining PID Information..." << endl;

                DKString ddoDocumentPidString = ((DKPidICM*)(ddoDocument->getPidObject()))->pidString();
                DKString ddoFolderPidString   = ((DKPidICM*)(ddoFolder->getPidObject()))->pidString();
                DKString ddoItemPidString     = ((DKPidICM*)(ddoItem->getPidObject()))->pidString();

                delete(ddoDocument);        // Free memory not needed anymore
                delete(ddoFolder);
                delete(ddoItem);

            cout << "Obtained PID Information." << endl;

            //-------------------------------------------------------------
            // Re-create Blank DDOs for Existing Items
            //-------------------------------------------------------------
            // As stated in the header documentation, this is one valid 
            // means to re-create a blank ddo for an single existing item 
            // and should be used over using the DKDDO() constructor.
            // Query/Search is the most popular and useful approach to 
            // obtaining empty items with Pid information filled out.
            cout << "Re-creating Blank DDOs for Existing Items..." << endl;

                ddoDocument = dsICM->createDDO(ddoDocumentPidString);
                ddoFolder   = dsICM->createDDO(ddoFolderPidString);
                ddoItem     = dsICM->createDDO(ddoItemPidString);

            cout << "Re-created Blank DDOs for Existing Items." << endl;

            //-------------------------------------------------------------
            // Retrieve Items
            //-------------------------------------------------------------
            // Retrieve the items, specifying Retrieval Options if desired.
            // For a list and explanation on Retrieval Options, see header documentation.
            cout << "Retrieving Items..." << endl;
            
               ddoDocument->retrieve();  // Use default retrieval options
               ddoFolder->retrieve(DK_CM_CONTENT_LINKS_OUTBOUND + DK_CM_CONTENT_LINKS_INBOUND); // Only Outbound is required for Folders.
               ddoItem->retrieve(DK_CM_CONTENT_ITEMTREE);  // Retrieve the whole tree of information.

            cout << "Retrieved Items." << endl;

            //-------------------------------------------------------------
            // Reading Attribute Values, Knowing Type
            //-------------------------------------------------------------
            // If the type of each attribute is known by the user program, the returned
            // java.lang.Object values can be cast into the correct value.
            cout << "Reading Attribute Values, Knowing Type..." << endl;

                // Windows Users:  You may cast to DKStrings directly, "(DKString)".
                //     AIX Users:  You must call the DKAny's ".toString()" function to convert to DKString.

                DKString attrVal1 =            ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_varchar"))).toString();
                long     attrVal2 = (long)     ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_integer"))); 
                double   attrVal3 = (double)   ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_double"))); 

                cout << "Attr 'S_varchar' value: " << attrVal1 << endl;
                cout << "Attr 'S_integer' value: " << attrVal2 << endl;
                cout << "Attr 'S_double'  value: " << attrVal3 << endl;

            cout << "Read Attribute Values, Knowing Type." << endl;

            //-------------------------------------------------------------
            // Read / Access Attribute Groups
            //-------------------------------------------------------------
            // As explained in both SAttributeGroupDefCreationICM and in
            // the Set / Modify Attributes portion of SItemCreationICM,
            // the correct format for accessing Attributes Belonging to
            // Attribute Groups is: Format:  <Attribute Group Name>.<Attribute Name>
            cout << "Read / Access Attribute Groups..." << endl;

                // Windows Users:  You may cast to DKStrings directly, "(DKString)".
                //     AIX Users:  You must call the DKAny's ".toString()" function to convert to DKString.

                DKString  attrVal4 = ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_address.S_street"))).toString(); 
                DKString  attrVal5 = ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_address.S_city"))).toString(); 
                DKString  attrVal6 = ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_address.S_state"))).toString(); 
                DKString  attrVal7 = ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,DKString("S_address.S_zipCode"))).toString(); 

                cout << "Group 'S_address', Attr 'S_street' value: " << attrVal4 << endl;
                cout << "Group 'S_address', Attr 'S_city'   value: " << attrVal5 << endl;
                cout << "Group 'S_address', Attr 'S_state'  value: " << attrVal6 << endl;
                cout << "Group 'S_address', Attr 'S_zipCode value: " << attrVal7 << endl;

            cout << "Read / Accessed Attribute Groups." << endl;

            //-------------------------------------------------------------
            // Read / Access Reference Attributes
            //-------------------------------------------------------------
            // As explained in SItemCreationICM & SReferenceAttrDefICM, usage
            // of Reference Attributes is just like regurlar Attributes.  A
            // DKDDO value is contained.  The system will manage everything.
            cout << "Read / Access Reference Attributes..." << endl;

                DKDDO* ddo = (DKDDO*) ddoItem->getData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_myRef")).value(); 

                if(ddo!=NULL){
                    cout << "             Reference:     Type:  " << ddo->getObjectType() << endl;
                    cout << "                         Item ID:  " << ( (DKPidICM*)ddo->getPidObject() )->getItemId() << endl;
                }
                else
                    cout << "             Reference:  null" << endl;

            cout << "Read / Accessed Reference Attributes." << endl;

            //-------------------------------------------------------------
            // Reading Attribute Values Programmatically: All Data Items
            //-------------------------------------------------------------
            // The type of attribute may be determined Programmatically.  A
            // mechanism is built in, or Java's "instanceof" operator may be
            // used. 
            //
            // All data items can be listed, which will obtain more than
            // the user-defined attributes.  It will find all data members
            // of the DDO.
            cout << "Reading All DDO Data Items Programmatically..." << endl;

                short dataid;
                cout << "Number of Attributes / Data Items: " << ddoItem->dataCount() << endl;
                for(dataid=1; dataid<=ddoItem->dataCount(); dataid++) { // go through all attributes in the ddo
                    DKString name = ddoItem->getDataName(dataid);
                    DKAny value   = ddoItem->getData(dataid);
                    short type    = (short) ddoItem->getDataPropertyByName(dataid,DK_CM_PROPERTY_TYPE);
                    DKString typeDescription = "To Be Determined...";
                    switch(type){
                        case DK_CM_DATAITEM_TYPE_UNDEFINED:
                            typeDescription = "Undefined";
                            break;
                        case DK_CM_DATAITEM_TYPE_STRING:
                            typeDescription = "String";
                            break;
                        case DK_CM_DATAITEM_TYPE_SHORT:
                            typeDescription = "Short";
                            break;
                        case DK_CM_DATAITEM_TYPE_LONG:
                            typeDescription = "Long";
                            break;
                        case DK_CM_DATAITEM_TYPE_FLOAT:
                            typeDescription = "Float";
                            break;
                        case DK_CM_DATAITEM_TYPE_DECIMAL:
                            typeDescription = "Decimal";
                            break;
                        case DK_CM_DATAITEM_TYPE_DATE:
                            typeDescription = "Date";
                            break;
                        case DK_CM_DATAITEM_TYPE_TIME:
                            typeDescription = "Time";
                            break;
                        case DK_CM_DATAITEM_TYPE_TIMESTAMP:
                            typeDescription = "Timestamp";
                            break;
                        case DK_CM_DATAITEM_TYPE_DOUBLE:
                            typeDescription = "Double";
                            break;
                        case DK_CM_DATAITEM_TYPE_BYTEARRAY:
                            typeDescription = "Byte Array";
                            break;
                        case DK_CM_DATAITEM_TYPE_DDOOBJECT:
                            typeDescription = "DDO";
                            break;
                        case DK_CM_DATAITEM_TYPE_XDOOBJECT:
                            typeDescription = "XDO";
                            break;
                        case DK_CM_DATAITEM_TYPE_DATAOBJECTBASE:
                            typeDescription = "Data Object Base";
                            break;
                        case DK_CM_DATAITEM_TYPE_COLLECTION:
                            typeDescription = "Collection";
                            break;
                        case DK_CM_DATAITEM_TYPE_COLLECTION_DDO:
                            typeDescription = "DDO Collection";
                            break;
                        case DK_CM_DATAITEM_TYPE_COLLECTION_XDO:  
                            typeDescription = "XDO Colleciton";
                            break;
                        case DK_CM_DATAITEM_TYPE_LINKCOLLECTION:
                            typeDescription = "DKLinkCollection";
                            break;
                        case DK_CM_DATAITEM_TYPE_ARRAY:
                            typeDescription = "Array";
                            break;
                        default:
                            typeDescription = DKString("Unknown (")+DKString(type)+DKString(")");
                            break;
                    }// end swith on data item type
                    
                    if(value.isNull())
                        cout << "Data Item '" << name << "' of type '" << typeDescription << "' with value: " << value << endl;
                    else
                        cout << "Data Item '" << name << "' of type '" << typeDescription << "' with value: null" << endl;
                }// end for all data items in DDO

            cout << "Read All DDO Data Items Programmatically." << endl;

            //-------------------------------------------------------------
            // Access Child Components
            //-------------------------------------------------------------
            cout << "Accessing Child Components..." << endl;

                // First, get the child colleciton within the parent for the child object type.
                dataid = ddoItem->dataId(DK_CM_NAMESPACE_CHILD,"S_simpleChild");
                DKChildCollection* children = (DKChildCollection*)(dkCollection*) ddoItem->getData(dataid);

                if(children!=NULL)
                    cout << "Found '" << children->cardinality() << "' Children of type 'S_address'" << endl;
                else
                    cout << "Found 'null' Children of type 'S_simpleChild'" << endl;

            cout << "Accessed Child Components." << endl;
            
            //-------------------------------------------------------------
            // Print DDO & Child components
            //-------------------------------------------------------------
            // Print a DDO using the wrapper at the bottom of this sample.            
            cout << "Printing DDO..." << endl;
                
                printDDO(ddoItem);      // Use wrapper defined at bottom of sample.

                delete(ddoDocument);    // Free Memory
                delete(ddoFolder);
                delete(ddoItem);

            cout << "Printed DDO." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            exit(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print the specified DDO.
    * @param ddo  DKDDO to print.
    **/
    void SItemRetrievalICM::printDDO(DKDDO * ddo){
        DKSequentialCollection childCollections; // when obtaining data items, collect child collections.
        DKSequentialCollection linkCollections;  // when obtaining data items, collect link collections.

        DKPidICM* pid = (DKPidICM*) ddo->getPidObject(); // Obtain the Pid Information
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) ddo->getDatastore()->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKComponentTypeViewDefICM* compType = (DKComponentTypeViewDefICM*) dsDefICM->retrieveComponentTypeView(pid->getObjectType());
        
        cout << "--------------------------------------------------"          << endl;
        cout << "                       DDO"                                  << endl;
        cout << "--------------------------------------------------"          << endl;
        cout << "PID:"                                                        << endl;
        cout << "        Component Type:  " << pid->getObjectType()           << endl;
        cout << "               Item ID:  " << pid->getItemId()               << endl;
        cout << "          Component ID:  " << pid->getComponentId()          << endl;
        cout << "     Component Type ID:  " << pid->getComponentTypeId()      << endl;
        cout << "            Primary ID:  " << pid->getPrimaryId()            << endl;
        cout << "        Datastore Type:  " << pid->getDatastoreType()        << endl;
        cout << "               Version:  " << pid->getVersionNumber()        << endl;
        cout << "Item Info:"                                                  << endl;
        if(compType->isRoot()==false){ // if it isn't the root
            if(ddo->getParent()==NULL) // if no parent is specified and it is a child, this means that this DDO was retrieved seperately from the parent and therefore the parent is not known to this object in memory.  Obtaining the parent and retrieving "One Level" or "Item Tree" would set this property.  Alternatively, query/search may be used to find the parent, as shown in the retrieveParent() function provided in this sample.  Refer to Information Integrator for Content eSupport for more information.
                cout << "                Parent:  Exists, But Unknown to Object in Memory" << endl;
            else{
                cout << "                Parent:     Type:  " << ddo->getParent()->getObjectType() << endl;
                cout << "                         Item ID:  " << ((DKPidICM*)((DKDDO*)ddo->getParent())->getPidObject())->getItemId() << endl;
            }
        }else
            cout << "                Parent:  None, Is Root"                  << endl;
        if(compType->isRoot()){ // Only report XDO information if it is the root item.
            DKItemTypeViewDefICM* itemTypeDef = (DKItemTypeViewDefICM*)compType;  // If it is a root, then it is an Item Type Def.
            if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_ITEM)
                cout << "          Resource XDO:  No"                     << endl;
            else{
                switch(itemTypeDef->getXDOClassID()){
                    case DK_ICM_XDO_VIDEOSTREAM_CLASS_ID:
                        cout << "          Resource XDO:  Yes - Video Stream"     << endl;
                        break;
                    case DK_ICM_XDO_STREAM_CLASS_ID:
                        cout << "          Resource XDO:  Yes - Stream"           << endl;
                        break;
                    case DK_ICM_XDO_IMAGE_CLASS_ID:
                        cout << "          Resource XDO:  Yes - Image"            << endl;
                        break;
                    case DK_ICM_XDO_TEXT_CLASS_ID:
                        cout << "          Resource XDO:  Yes - Text"             << endl;
                        break;
                    case DK_ICM_XDO_LOB_CLASS_ID:
                        cout << "          Resource XDO:  Yes - Lob"              << endl;
                        break;
                    default:
                        cout << "          Resource XDO:  Yes - Other (" << itemTypeDef->getXDOClassID() << ")" << endl;
                        break;
                }//end switch on XDO Class.
            }

            if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_DOC_MODEL)
                cout << "     CM Document Model:  Yes" << endl;
            else if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_DOC_PART)
                cout << "     CM Document Model:  Yes (Part)" << endl;
            else
                cout << "     CM Document Model:  No" << endl;
        }
        
        if(compType->isRoot()){ // only applicable to roots
            long  semanticType     = (long)  ddo->getPropertyByName(DK_ICM_PROPERTY_SEMANTIC_TYPE);
            short itemPropertyType = (short) ddo->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);

            switch(itemPropertyType){
                case DK_CM_DOCUMENT:
                    cout << "         Item Property:  Document"              << endl;
                    break;
                case DK_CM_FOLDER:
                    cout << "         Item Property:  Folder"                << endl;
                    break;
                case DK_CM_ITEM:
                    cout << "         Item Property:  Item"                  << endl;
                    break;
                default:
                    cout << "         Item Property:  Other (" << itemPropertyType << ")" << endl;
                    break;
            }//end switch on semantic type
            switch(semanticType){
                case DK_ICM_SEMANTIC_TYPE_DOCUMENT:
                    cout << "         Semantic Type:  Document"              << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_FOLDER:
                    cout << "         Semantic Type:  Folder"                << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_CONTAINER:
                    cout << "         Semantic Type:  Container"             << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_ANNOTATION:
                    cout << "         Semantic Type:  Annotation"            << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_HISTORY:
                    cout << "         Semantic Type:  History"               << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_NOTE:
                    cout << "         Semantic Type:  Note"                  << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_BASE:
                    cout << "         Semantic Type:  Base"                  << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_BASETEXT:
                    cout << "         Semantic Type:  Base Text"             << endl;
                    break;
                case DK_ICM_SEMANTIC_TYPE_BASESTREAM:
                    cout << "         Semantic Type:  Base Stream"           << endl;
                    break;
                default:
                    cout << "         Semantic Type:  Other (" << semanticType << ")" << endl;
                    break;
            }//end switch on semantic type
        }// end if is root
        if(ddo->updatable())
            cout << "            Updateable:  Yes"                           << endl;
        else
            cout << "            Updateable:  No"                            << endl;

        cout << "Attributes / Data Items:"                                   << endl;
        cout << "                Number:  " << ddo->dataCount()              << endl;
        short dataid;
        // List all Data Items
        for(dataid=1; dataid<=ddo->dataCount(); dataid++) { // go through all attributes in the ddo
            cout << "             Attribute:     Name:  " << ddo->getDataName(dataid) << endl;
            DKString name = ddo->getDataName(dataid);
            DKAny   value = ddo->getData(dataid);
            short    type = (short) ddo->getDataPropertyByName(dataid, DK_CM_PROPERTY_TYPE);
            DKString typeDescription = "To Be Determined...";
            switch(type){
                case DK_CM_DATAITEM_TYPE_UNDEFINED:
                    cout << "                            Type:  Undefined"     << endl;
                    if(value.isNull()==FALSE) 
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_STRING:
                    cout << "                            Type:  String"        << endl;
                    if(value.isNull()==FALSE) 
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_SHORT:
                    cout << "                            Type:  Short"         << endl;
                    if(value.isNull()==FALSE) 
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_LONG:
                    cout << "                            Type:  Long"          << endl;
                    if(value.isNull()==FALSE) 
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_FLOAT:
                    cout << "                            Type:  Float"         << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_DECIMAL:
                    cout << "                            Type:  Decimal"       << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_DATE:
                    cout << "                            Type:  Date"          << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_TIME:
                    cout << "                            Type:  Time"          << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_TIMESTAMP:
                    cout << "                            Type:  Timestamp"     << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_DOUBLE:
                    cout << "                            Type:  Double"        << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_BYTEARRAY:
                    cout << "                            Type:  Byte Array (Blob)" << endl;
                    if(value.isNull())
                        cout << "                           Value:  " << value << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_DDOOBJECT:
                    cout << "                            Type:  DDO"           << endl;
                    if(value.isNull()==FALSE){ 
                        cout << "                           Value:  DDO of type '" << ((DKPidICM*)((DKDDO*)(dkDataObjectBase*)value)->getPidObject())->getObjectType() << "'" << endl;
                        cout << "                                       Item ID '" << ((DKPidICM*)((DKDDO*)(dkDataObjectBase*)value)->getPidObject())->getItemId() << "'" << endl;
                    }
                    break;
                case DK_CM_DATAITEM_TYPE_XDOOBJECT:
                    cout << "                            Type:  XDO"           << endl;
                    if(value.isNull()==FALSE){
                        cout << "                           Value:  XDO of type '" << ((DKPidICM*)((DKDDO*)(dkDataObjectBase*)value)->getPidObject())->getObjectType() << "'" << endl;
                        cout << "                                       Item ID '" << ((DKPidICM*)((DKDDO*)(dkDataObjectBase*)value)->getPidObject())->getItemId() << "'" << endl;
                    }
                    break;
                case DK_CM_DATAITEM_TYPE_DATAOBJECTBASE:
                    cout << "                            Type:  Data Object Base" << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  <Data Object Base Object>" << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_COLLECTION:
                    cout << "                            Type:  Collection"    << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  Collection with '" << ((dkCollection*)value)->cardinality() << "' Objects." << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_COLLECTION_DDO:
                    cout << "                            Type:  DDO Collection" << endl;
                    if(value.isNull()==FALSE){
                        cout << "                           Value:  DDO Collection with '" << ((dkCollection*)value)->cardinality() << "' DDOs." << endl;
                        // This should be a DKChildCollection, add it to a list for printing children later
                        childCollections.addElement((dkCollection*)value);
                    }
                    break;
                case DK_CM_DATAITEM_TYPE_COLLECTION_XDO:  
                    cout << "                            Type:  XDO Collection" << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  XDO Collection with '" << ((dkCollection*)value)->cardinality() << "' XDOs." << endl;
                    break;
                case DK_CM_DATAITEM_TYPE_LINKCOLLECTION:
                    cout << "                            Type:  Link Collection" << endl;
                    if(value.isNull()==FALSE){
                        cout << "                           Value:  Link Collection with '" << ((dkCollection*)value)->cardinality() << "' Links." << endl;
                        // This should be a DKLinkCollection, add it to a list for printing Links later
                            linkCollections.addElement((dkCollection*)value);
                    }
                    break;
                case DK_CM_DATAITEM_TYPE_ARRAY:
                    cout << "                            Type:  Array" << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
                default:
                    cout << "                            Type:  Unknown (" << type << ")" << endl;
                    if(value.isNull()==FALSE)
                        cout << "                           Value:  " << value << endl;
                    break;
            }// end swith on data item type

            if(value.isNull()==TRUE) 
                    cout << "                           Value:  null" << endl;
            cout << endl;
        }// end for all data items in DDO

        // Print Children (Brief)
        cout << "Children / Multivalue Attributes:" << endl;
        dkIterator* childCollectionsIter = childCollections.createIterator(); // Create iterator to go through the gathered collections.
        while(childCollectionsIter->more()){  // go through all collection we gathered when listing the data items of the DDO.
            DKChildCollection* childCollection = (DKChildCollection*) childCollectionsIter->next()->value();
            dkIterator* iter = childCollection->createIterator(); // Create an iterator to go through Child Collection
            while(iter->more()){ // While there are children, print list
                DKDDO* child = (DKDDO*) iter->next()->value(); // Move pointer to next child & return that object.
                cout << "                 Child:     Type:  " << child->getPidObject()->getObjectType() << endl;
                cout << "                         Item ID:  " << ((DKPidICM*)child->getPidObject())->getItemId() << endl;
                cout << endl;
            }
            delete(iter);
        }
        delete(childCollectionsIter);
            
        // Print Parts, if a Parts Collection Exists (See Document Model Samples for more information)
        dataid = ddo->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKPARTS);
        if(dataid > 0){ // if a DKParts collection exists, print contents
            DKParts* dkParts = (DKParts*)(dkCollection*) ddo->getData(dataid); // obtain the DKParts collection
            cout << "Parts:" << endl;
            cout << "                Number:  " << dkParts->cardinality() << endl;
            dkIterator* iter = dkParts->createIterator(); // Create an iterator to go through Collection
            while(iter->more()){ // While there are children, print list
                DKDDO* part = (DKDDO*) iter->next()->value(); // Move pointer to next part & return that object.
                cout << "                  Part:     Type:  " << part->getPidObject()->getObjectType() << endl;
                cout << "                         Item ID:  " << ((DKPidICM*)part->getPidObject())->getItemId() << endl;
                cout << endl;
            }
            delete(iter);
        }

        // Print Folder Contents, if a DKFolder Collection Exists (See Folder Samples for more information)
        dataid = ddo->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER);
        if(dataid > 0){ // if a DKFolder collection exists, print contents
            DKFolder* dkFolder = (DKFolder*)(dkCollection*) ddo->getData(dataid); // obtain the DKFolder collection
            cout << "Folder Contents:" << endl;
            cout << "                Number: " << dkFolder->cardinality() << endl;
            dkIterator* iter = dkFolder->createIterator(); // Create an iterator to go through Collection
            while(iter->more()){ // While there are children, print list
                DKDDO* contents = (DKDDO*) iter->next()->value(); // Move pointer to next element & return that object.
                cout << "                  Item:     Type:  " << contents->getPidObject()->getObjectType() << endl;
                cout << "                         Item ID:  " << ((DKPidICM*)contents->getPidObject())->getItemId() << endl;
                cout << endl;
            }
            delete(iter);
        }
        
        // Print Links
        cout << "Links:" << endl;
        dkIterator* linkCollectionsIter = linkCollections.createIterator();   // Create an iterator to go through the link collections we gathered.
        while(linkCollectionsIter->more()){  // go through all link collections we gathered when listing the data items of the DDO.
            DKLinkCollection* linkCollection = (DKLinkCollection*) linkCollectionsIter->next()->value(); // get each Link Collection
            dkIterator* iter = linkCollection->createIterator();  // Create an iterator to go through Link Collection
            while(iter->more()){ // While there are links, print list
                DKLink* dkLink = (DKLink*) iter->next()->value(); // Move pointer to next element & return that object.
                cout << "                  Link:     Type:  " << dkLink->getTypeName() << endl;
                cout << "                       Source ID:  " << ((DKPidICM*)((DKDDO*)dkLink->getSource())->getPidObject())->getItemId() << "(" << ((DKDDO*)dkLink->getSource())->getPidObject()->getObjectType() << ")" << endl;
                cout << "                       Target ID:  " << ((DKPidICM*)((DKDDO*)dkLink->getTarget())->getPidObject())->getItemId() << "(" << ((DKDDO*)dkLink->getTarget())->getPidObject()->getObjectType() << ")" << endl;
                if(dkLink->getLinkItem()!=NULL)
                    cout << "                     LinkItem ID:  " << ((DKPidICM*)((DKDDO*)dkLink->getLinkItem())->getPidObject())->getObjectType() << endl;
                else
                    cout << "                     LinkItem ID:  null" << endl;
                cout << endl;
            }
            delete(iter);
        }
        delete linkCollectionsIter;
        cout << "--------------------------------------------------------" << endl;
        cout << endl;
        
        // Print Full Children
        childCollectionsIter = childCollections.createIterator();
        while(childCollectionsIter->more()){  // go through all collection we gathered when listing the data items of the DDO.
            DKChildCollection* childCollection = (DKChildCollection*) childCollectionsIter->next()->value(); // get each child collection
            dkIterator* iter = childCollection->createIterator(); // Create an iterator to go through Child Collection
            while(iter->more()){ // While there are children, print each
                DKDDO* child = (DKDDO*) iter->next()->value(); // Move pointer to next child & return that object.
                printDDO(child);
            }
            delete(iter);
        }
        delete(childCollectionsIter);
        delete(compType);
    }// end printDDO

   /**
    * Retrieve the parent DKDDO of the specified child component
    * if one exists.  If the parent reference is set in the 
    * child component, it will be returned from memory without
    * performing a search.  If the DKDDO is discovered to be
    * a root component, NULL will be returned before searching.
    * The following query will be executed if this is a child
    * component.  
    * 
    * //A[@ITEMID="B" AND (./C[@COMPONENTID="D" AND @VERSIONID=E])]
    * A:  Parent Component View Type Name or *.
    * B:  Item ID (All components of the same item share the same Item ID).
    * C:  Child Component View Type Name.
    * D:  Child Component ID.
    * E:  Version ID (All components of the same item version share the same version ID). 
    *
    * Faster solution exist.  This function covers one alternative
    * to retrieve the parent when DKDDO.getParent() returns 'null'.
    * Please refer to the Information Integrator for Content eSupport 
    * for more information on obtaining the parent of a child component.
    *
    * @param ddo - DDO to get the parent of.
    * @return Returns the parent with completed PID information or
    *         NULL if no parent exists.
	* MEMORY MANAGEMENT: Caller is responsible for deleting the DDO returned.
    **/
    DKDDO* SItemRetrievalICM::retrieveParent(DKDDO* ddo){

        cout << "Retrieving Parent DDO..." << endl;

        // First Check if Parent Reference is Set
        DKDDO* parentDDO = (DKDDO*) ddo->getParent();
        if(parentDDO!=NULL){ // If it was set, return it.
            cout << "Parent reference was set in memory.  Returned local reference." << endl;
            return(parentDDO);   
        }

        // Gether Information
        DKPidICM* pid = (DKPidICM*) ddo->getPidObject();
        DKString itemId           = pid->getItemId();        // B
        DKString childViewName    = pid->getObjectType();    // C
        DKString compId           = pid->getComponentId();   // D
        DKString verId            = pid->getVersionNumber(); // E

        // Next, check if it is a root component.
        DKDatastoreICM*    dsICM    = (DKDatastoreICM*)    ddo->getDatastore();
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();
        DKComponentTypeViewDefICM* childViewDef = (DKComponentTypeViewDefICM*) dsDefICM->retrieveComponentTypeView(childViewName);
        if(childViewDef->isRoot()){ // If it is a root component, return 'null'.
            cout << "Type '" << childViewName << "' determined to be a root component.  Returning 'null' since no parent can exist." << endl;
            delete(childViewDef); // Free Memory
            return(NULL);
        }
        
        // Look up parent component view type name.
        DKComponentTypeViewDefICM* parentViewDef = (DKComponentTypeViewDefICM*) childViewDef->retrieveParent();
        DKString parentViewName = parentViewDef->getName(); // A

        // Build Query String
        DKString query = "//";
        query += parentViewName;
        query += "[@ITEMID=\"";
        query += itemId;
        query += "\" AND (./";
        query += childViewName;
        query += "[@COMPONENTID=\"";
        query += compId;
        query += "\" AND @VERSIONID=";
        query += verId;
        query += "])]";
            
        // Build Options
        DKNVPair* options = new DKNVPair[3];
        options[0].set(DK_CM_PARM_MAX_RESULTS, DKString("1"));              // Specify max using a string value.
        options[1].set(DK_CM_PARM_RETRIEVE   , (long)DK_CM_CONTENT_IDONLY); // Specify any Retrieval Options desired.  Default is ATTRONLY.
        options[2].set(DK_CM_PARM_END        , (long)0);                    // Must mark the end of the NVPair

        // Process Query
        cout << "Executing Query:  " << query << endl;
        dkResultSetCursor* cursor = dsICM->execute(query, DK_CM_XQPE_QL_TYPE, options);
            
        // Return Parent
        parentDDO = cursor->fetchNext();

        if(parentDDO!=NULL) cout << "Parent DDO found:  Type = '" << parentDDO->getObjectType() << "', CompID = '" << ((DKPidICM*)parentDDO->getPidObject())->getComponentId() << "'" << endl;
        else                cout << "No parent found." << endl;

        // Free Resources
        cursor->destroy();      // Close & Destroy Cursor, Ending Implied Transaction.
        delete(cursor);         // DKResultSetCursor is an advanced object that is aware of the content within.  It will handle the deletion of the DDOs within.
        delete[] options;
        delete(childViewDef);
        delete(parentViewDef);

        return(parentDDO);
    }//end retrieveParent
