/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SResourceMgrDefDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceMgrDefDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Resource Manager Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes Resource Manager Definitions.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SResourceMgrDefDeletionICM <Existing RM User-Defined Reference Name>
 *                                               <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined.  It should be
 *                          all upper-case and should be the same name specified in the
 *                          SResourceMgrDefCreationICM Sample.
 *
 *                    NOTE: SMS Collection Definitions for this RM Definition must first be
 *                          deleted.  Please refer to the SSMSCollectionDefDeletionICM Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SResourceMgrDefDeletionICM MY_RM
 *                    ex: SResourceMgrDefDeletionICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined.  This should be 
 *                    the same name specified in the SResourceMgrDefCreationICM Sample.
 *                    All SMS Collection must first be deleted.  Please refer to the SMS
 *                    Collection Definition Deletion Sample, SSMSCollectionDefDeletionICM.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SResourceMgrDefDeletionICM_launch.cpp
 *                    SResourceMgrDefDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceMgrDefDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: "  << endl;
            cout << "  SResourceMgrDefDeletionICM <Existing RM User-Defined Reference Name>"      << endl;
            cout << "                             <database> <userName> <password>"               << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined.  This should"  << endl;
            cout << "        be all upper-case and should be the same name specified in the"      << endl;
            cout << "        SResourceMgrDefCreationICM Sample."                                  << endl;
            cout << endl; 
            cout << "  NOTE: SMS Collection Definitions for this RM Definition must first be"     << endl;
            cout << "        deleted.  Please refer to the SSMSCollectionDefDeletionICM Sample."  << endl;
            cout << endl; 
            cout << "  NOTE: Defaults will be used if the database, username, & password are not" << endl;
            cout << "        specified.  All others are required."                                << endl;
            cout << endl;
            cout << "  ex: SResourceMgrDefSetDefaultICM RMDB"                                     << endl;
            cout << "  ex: SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password"         << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="          << endl;
        cout << "IBM Information Integrator for Content v" << ver      << endl;
        cout << "Sample Program:  SResourceMgrDefDeletionICM"          << endl;
        cout << "-------------------------------------------"          << endl;
        cout << "           Database:  " << database                   << endl;
        cout << "           UserName:  " << userName                   << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName << endl;       
        cout << "===========================================\n"        << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Obtaining the Configuration Management Objects
            //-------------------------------------------------------------
            // The sample code blow demonstrates how to obtain objects for
            // Datastore Definition, Datastore Administration, Configuration
            // Management, and RM Configuration Management.
            cout << "Obtaining Configuration Management Objects..." << endl;

                DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
                DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
                DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
                DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            cout << "Obtained Configuration Management Objects." << endl;

            //-------------------------------------------------------------
            // Delete the Resource Manager Definition                        
            //-------------------------------------------------------------
            cout << "Deleting Resource Manager Definition..." << endl;

                // Only Delete if It Exists
            if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM,rmUserDefinedReferenceName))
                    rmCfg->delResourceMgr(rmUserDefinedReferenceName);

            cout << "Deleted Resource Manager Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // deleteResourceMgrDef
    //******************************************************************
   /**
    * Delete the specified Resource Manager Definition.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that Items of this Item Type should be stored in.  It should consist of all upper-case letters.
    **/
    void SResourceMgrDefDeletionICM::deleteResourceMgrDef(DKDatastoreICM* dsICM, char* rmUserDefinedReferenceName){
        cout << "Deleting Resource Manager Definition Named '" << rmUserDefinedReferenceName << "'..." << endl;
        
            // Make sure the RM Definition is setup.
            if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
                throw DKException("Resource Manager Definition not defined.");

            // Obtaining the Configuration Management Objects
            DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
            DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
            DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
            DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            rmCfg->delResourceMgr(rmUserDefinedReferenceName); // Delete the ResourceMgrDef

        cout << "Deleted Resource Manager Definition Named '" << rmUserDefinedReferenceName << "'." << endl;
    }
