/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Text Indexes
    A text index is used to perform text searches on text in attributes or in 
    document content.  The user may specify a list of search words and obtain 
    results.  Searching text this way is far more efficient and flexible than
    using LIKE in a SQL query.
    
Updateing Text Indexes
    When an Item (database row) is added (to an ItemType table) that has a 
    Text Index, that Item (row) is put in a queue to be added to the Text
    Index.  After some period of time, the information is processed, taken
    out of the queue, and can be found by searching the Text Index.
    
    Updating a text index is performed to force TIE to immediately process 
    all the Items (rows) in its queue.

Reorganizing Text Indexes
    Text indexes should be reorganized for optimal performance after updating
    the text index.

For detailed information about DB2 Text Information Extender, please refer to the
DB2 Text Information Extender documentation.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "STextIndexUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: STextIndexUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Text Indexes, Updating Text Indexes, Reorganizing Text Indexes, Updating
 *                    all Text Indexes in System.  Listing all Component Type Definitions in
 *                    System.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Text Indexes
 *                    Updating Text Indexes
 *                    Reorganize Text Indexes
 *                    Listing all Component Type Definitions in System
 *                    Updating all Text Indexes in System
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: STestIndexUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: none
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    STextIndexUpdateICM_launch.cpp
 *                    STextIndexUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int STextIndexUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  STextIndexUpdateICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  STextIndexUpdateICM"            << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Obtain a list of Component Type Definitions
            //-------------------------------------------------------------
            // First, we need a list of Component Type Definitions for
            // which we will update the Text Index for each.
            cout << "Obtaining List of All Component Type Definitions in System..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*)dsICM->datastoreDef();        // Obtain the Datastore Definition Object
                dkCollection* coll = dsDefICM->listComponentTypes(DK_ICM_ALL_COMPONENT_TYPES);  // Obtain all Component Type Definitions in the System (Including System-Defined)

            cout << "Obtained List of All Component Type Definitions in System." << endl;

            //-------------------------------------------------------------
            // Update Text Index for Each
            //-------------------------------------------------------------
            // Go through all Component Types in the System obtained above
            // and update the text index on each.
            //
            // Each index should also be reorganized for optimal performance.
            cout << "Updating Text Indexes..." << endl;

                dkIterator* iter = coll->createIterator();
                while (iter->more()) {
                    DKComponentTypeDefICM* compTypeDef = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                    dsDefICM->updateTextIndexes(compTypeDef->getComponentTypeId());             // Update the Text Index for Each.
                    cout << "     Updated Text Index for " << compTypeDef->getName() << ":  " << compTypeDef->getDescription() << endl;
                    dsDefICM->reorgTextIndexes(compTypeDef->getComponentTypeId());              // Reorganize for Optimal Performance.
                    cout << " Reorganized Text Index for " << compTypeDef->getName() << ":  " << compTypeDef->getDescription() << endl;
                    delete(compTypeDef);  // Free Memory.  Contents must be deleted individually.
                }
                delete(iter);             // Free Memory
                delete(coll);             // Contents must already be deleted explicitly by the User-Application.

            cout << "Updated Text Indexes." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Update all Text Indexes in System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void STextIndexUpdateICM::updateAllTextIndexes(DKDatastoreICM* dsICM){
        cout << "Updating All Text Indexes in System..." << endl;

            // Obtain a list of Component Type Definitions
            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*)dsICM->datastoreDef();       // Obtain the Datastore Definition Object
            dkCollection* coll = dsDefICM->listComponentTypes(DK_ICM_ALL_COMPONENT_TYPES); // Obtain all Component Type Definitions in the System (Including System-Defined)

            // Update Text Index for Each
            dkIterator* iter = coll->createIterator();
            while (iter->more()) {
                DKComponentTypeDefICM* compTypeDef = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                dsDefICM->updateTextIndexes(compTypeDef->getComponentTypeId());             // Update the Text Index for Each.
                cout << "     Updated Text Index for " << compTypeDef->getName() << ":  " << compTypeDef->getDescription() << endl;
                dsDefICM->reorgTextIndexes(compTypeDef->getComponentTypeId());              // Reorganize for Optimal Performance.
                cout << " Reorganized Text Index for " << compTypeDef->getName() << ":  " << compTypeDef->getDescription() << endl;
                delete(compTypeDef);      // Free Memory.  Contents must be deleted individually within a generic collection object.
            }
            delete(iter);                 // Free Memory.
            delete(coll);                 // Contents must already be deleted explicitly by the User-Application.

        cout << "Updated All Text Indexes in System." << endl;
    }//end updateAllTextIndexes
