//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//

// testapi.cpp

#include <time.h>
#include "DKString.hpp"
#include "DKDate.hpp"
#include "esSession.h"
#include "esException.h"
#include "esApplication.h"
#include "esDatabase.h"
#include "esPropertyList.h"
#include "esBroker.h"
#include "esDocument.h"
#include "esArray.h"
#include "esConstraint.h"


void displayResult(esDocListImpl& MyDocs)
{
	int max2 = MyDocs.getDocumentCount();

	for (int j=0; j< max2; j++)
	{
		esDocument& doc = MyDocs.getDocument(j);
		cout << "Doc Id = " << doc.getDocId() << endl;
		esPropertyList& list = doc.getProperties();
		int iPCount = list.getPropertyCount();
		cout << "field count = " << iPCount << "\n\n";
		for (int k = 0; k < iPCount; k++)
		{
			esProperty& prop = list.getProperty(k);
			switch (prop.getSchema())
			{
				case ES_PROPERTY_BLOB:
				{
					cout << "field " << prop.getName() << " contains blob\n";
					FILE * pFile = fopen("c:\\temp\\blob1.dat", "a+b");
					if (pFile) 
					{
						cout << "write a blob to an output file" << endl;
						if (prop.getValueCount() > 0) 
						{
							esBlob blob= prop.getValueAsBlob(0);
							fwrite(blob.getValue(), 1,blob.getLength(), pFile);
							fclose(pFile);
						}
					}
				}
				break;

				case ES_PROPERTY_DATETIME:
				{
					cout << "field " << prop.getName() << "=";
					esDate date = prop.getValueAsDate(0);
					struct tm t = date.getValue();
//					DKTimestamp tt((t.tm_year+1900), t.tm_mon+1, t.tm_mday,
//						t.tm_hour, t.tm_min, t.tm_sec, 0, t.tm_isdst);
					cout << date << endl;
					cout << t.tm_mon+1 << "/" << t.tm_mday << "/" << t.tm_year << " " 
						<< t.tm_hour << ":" << t.tm_min << ":" << t.tm_sec << endl;
				}
				break;
				case ES_PROPERTY_STRING:
				{
					cout << "field " << prop.getName() << "=";
					esString esStr = prop.getValueAsString(0);
					cout << esStr << endl;;
				}
				break;
				case ES_PROPERTY_INT:
				{
					cout << "field " << prop.getName() << "=";
					long l = (long)prop.getValueAsInt(0);
					cout << l << endl;
				}
				break;
				case ES_PROPERTY_DOUBLE:
				{
					cout << "field " << prop.getName() << "=";
					double db = (double)prop.getValueAsDouble(0);
					cout << db << endl;
				}
				break;

			}
		}
		cout << endl;
	}
}

int main(int argc, char* argv[])
{
	esArray<esDatabase>   SearchDbs(2);

	esArray<esString> QueryProps(3);

	esArray<esConstraint>  NoConstraints(0);
	esArray<esConstraint>  MaxHits(1);

	MaxHits[0].Name     = "MAX HITS";
	MaxHits[0].Value    = "2";

	try
	{
		esApplication app("edsm2","edsm2");
		esBroker      svr("ross.stl.ibm.com", 80);

		esSession     *sess = new esSession(app, svr);

		esUser        usr("edsm2", "edsm2");
		(*sess).assignUser(usr);

		esArray<esDatabase> dba = svr.getDatabases(*sess, NoConstraints);
		int iCount = dba.length();
		esString esStr;
		DKString dkStr;
		int i = 0;
		esString GQL = "";

		//////////////////////////////////////////////////////////////
		// Test - File system
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - File system" << endl;
		esDocListImpl         MyDocs3;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "ES Documentation")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(IN \"Doc$FileName\" \"a\")";
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs3,MaxHits);

		cout << "num docs found =" << MyDocs3.getDocumentCount() << "\n";

		if (MyDocs3.getDocumentCount() > 0)
			svr.fetchDocFields(*sess,MyDocs3.getDocument(0),NoConstraints);

		displayResult(MyDocs3);


		//////////////////////////////////////////////////////////////
		// Test - NotesPump database
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - NotesPump databases" << endl;
		esDocListImpl         MyDocs4;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "EMPLOYEE")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(GT \"BIRTHDATE\" \"19330824\")";
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs4,MaxHits);

		cout << "num docs found =" << MyDocs4.getDocumentCount() << "\n";

		if (MyDocs4.getDocumentCount() > 0)
			svr.fetchDocFields(*sess,MyDocs4.getDocument(0),NoConstraints);

		displayResult(MyDocs4);


		//////////////////////////////////////////////////////////////
		// Test - NotesPump database, fetch a blob
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - NotesPump databases, fetch a blob" << endl;
		esDocListImpl         MyDocs5;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "EMP_PHOTO")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(EQ \"EMPNO\" \"000130\")";
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs5,MaxHits);

		cout << "num docs found =" << MyDocs5.getDocumentCount() << "\n";

		if (MyDocs5.getDocumentCount() > 0)
			svr.fetchDocFields(*sess,MyDocs5.getDocument(0),NoConstraints);

		displayResult(MyDocs5);

		//////////////////////////////////////////////////////////////
		// Test - Web search engine, text search
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - HotBot - Web" << endl;
		esDocListImpl         MyDocs6;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "HotBot - Web")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(AND \"clinton\" \"internet\")";
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs6,MaxHits);

		cout << "num docs found =" << MyDocs6.getDocumentCount() << "\n";

		if (MyDocs6.getDocumentCount() > 0)
			displayResult(MyDocs6);

		//////////////////////////////////////////////////////////////
		// Test - Lotus Notes database
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - Lotus Notes databases" << endl;
		esDocListImpl         MyDocs1;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "Notes 5 Help")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(OR \"multiple\" \"users\")";
		QueryProps[0] = "Subject";
		QueryProps[1] = "Indent";
		QueryProps[2] = "Doc_Number";
		svr.setDocQueryProperties(QueryProps);
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs1,MaxHits);

		cout << "num docs found =" << MyDocs1.getDocumentCount() << "\n";

		if (MyDocs1.getDocumentCount() > 0)
			svr.fetchDocFields(*sess,MyDocs1.getDocument(0),NoConstraints);

		displayResult(MyDocs1);

		//////////////////////////////////////////////////////////////
		// Test - Lotus Notes database
		// query:    Complex expression (AND (GT"Doc_Number""2.0") (EQ"Indent""2.0"))
		////////////////////////////////////////////////////////////
		cout << "\n\nTest - Lotus Notes databases, complex expression" << endl;
		esDocListImpl         MyDocs2;
		for (i = 0; i< iCount; i++) 
		{
			esStr = dba[i].getName();
			dkStr = esStr.chars();
			if (dkStr == "Notes 5 Help")
			{
				SearchDbs[0] = dba[i];
				SearchDbs[0].setEnabled(true);
				cout << "db[" << i << "] is selected\n";
				break;
			}
		}

		cout << "Query Document\n\n";
		GQL = "(AND(GT\"Doc_Number\"\"2.0\")(EQ\"Indent\"\"2.0\"))";
		QueryProps[0] = "Subject";
		QueryProps[1] = "Indent";
		QueryProps[2] = "Doc_Number";
		svr.setDocQueryProperties(QueryProps);
		svr.queryDocs(*sess,GQL,SearchDbs,MyDocs2,MaxHits);

		cout << "num docs found =" << MyDocs2.getDocumentCount() << "\n";

		if (MyDocs2.getDocumentCount() > 0)
			svr.fetchDocFields(*sess,MyDocs2.getDocument(0),NoConstraints);

		displayResult(MyDocs2);


		delete sess;
	}
	catch (esException e)
	{
		e.display();
	}

	return 0;
}
