/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import com.ibm.mm.sdk.common.*;
import java.beans.*;
import java.util.*;

/**
 * This sample demonstrates performing a search using EIP search templates.
 * The search technique used is asynchronous search, which fires CMBResultEvents
 * on a secondary thread as results arrive.
 */
public class DemoAsynchSearch implements CMBResultListener {
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    CMBSchemaManagement schemaBean = null;
    CMBSearchResults resultBean = null;
    int _templateMaxHits = 0;
    int _templateTimeOut = 0;
    boolean _templateTrace = false;

    DemoAsynchSearch() {
        try {
            System.out.println("Creating CMBSchemaManagement bean using " + "ser/class/jar specified in the classpath.");
            schemaBean = (com.ibm.mm.beans.CMBSchemaManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBSchemaManagement");
            resultBean = (com.ibm.mm.beans.CMBSearchResults) Beans.instantiate(null, "com.ibm.mm.beans.CMBSearchResults");
            resultBean.addCMBResultListener(this);
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public void onCMBResult(CMBResultEvent evt) {
        int id = evt.getID();
        System.out.println("Main.onCMBResult: id = " + id);
        switch (id) {
            case CMBResultEvent.CMB_RESULT_UPDATED :
                //int nHits = resultBean.getCount();
                System.out.println("\tCMB_RESULT_UPDATED:....");
                Vector resList = (Vector) evt.getData();
                int nHits = resList.size();
                System.out.println("There is " + nHits + " from the search");

                for (int k = 0; k < nHits; k++) {
                    try {
                        CMBHitItem hit = resultBean.getHitItem(k);
                        System.out.println("Hit item #" + k + ", pidString=" + hit.getPidString());
                        String[] values = hit.getAttrValue();
                        for (int l = 0; l < values.length; l++)
                            System.out.println("\tattrValue: " + values[l]);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }

                break;
            case CMBResultEvent.CMB_RESULT_CLEARED :
                System.out.println("\tCMB_RESULT_CLEARED:....");
                break;
            default :
                break;
        }
    }

    void changeProperty() {
        boolean boolVal = false;
        int intVal = 0;

        boolVal = ui.getBoolean("Enable the schema bean cache capability?");
        schemaBean.setCacheEnabled(boolVal);

        boolVal = ui.getBoolean("Enable the schema bean trace capability?");
        schemaBean.setTraceEnabled(boolVal);

        intVal = ui.getInteger("New the results bean page size [default is 20] ?");
        if (intVal > 0)
            resultBean.setPageSize(intVal);

        boolVal = ui.getBoolean("Enable the results bean trace capability?");
        resultBean.setTraceEnabled(boolVal);

        _templateTrace = ui.getBoolean("Enable the search template bean trace capability?");
        _templateTimeOut = ui.getInteger("Max query time before timeout? [default is no timeout");
    }

    void run() {
        int count;
        try {

            // get search template names
            System.out.println("Getting search template names...");
            String stNames[] = schemaBean.getSearchTemplateNames();
            count = stNames.length;
            System.out.println("Done, search template count is: " + count + "....");

            CMBSearchTemplate stObj = null;
            for (int i = 0; i < count; ++i) {
                System.out.println("Template[" + i + "] = " + stNames[i]);

                System.out.println("Starting getting each search template and its criteria ...");
                stObj = schemaBean.getSearchTemplate(i);
                System.out.println("templateName = " + stObj.getName());

            }

            System.out.println("");

            boolean runQry = false;
            String stName = "";
            while (!runQry) {
                stName = ui.getString("Please enter the search template name:");
                stObj = schemaBean.getSearchTemplate(stName);
                if (stObj == null)
                    System.out.println("Unknown template name, please try again");
                else
                    runQry = true;
            }

            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i = 0; i < critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
            }

            runQry = false;
            while (!runQry) {
                System.out.println("Selections:");
                System.out.println("1. Specify a search criterion");
                System.out.println("2. Run query ...");
                switch (ui.getInteger("Enter your selection:")) {
                    case 2 :
                        runQry = true;
                        break;
                    case 1 :
                        getSearchCriterion(stObj);
                        break;
                }
            }

            System.out.println("");

            stObj.addCMBSearchReplyListener(resultBean);
            stObj.setTimeout(_templateTimeOut);
            stObj.setAsynchSearch(true); //Asynchronous search
            stObj.addCMBSearchReplyListener(resultBean);
            stObj.runQuery();
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    private void getSearchCriterion(CMBSearchTemplate stObj) {
        CMBSTCriterion critObj = null;
        try {
            String critName = ui.getString("Please enter criterion name: ");
            Vector critList = stObj.getSearchCriteria();
            for (int i = 0; i < critList.size(); i++) {
                critObj = (CMBSTCriterion) critList.elementAt(i);
                if (critObj.getName().equals(critName))
                    break;
            }

            if (critObj == null) {
                System.err.println("Invalid criterion name, please try again...");
                return;
            }

            if (critObj.getType() == CMBSTCriterion.CMB_CRIT_TYPE_PARAMETRIC) {
                int critOp =
                    ui.getInteger(
                        "Please enter search operator: "
                            + "\nEQ: 1\t\tNEQ: 2\t\tLEQ: 3\t\tLT: 4"
                            + "\nGEQ: 5\t\tGT: 6\t\tBETWEEN: 7\tNOTBETWEEN: 8"
                            + "\nIN: 9\t\tNOTIN: 10\tLIKE: 11\tNOTLIKE: 12\n");
                switch (critOp) {
                    case 1 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_EQUAL);
                        break;
                    case 2 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_EQUAL);
                        break;
                    case 3 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_LESS_EQUAL);
                        break;
                    case 4 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_LESS);
                        break;
                    case 5 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER_EQUAL);
                        break;
                    case 6 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER);
                        break;
                    case 7 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_BETWEEN);
                        break;
                    case 8 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_BETWEEN);
                        break;
                    case 9 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_IN);
                        break;
                    case 10 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_IN);
                        break;
                    case 11 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_LIKE);
                        break;
                    case 12 :
                        critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_LIKE);
                        break;
                    default :
                        break;
                }
                if (critOp == 7 || critOp == 8) {
                    String critVal1 = ui.getString("Please enter 1st search value: ");
                    String critVal2 = ui.getString("Please enter 2nd search value: ");
                    String[] critVals = new String[2];
                    critVals[0] = critVal1;
                    critVals[1] = critVal2;
                    critObj.setValues(critVals);
                } else if (critOp == 9 || critOp == 10) {
                    Vector vals = new Vector();
                    boolean notDone = true;
                    while (notDone) {
                        String val = ui.getString("Please enter value: (-1 if done entering)");
                        if (val.equals("-1"))
                            notDone = false;
                        else
                            vals.addElement(val);
                    }
                    String[] critVals = new String[vals.size()];
                    vals.copyInto(critVals);
                    critObj.setValues(critVals);
                } else {
                    String critVal = ui.getString("Please enter search value: ");
                    critObj.setValue(critVal);
                }
            } else {
                String critVal = ui.getString("Please enter text search value: ");
                critObj.setValue(critVal);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    void load() {
        schemaBean = (CMBSchemaManagement) per.load();
    }

    void save() {
        per.save(schemaBean);
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoDataMgt";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoConnect connDemo = new DemoConnect();
        DemoAsynchSearch demo = new DemoAsynchSearch();
        boolean loopControl = true;

        System.out.println("Going to connect to the server...");

        connDemo.connBean.addCMBConnectionReplyListener(demo.schemaBean);
        connDemo.connBean.addCMBConnectionReplyListener(demo.resultBean);
        connDemo.connect();

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. Re-load the CMBSchemaManagement bean from a file");
            System.out.println("2. Change property values");
            System.out.println("3. Run the bean");
            System.out.println("4. Save this CMBSchemaManagement bean to a file");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    demo.load();
                    break;
                case 2 :
                    demo.changeProperty();
                    break;
                case 3 :
                    demo.run();
                    break;
                case 4 :
                    demo.save();
                    break;
            }
        }

        connDemo.disconnect();
        System.out.println("Bye.");
    }
}