/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// Class Name  = SampleGuiSchema
// Description = Sample visual schema management bean
// Author      = afic
// Date        = 04/01/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.beans.*;
import java.io.*;
import java.util.Vector;

/*
import  javax.swing.*;
import  javax.swing.event.*;
import  javax.swing.border.*;
*/

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

//*****************************************************************************
// class/interface definition
//*****************************************************************************
/**
 * Sample GUI bean to prompt for user ID and password. Can be used with CM
 * connection bean to perform the functions.
 *
 * <pre>
 *   standalone properties  cache, traceEnabled.
 *   source of events       CMBSchemaRequestEvent     - to list catalog info
 * </pre>
 */
public class SampleGuiSchema
    implements Serializable, CMBSchemaReplyListener, CMBConnectionReplyListener, CMBUtilConstant {

    //*****************************************************************************
    // constructor
    //*****************************************************************************
    /**
     * Default constructor.
     */
    public SampleGuiSchema() {
        listenerVector = new Vector();
        initGUI();
    }

    /**
     * Destructor,  dispose the window.
     */
    public void finalize() {
        frame.dispose();
    }

    public void onCMBSchemaReply(CMBSchemaReplyEvent evt) {
        int id = evt.getID();
        Object data = evt.getData();
        Object src = evt.getSource();

        schemaBean = (CMBSchemaManagement) src;

        if (evt.getStatus() == CMBBaseConstant.CMB_STATUS_FAILED) {
            Exception ex = (Exception) data;
            JOptionPane.showMessageDialog(
                frame,
                ex.toString(),
                "Error: Exception in CMBSchemaManagement bean",
                JOptionPane.ERROR_MESSAGE);
            return;
        }

        //remove all items from dataList
        dataList.removeAll();

        switch (id) {
            case CMBSchemaReplyEvent.CMB_REPLY_SEARCH_TEMPLATE_NAMES :
                attrButton.setEnabled(false);
                isTemplate = true;
                break;
            case CMBSchemaReplyEvent.CMB_REPLY_ENTITY_ATTR_NAMES :
                attrButton.setEnabled(false);
                isTemplate = false;
                break;
            case CMBSchemaReplyEvent.CMB_REPLY_ENTITY_NAMES :
                attrButton.setEnabled(true);
                isTemplate = false;
                break;
        }

        dataList.setListData((String[]) data);
        scrollPane.doLayout();
        scrollPane.repaint();
        frame.repaint();
    }

    /**
     *  Listens to CMBConnectionReplyEvent, if reply status is ok, show the
     *  the window.
     *  @param evt CMBConnectionReplyEvent object
     */
    public void onCMBConnectionReply(CMBConnectionReplyEvent evt) {
        int status = evt.getStatus();
        if (status == CMBBaseConstant.CMB_STATUS_OK) {
            frame.setVisible(true);
            frame.requestFocus();
        }
    }

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // CMBSchemaEvent
    //*****************************************************************************
    /**
     * CMBSchemaEvent registration method.
     * It ignores a registration request if the listener is already registered.
     * @param listener listener to be registered
     */
    public void addCMBSchemaRequestListener(CMBSchemaRequestListener listener) {
        if (!listenerVector.contains(listener))
            listenerVector.addElement(listener);
    }

    /**
     * CMBSchemaRequestEvent unregistration method.
     * @param listener listener to be unregistered
     */
    public void removeCMBSchemaRequestListener(CMBSchemaRequestListener listener) {
        if (listenerVector.contains(listener))
            listenerVector.removeElement(listener);
    }

    //*****************************************************************************
    // TemplateChangeEvent
    //*****************************************************************************
    /**
     * TemplateChangeEvent registration method.
     * It ignores a registration request if the listener is already registered.
     * @param listener listener to be registered
     */
    public void addTemplateChangeListener(TemplateChangeListener listener) {
        System.out.println("addTemplateChangeListener:  adding new listener");
        if (!stChgListenerVector.contains(listener))
            stChgListenerVector.addElement(listener);
    }

    /**
     * TemplateChangeEvent unregistration method.
     * @param listener listener to be unregistered
     */
    public void removeTemplateChangeListener(TemplateChangeListener listener) {
        if (stChgListenerVector.contains(listener))
            stChgListenerVector.removeElement(listener);
    }

    /**
     *	Get the template change event object.
     *  @return TemplateChangeEvent object
     */
    public TemplateChangeEvent getTemplateChangeEvent() {
        return _templateChangeEvent;
    }

    /**
     *	Get the schema request event object.
     *  @return CMBSchemaRequestEvent object
     */
    public CMBSchemaRequestEvent getSchemaRequestEvent() {
        return _schemaRequestEvent;
    }

    /*
     * CMBSchemaRequestEvent notification method.
     * The event firing order is based on the order of registration.
     * @param evt the CMBSchemaRequestEvent
     */
    private void postCMBSchemaRequest(CMBSchemaRequestEvent evt) {
        _schemaRequestEvent = evt;

        Vector v;
        synchronized (this) {
            v = (Vector) listenerVector.clone();
        }

        int cnt = v.size();
        CMBSchemaRequestListener listener = null;

        for (int i = 0; i < cnt; ++i) {
            listener = (CMBSchemaRequestListener) v.elementAt(i);
            listener.onCMBSchemaRequest(evt);
        }
    }

    /*
     * CMBSchemaRequestEvent notification method.
     * @param id         the event type id
     */
    private void postCMBSchemaRequest(int id) {
        switch (id) {
            case CMBSchemaRequestEvent.CMB_REQUEST_SEARCH_TEMPLATE_NAMES :
            case CMBSchemaRequestEvent.CMB_REQUEST_ENTITY_NAMES :
                postCMBSchemaRequest(new CMBSchemaRequestEvent(this, id));
                break;
            case CMBSchemaRequestEvent.CMB_REQUEST_ENTITY_ATTR_NAMES :
                String entityName = (String) dataList.getSelectedValue();
                String[] args = new String[1];
                args[0] = entityName;
                postCMBSchemaRequest(new CMBSchemaRequestEvent(this, id, args));
                break;
            default :
                break;
        }

    }

    //*****************************************************************************
    // property accessors
    //*****************************************************************************
    /**
     * Property getter.
     * @return  window title
     */
    public String getWindowTitle() {
        return frame.getTitle();
    }

    /**
     * Property setter.
     * @param   newValue new window title
     */
    public void setWindowTitle(String newValue) {
        if (newValue != null)
            frame.setTitle(newValue);
    }

    /**
    * Property getter.
    * @return  connect button label
    */
    public String getEntityLabel() {
        return entityButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new connect button label
    */
    public void setEntityLabel(String newValue) {
        if (newValue != null)
            entityButton.setText(newValue);
    }

    /**
    * Property getter.
    * @return  attribute button label
    */
    public String getAttrLabel() {
        return attrButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new attribute button label
    */
    public void setAttrLabel(String newValue) {
        if (newValue != null)
            attrButton.setText(newValue);
    }

    /**
     * Property getter.
     * @return  search template button label
     */
    public String getTemplateLabel() {
        return templateButton.getText();
    }

    /**
     * Property setter.
     * @param   newValue new search template button label
     */
    public void setTemplateLabel(String newValue) {
        if (newValue != null)
            templateButton.setText(newValue);
    }

    public boolean isVisible() {
        return frame.isVisible();
    }

    //*****************************************************************************
    // Private methods
    //*****************************************************************************
    // Serialize transient data
    //*****************************************************************************
    /**
     * @serialData the data is serialized in the following sequence:
     * <pre>
     *    this object
     *    event listeners if the event listener is serializable
     *    a null object   tells the end of serialization
     * </pre>
     */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.defaultWriteObject();

        Vector v = null;
        synchronized (this) {
            if (listenerVector.size() > 0)
                v = (Vector) listenerVector.clone();
        }

        if (v != null) {
            int cnt = v.size();
            CMBSchemaReplyListener listener = null;
            for (int i = 0; i < cnt; ++i) {
                listener = (CMBSchemaReplyListener) v.elementAt(i);
                if (listener instanceof Serializable)
                    stream.writeObject(listener);
            }
        }

        stream.writeObject(null); // tells the end of listener
    }

    //*****************************************************************************
    // De-serialize/initialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is deserialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners may be none
    *    a null object   the marker of serialization
    * </pre>
    */
    private void readObject(ObjectInputStream stream) throws Exception {
        listenerVector = new Vector(); // always initialize listener list
        stream.defaultReadObject();

        Object o;
        while ((o = stream.readObject()) != null)
            listenerVector.addElement((CMBSchemaReplyListener) o);
        frame.setVisible(true);
    }

    private void fireTemplateChangeEvent(String templateName) {
        System.out.println("before firing event: templateName = " + templateName);
        _templateChangeEvent =
            new TemplateChangeEvent(this, TemplateChangeEvent.SAMPLE_TEMPLATE_SELECTED, templateName, schemaBean);
        Vector v;
        synchronized (this) {
            v = (Vector) stChgListenerVector.clone();
        }

        int cnt = v.size();
        TemplateChangeListener listener = null;
        for (int i = 0; i < cnt; ++i) {
            listener = (TemplateChangeListener) v.elementAt(i);
            listener.onTemplateChange(_templateChangeEvent);
        }
    }

    //*****************************************************************************
    // initGUI
    //*****************************************************************************
    private void initGUI() {

        frame = new JFrame("IBM Enterprise Information Portal Sample Application");
        mainPane = new CMBBasePanel();

        Border raisedBorder = new SoftBevelBorder(BevelBorder.RAISED);
        Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

        //Menu bar
        fileMenu.add(exitMenu);
        helpMenu.add(aboutMenu);
        menuBar.add(fileMenu);
        menuBar.add(helpMenu);

        frame.setJMenuBar(menuBar);

        //*****************************************************************************
        // Widgets creation and event listener setup
        //*****************************************************************************
        // Center panel
        //*****************************************************************************

        dataList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        dataList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                if (isTemplate) {
                    fireTemplateChangeEvent((String) dataList.getSelectedValue());
                }
            }
        });
        scrollPane = new JScrollPane(dataList);

        //*****************************************************************************
        // Bottom panel
        //*****************************************************************************
        templateButton = new JButton("Template");
        entityButton = new JButton("Entity");
        attrButton = new JButton("Attribute") {
            public Insets getInsets() {
                return new Insets(6, 12, 6, 12); // make it spacy
            }
        };

        templateButton.setActionCommand(_cmdTemplate);
        entityButton.setActionCommand(_cmdEntity);
        attrButton.setActionCommand(_cmdAttr);

        CMBActionAdapter al = new CMBActionAdapter() {
            public void actionPerformed(ActionEvent evt) {
                String cmd = evt.getActionCommand();

                if (cmd.equals(_cmdTemplate))
                    postCMBSchemaRequest(CMBSchemaRequestEvent.CMB_REQUEST_SEARCH_TEMPLATE_NAMES);

                else if (cmd.equals(_cmdEntity))
                    postCMBSchemaRequest(CMBSchemaRequestEvent.CMB_REQUEST_ENTITY_NAMES);

                else if (cmd.equals(_cmdAttr))
                    postCMBSchemaRequest(CMBSchemaRequestEvent.CMB_REQUEST_ENTITY_ATTR_NAMES);
                evt = null;
            }
        };

        templateButton.addActionListener(al);
        attrButton.addActionListener(al);
        entityButton.addActionListener(al);

        templateButton.setBorder(raisedBorder);
        attrButton.setBorder(raisedBorder);
        entityButton.setBorder(raisedBorder);

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new GridLayout(1, 3, 4, 0));
        buttonPane.add(templateButton);
        buttonPane.add(entityButton);
        buttonPane.add(attrButton);
        attrButton.setEnabled(false);
        _setDefaultEvents();

        //*****************************************************************************
        // Add widgets to window frame
        //*****************************************************************************
        mainPane.addCenterPanel(scrollPane);
        mainPane.addBottomPanel(buttonPane);
        frame.getContentPane().add(mainPane, BorderLayout.CENTER);
        frame.pack();
        CMBUtil.setLocation(frame, CMBUtil.CENTER);
        frame.setVisible(true);
        frame.requestFocus();
    }

    private void _setDefaultEvents() {
        exitMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                frame.setVisible(false);
            }
        });

        helpMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                String msg =
                    "Welcome to IBM Enterprise Information Portal Sample Application\n\n"
                        + "IBM Enterprise Information Portal for Multiplatforms V6.1 (program number 5648-D03)\n"
                        + "(c ) Copyright IBM Corp. 1998, 1999.\n"
                        + "IBM Content Manager for Multiplatforms V6.1 (program number 5648-D04)\n"
                        + "(c ) Copyright IBM Corp. 1994, 1999.";
                JOptionPane.showMessageDialog(
                    frame,
                    msg,
                    "About: IBM Enterprise Information Portal Sample Application",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        });
    }

    //*****************************************************************************
    // private data
    //*****************************************************************************
    private final String _cmdEntity = "entity";
    private final String _cmdAttr = "attribute";
    private final String _cmdTemplate = "template";
    private boolean isTemplate = false;

    //private VetoableChangeSupport vcSupport   = new VetoableChangeSupport(this);
    //private PropertyChangeSupport pcSupport   = new PropertyChangeSupport(this);

    //*****************************************************************************
    // GUI widgets, all labels are editable properties
    //*****************************************************************************
    private JButton entityButton = null;
    private JButton attrButton = null;
    private JButton templateButton = null;

    private JFrame frame = null;
    private CMBBasePanel mainPane = null;
    private JScrollPane scrollPane = null;
    private JList dataList = new JList();
    private JMenuBar menuBar = new JMenuBar();
    private JMenu fileMenu = new JMenu("File");
    private JMenu helpMenu = new JMenu("Help");
    private JMenu viewMenu = new JMenu("View");

    private JMenuItem exitMenu = new JMenuItem("Exit");
    private JMenuItem aboutMenu = new JMenuItem("About Driver2");

    //*****************************************************************************
    // Transient data members
    //*****************************************************************************
    private transient Vector listenerVector = new Vector();
    private transient Vector stChgListenerVector = new Vector();

    private CMBSchemaRequestEvent _schemaRequestEvent = null;
    private TemplateChangeEvent _templateChangeEvent = null;

    private CMBSchemaManagement schemaBean = null;
    //*****************************************************************************
    // Copyright
    //*****************************************************************************
    private static final String ibmid = "5648-D03 and 5648-D04  " + "(C) Copyright IBM Corp. 1994,1999";

}