/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import java.util.*;

/**
 * Performs an asynchronous search.  This sample is very similar to TSearch (and uses
 * the search method of TSearch), but provides an event handler that is called
 * on a secondary thread as search results are returned.
 * <p>
 * Note: Not all content servers support asynchronous search.  For those that do not,
 * asynchronous search is simulated, firing the event once when the search has
 * completed.
 */
public class TAsynchSearch {

	static Object searchMonitor = new Object();

	public static void main(String[] args) throws Exception {
        if (args.length < 5) {
			System.out.print(
				"Performs an asynchronous search. \n"
					+ "Syntax: \n"
                    + "  java TAsynchSearch <dstype> <server> <userid> <password> <entity> <condition> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
					+ "Examples: \n"
                    + "  java TAsynchSearch ICM icmnlsdb icmadmin password Book @Title LIKE \\\"%\\\" \n"
                    + "  java TAsynchSearch DL libsrvrn frnadmin password NOINDEX Userid LIKE '%' \n"
                    + "  java TAsynchSearch Fed cmbdb cmbadmin password NoIndex Userid LIKE '%' \n"
                    + "  java TAsynchSearch DB2 sample db2admin password EMP_PHOTO PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TAsynchSearch OD odserver userid password Balance CustName like '%BLONDIE%'");
			return;
        }
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        for (int i = 5; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

        // Set search reply event listener.  This listener will be
        // called as search results are available.
        queryService.addCMBSearchReplyListener(new SearchReplyListener(queryService, searchResults));
        
        // Construct query string and perform the search
		TSearch.search(queryService, dstype, entity, condition, true);
		synchronized (searchMonitor) {
			searchMonitor.wait();
		}

        System.out.println("There are " + searchResults.getCount() + " items found.");
        for (int i = 0; i < searchResults.getCount(); i++) {
            CMBItem item = searchResults.getItem(i);
            System.out.println("  Item[" + i + "] = " + item.getName());
            TSearch.printItemDetails(dataManagement, item, 1, true);
        }

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

        System.exit(0);
    }
    
    static class SearchReplyListener implements CMBSearchReplyListener
    {
    	CMBQueryService queryService;
    	CMBSearchResults searchResults;
    	
    	public SearchReplyListener(CMBQueryService queryService, CMBSearchResults searchResults) {
    		this.queryService = queryService;
    		this.searchResults = searchResults;
    	}
    	
    	/**
		 * @see CMBSearchReplyListener#onCMBSearchReply(CMBSearchReplyEvent)
		 */
		public void onCMBSearchReply(CMBSearchReplyEvent evt) {
			try {
				int hits = 0;
				Object data = evt.getData();
				if (data instanceof CMBResultData) {
					data = ((CMBResultData)data).getResultObject();
				}
				if (data instanceof Vector) {
					hits = ((Vector)data).size();
				}
				if (evt.getStatus() == CMBBaseConstant.CMB_STATUS_RESULT_NEW) {
					System.out.println("--> Callback: New  - "+hits+" hits");
					searchResults.newResults(queryService.getResults());
				} else if (evt.getStatus() == CMBBaseConstant.CMB_STATUS_RESULT_MORE) {
					System.out.println("--> Callback: More - "+hits+" hits");
					searchResults.appendResults(queryService.getResults());
				} else if (evt.getStatus() == CMBBaseConstant.CMB_STATUS_RESULT_END) {
					System.out.println("--> Callback: End  - "+hits+" hits");
					searchResults.appendResults(queryService.getResults());
					synchronized (searchMonitor) {
						searchMonitor.notify();
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
    }

}
