/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Lists search templates and their criteria for a server.  Search templates
 * are available for federated and OnDemand servers:
 * <ul>
 * <li>Fed - these are known as federated search templates, defined by the administrator.
 * <li>OD - these are known as Folders, also defined by the administrator.
 * </ul>
 */
public class TListTemplates {

    public static void main(String[] args) throws Exception {
        if (args.length < 4) {
            System.out.println(
                "Displays details on search templates defined on the server. \n"
                    + "Syntax: \n"
                    + "  java TListTemplates <dstype> <server> <userid> <password> [<template>] \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, OD) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <template> optional, the name of a particular template.  Detailed \n"
                    + "           information will be displayed about the template. \n"
                    + "Examples: \n"
                    + "  java TListTemplates Fed cmbdb cmbadmin password \n"
                    + "  java TListTemplates OD odserver userid password");
        	System.exit(0);
        }
        
    	String dstype = args[0];
    	String server = args[1];
    	String userid = args[2];
    	String password = args[3];
        String template = (args.length > 4) ? args[4] : null;

        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        TConnect.connect(connection, dstype, server, userid, password);
        if (template == null) {
            listTemplates(schemaManagement);
        } else {
            printTemplateDetails(schemaManagement.getSearchTemplate(template), true);
        }
        TConnect.disconnect(connection);
    }

    /**
     * Lists the search templates defined on the server.
     * @param schemaManagement an instance of CMBSchemaManagement bean
     */
    public static void listTemplates(CMBSchemaManagement schemaManagement) throws CMBException {
        CMBSearchTemplate[] templates;
        try {
            templates = schemaManagement.getSearchTemplates();
        } catch (CMBException e) {
            System.out.println("Search templates are not supported by this server.");
            return;
        }
        System.out.println("Templates:");
        for (int i = 0; i < templates.length; i++) {
            System.out.println("  Template[" + i + "] = " + templates[i].getName());
            printTemplateDetails(templates[i],false);
        }
    }

    /**
     * Prints the criteria of a search template.
     * @param template an instance of CMBSearchTemplate for the template
     * @param withDetails if true, additional criteria detail is written.
     */
    public static void printTemplateDetails(CMBSearchTemplate template, boolean withDetails) throws CMBException {
        System.out.println("  " + template.getName());
        CMBSTCriterion[] criteria = template.getSearchCriterion();
        for (int i = 0; i < criteria.length; i++) {
        	CMBSTCriterion criterion = criteria[i];
            System.out.println("    Criteria[" + i + "] = " + criterion.getName());
            if (withDetails) {
	            System.out.println("      template name: " + criterion.getTemplateName());
	            System.out.println("      entity name: " + criterion.getEntityName());
	            System.out.println("      attribute name: " + criterion.getAttrName());
	            System.out.println("      attribute type: " + criterion.getAttrType());
	            System.out.println("      criteria position: " + criterion.getCriteriaPosition());
	            System.out.print("      valid operators:");
	            short[] validOperators = criterion.getValidOperators();
	            for (int j = 0; j < validOperators.length; j++) {
	            	System.out.print(" ["+getOperatorName(validOperators[j])+"]");
	            }
	            System.out.println();
	            System.out.println("      default operator: [" + getOperatorName(criterion.getDefaultOperator()) + "]");
	            System.out.print("      predefined values:");
	            String[] predefinedValues = criterion.getPredefinedValues();
	            if (predefinedValues != null) {
		            for (int j = 0; j < predefinedValues.length; j++) {
		            	System.out.print(" ["+predefinedValues[j]+"]");
		            }
	            }
	            System.out.println();
	            System.out.print("      default value:");
	            String[] defaultValues = criterion.getDefaultValue();
	            if (defaultValues != null) {
	            	for (int j = 0; j < defaultValues.length; j++) {
	            		System.out.print(" ["+defaultValues[j]+"]");
	            	}
	            }
	            System.out.println();
	            System.out.println("      display name: " + criterion.getDisplayName());
	            System.out.println("      display position: " + criterion.getDisplayPosition());
	            System.out.println("      display width: " + criterion.getDisplayWidth());
	            System.out.println("      type: " + criterion.getType());
            }
        }
    }
    
    /**
     * Returns the name of the operator.
     */
    public static String getOperatorName(short operator) {
    	switch (operator) {
    		case CMBBaseConstant.CMB_OP_UNDEFINED:		return "Undefined";
    		case CMBBaseConstant.CMB_OP_EQUAL:			return "=";
    		case CMBBaseConstant.CMB_OP_NOT_EQUAL:		return "!=";
    		case CMBBaseConstant.CMB_OP_LESS_EQUAL:		return "<=";
    		case CMBBaseConstant.CMB_OP_LESS:			return "<";
    		case CMBBaseConstant.CMB_OP_GREATER_EQUAL:	return ">=";
    		case CMBBaseConstant.CMB_OP_GREATER:		return ">";
    		case CMBBaseConstant.CMB_OP_BETWEEN:		return "between";
    		case CMBBaseConstant.CMB_OP_NOT_BETWEEN:	return "not between";
    		case CMBBaseConstant.CMB_OP_IN:				return "in";
    		case CMBBaseConstant.CMB_OP_NOT_IN:			return "not in";
    		case CMBBaseConstant.CMB_OP_LIKE:			return "like";
    		case CMBBaseConstant.CMB_OP_NOT_LIKE:		return "not like";
    		case CMBBaseConstant.CMB_OP_CONTAINS_TEXT:	return "contains text";
    		default:	return "Unknown";
    	}
    }
    
}

