//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


/*
 * Filename       : TCallbackDES.java
 * Description    : Server Definition class for DES
 * Change History :
 *
 * Date        Name  Description
 * -------    -----  ---------------------------
 *
 */

import com.ibm.mm.sdk.common.*;

import java.util.*;
import java.lang.*;

/**
 TCallbackDES : This class represents a server definition.
 */

public class TCallbackDES
	implements dkCallback, DKConstantDES, DKMessageIdDES, java.io.Serializable
{

  // constructor
 /**
  *  Constructs an callback for DES
  */
  public TCallbackDES()
  {
   _threshold = 5;
  }

 /**
  *  Sets the number of returned hits for each data callback.
  *  This method should be set by applications for the datastore.
  *  @param 	nValue the number of hits in the result collection
  *  		for each callback.
  */
 public void setThreshold(int nValue)
 {
  _threshold = nValue;
 }

 /**
  *  Gets the number of returned hits that the server return
  *  for each callback.  The datastore will call this method for
  *  each query.
  *  @return	the threshold value,
  */
 public int getThreshold()
 {
  return _threshold;
 }

 /**
  *  Query done check
  *  @return	true when done
  */
 public boolean isDone()
 {
  return _isDone;
 }

 /**
  *  Indicates that the user has canceled the query.
  *  It should be called by the application to let the server
  *  knows to stop the query.
  */
 public void cancelQuery()
 {
  _cancel = true;
 }

 /**
  *  Returns nHits from the backend datastore
  *  @param 	ddoList a collection of DKDDO objects
  *  @param 	isDone  a boolean value indicating whether the server is
  *  			done with the query.  That is, there is more hits.
  *  @return	true if query is done or user has canceled the query.
  */
 public boolean dataCallback(dkCollection ddoList, boolean isDone)
 {
  boolean fDone = false;
  _numCalled++;
  if (_cancel == true || isDone == true)
  {
   fDone = true;
   _isDone = true;
  }
  else
  {
   System.out.println("*** " + _numCalled + " time called ***");
   try
   {
   processResults(ddoList);
   }
   catch (DKException exc)
   {
    System.out.println("Exception name " + exc.name());
    System.out.println("Exception message " + exc.getMessage());
    System.out.println("Exception error code " + exc.errorCode());
    System.out.println("Exception error state " + exc.errorState());
    exc.printStackTrace();
   }
   catch (Exception exc)
   {
    System.out.println("Exception message " + exc.getMessage());
    exc.printStackTrace();
   }
  }
  return fDone;
 }

 /**
  *  Report an exception from the backend datastore
  *  @param 	exc an exception from the backend datastore
  */
 public void reportException(DKException exc)
 {
  _exc = exc;
 }

 /**
  *  Get an exception from the backend datastore
  *  @return an exception from the backend datastore
  */
 public DKException getException()
 {
  return _exc;
 }

 /**
  *  Clear exception object
  */
 public void resetException()
 {
   _exc = null;
 }

  public void processResults(dkCollection results)
                throws DKException, Exception
  {
    Short    sVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println( "number of items " + results.cardinality());

    dkIterator iter = results.createIterator();

    while(iter.more()) {
       item = (DKDDO) iter.next();
       if (item != null) {
//   	  item.retrieve();    // document was already retrieved by the execute
   	  sVal = (Short)item.getPropertyByName(DK_PROPERTY_ITEM_TYPE);
   	  itemType = sVal.shortValue();
   	  switch(itemType) {
   	    case DK_DOCUMENT:
   	      System.out.println(" Item is a document, number of data items : " + item.dataCount());
   	      for (i = 1; i <= item.dataCount(); i++) {
   		   dataName = item.getDataName(i);
   		   System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   	      }
   	      break;
   	    default:
   	      System.out.println("  Item is not recognized ");
   	      break;
   	  }
       }
    }
  }

 private boolean _cancel = false;
 private boolean _isDone = false;
 private int _threshold;
 private int _numCalled = 0;
 private DKException _exc = null;
}
