/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 * Filename       : LoadFolderTSQBICDL.java
 *
 * Description    : This loader is a sample of using java class library's API
 * to create folders and load sample data into the Digital Library with parts
 * to be indexed by Text Search  and QBIC.  You should run it with the provided 
 * input data file "folderQBIC.dat". The physical files for this loader are 
 * packaged in "samples.jar".  You should use command "jar -xvf samples.jar"
 * to unpack it.	
 * 
 * To run it as default, this loader assumes following are already defined: 
 * 
 *    Index Class Name - "FOLDERDOC" (defined with the key fields in step 4)
 *    Text search server(search service name) - "TM"
 *    Text search index - "TMINDEX"
 *    QBIC DBName(database name) - "SAMPLEDB"
 *    QBIC Catalog - "SAMPLECAT"
 *    QBIC Host Alias of QBIC server - "qbicsrv"
 * 
 * Invoke Syntax:
 *
 *    java LoadFolderTSQBICDL <inputDataFile> <outputLog> <userid> <password> <libraryServer>
 *
 *    where:  
 *      <inputDataFile>: a input control file, see folder.dat for example. 
 *      <outputLog>: the log file to store the process results, it can be used
 *                   for debug purpose, you can delete it later.  
 *      <userid>: the userid you use to logon the library server.
 *      <password>: the password to logon the library server.
 *      <libraryServer>: the library server name
 * 
 * ### NOTE: to compile  ###
 *    NT:
 *      Edit the x:\cmbroot\cmbenv81.bat file to set the environment variables.
 *      for example:  set CMBROOT=e:\cmbroot
 *                    set DB2HOME=e:\sqllib
 *      under x:\cmbroot directory enter: cmbenv81   
 *
 *      under x:\cmbroot\samples\java\dl directory enter following to compile:
 *            javac LoadFolderTSQBICDL.java
 *
 *      The LoadFolderTSQBICDL.class will be created if compiled successfully.     
 *
 *    AIX:
 *      Make sure the CLASSPATH environment variable has the following values:
 *            /usr/lpp/cmb/lib/cmbcm81.jar
 *            /usr/lpp/cmb/lib/cmbdl81.jar
 *      under /usr/lpp/cmb/samples/java/dl enter following to compile:
 *            javac LoadFolderTSQBICDL.java 
 *
 *      The LoadFolderTSQBICDL.class will be created if compiled successfully.      
 *
 ******************************************************************************
 * Following is the instruction to customize the loader to run your own data 
 ******************************************************************************
 * This loader can be modified to work with any index class defined with		
 * different attribute types(key fields).  Rules to follow:
 *
 *  1. The attribute name in the inputDataFile(ie. folderQBIC.dat) should
 *     exactly match with the index class's attribute name(case sensitive). 
 *     for example: 
 *
 *     index class attribute names     in folderQBIC.dat file
 *     ---------------------------   ----------------------
 *      DLSEARCH_Author              DLSEARCH_Author=Janet Hedges
 *      DLSEARCH_Publisher           DLSEARCH_Publisher=IBM
 *
 *     index class with your           your <inputDataFile> 
 *     defined attribute names         (ie. mine.dat)
 *     ---------------------------   ----------------------
 *      Department                   Department=U6M
 *      Manager                      Manager=John Smith
 *
 *  2. In your inputDataFile, the prefix for the parts to be added should be 
 *     "xdo", "smi", "html", "thmb" or "qbic".  The "xdo" is for a regular part,
 *     "smi" is for the part to be indexed by Text Search, "html" is for the 
 *     html part, "thmb" is for the thumbnail part and "qbic" is for the part
 *     to be indexed by the QBIC. 
 *
 *  3. In inputDataFile each folder should start with "item= " followed with
 *     "Folder= ", and each document should start with "item= " followed with
 *     "Document= ".  Each item is defined by beginning with "item= " and 
 *     ending with "end= ".  So you should make sure the pairs of "item= " and
 *     "end= " are correctly matched for each folder and each document. 
 *     Please use "EndOfInput" to indicate the end of inputDataFile.
 *
 *
 *  4. Examples of index class's attributes and folderQBIC.dat
 *
 *     index class attr. name     character length min - max
 *     ----------------------    ----------------------------
 *      DLSEARCH_Author            1 - 50
 *      DLSEARCH_Publisher         1 - 50
 *      DLSEARCH_DocType           1 - 32
 *      DLSEARCH_Date              1 - 50
 *      DLSEARCH_Title             1 - 100
 *      DLSEARCH_Subject           1 - 100
 *      DLSEARCH_Authors           1 - 100
 *      DLSEARCH_Editors           1 - 50
 *      DLSEARCH_Source            1 - 100
 *      DLSEARCH_Volume            1 - 20
 *      DLSEARCH_Issue             1 - 20
 *
 *    (all above key field names are defined with key field type as
 *     "Variable character" and character type as "Extended alphanumeric", the
 *     character length min - max are shown as above)
 *
 *     the folderQBIC.dat file: (shown as following)
 *    --------------------------------------------
 *
 *   ############ below is a folder(Vol11)##################################
 *   item=L1
 *   Folder=Vol11
 *   DLSEARCH_DocType=Vol11 Folder
 *   DLSEARCH_Subject=Vol11 contents
 *     ############# below is a document contained in folder(Vol11)#########
 *     item=L2-1
 *     Document=Vol11-1
 *     DLSEARCH_DocType=Vol11 Document1
 *     DLSEARCH_Author=Anonymous
 *     DLSEARCH_Subject=Contents
 *     DLSEARCH_Publisher=IBM
 *     DLSEARCH_Date=1994
 *     DLSEARCH_Source=Software Quarterly
 *     DLSEARCH_Title=Contents of Software Quarterly Volume 1 Issue 1
 *     DLSEARCH_Volume=1
 *     DLSEARCH_Issue=1
 *     html=vol11/contents.htm
 *     qbic=images/321sm.gif
 *     thmb=images/321smt.gif
 *     xdo=vol11/contents.txt
 *     xdo=images/sqcloud0.gif
 *     xdo=images/sqclouda.gif
 *     smi=vol11/contents.dat
 *     end= item L2-1
 *     ############# end of the document(Vol11-1) ##########################
 *     ############# below is another document contained in folder(Vol11)###
 *     item=L2-2
 *     Document=Vol11-2
 *     DLSEARCH_DocType=Vol11 Document2
 *     DLSEARCH_Author=Randall C. Kennedy
 *     DLSEARCH_Subject=32-bit operating systems
 *     DLSEARCH_Publisher=IBM
 *     DLSEARCH_Date=1995
 *     DLSEARCH_Source=Software Quarterly
 *     DLSEARCH_Title=Moving to a 32-Bit Operating System: Myth vs. Reality
 *     DLSEARCH_Volume=1
 *     DLSEARCH_Issue=1
 *     html=vol11/32-bit.htm
 *     xdo=vol11/32-bit.txt
 *     thmb=images/sqcdromt.gif
 *     qbic=images/sqcdrom.gif
 *     xdo=images/323.gif
 *     smi=vol11/32-bit.dat
 *     end= item L2-2
 *     ############# end of the document(Vol11-2) ##########################
 *   end=item L1 
 *   ############### end of the folder(Vol11) ##############################
 *   ############ below is a folder(Vol2) ##################################
 *   item=L1
 *   Folder=Vol2
 *   DLSEARCH_DocType=Vol2 Folder
 *   DLSEARCH_Subject=Vol2 contents
 *     ########## below is a folder(Vol21) contained in folder(Vol2) #######
 *     item=L21
 *     Folder=Vol21
 *     DLSEARCH_DocType=Vol21 Folder
 *     DLSEARCH_Subject=Vol21 contents
 *       ########### below is a document contained in folder(Vol21)#########
 *       item=L3-1
 *       Document=Vol21-1
 *       DLSEARCH_DocType=Vol21 Document1
 *       DLSEARCH_Author=Anonymous
 *       DLSEARCH_Publisher=IBM
 *       DLSEARCH_Date=1996
 *       DLSEARCH_Source=Software Quarterly
 *       DLSEARCH_Title=Contents of Software Quarterly Volume 2 Issue 1
 *       DLSEARCH_Volume=2
 *       DLSEARCH_Issue=1
 *       html=vol21/contents.htm
 *       xdo=vol21/contents.txt
 *       thmb=images/covert.gif
 *       xdo=images/sqhdr.gif
 *       qbic=images/cover.gif
 *       end=item L3-1
 *       ############# end of the document(Vol21-1) ########################
 *     end=item L21
 *     ############### end of the folder(Vol21) ############################
 *   end=item L1 
 *   ################# end of the folder(Vol2) #############################
 *   EndOfInput
 *   ################# end of the data input file ##########################
 *
 *   Note: about the prefix - smi, qbic, html, thmb
 *     smi - for the part to be indexed by the Text Search, ie. contents.dat
 *     qbic - for the part to be indexed by QBIC, ie. 321sm.gif
 *     html - for the html part, ie. contents.htm
 *     thmb - for the thumbnail part, ie. 321smt.gif
 *
 *  5. Invoke example: (in NT system, with files in the following directories)
 *     x:\cmbroot\samples\java\dl\LoadFolderTSQBICDL.java -- this program
 *     x:\cmbroot\samples\java\dl\folderQBIC.dat        -- inputDataFile
 *     x:\cmbroot\samples\java\dl\vol11\*.*        -- input text files are here
 *     x:\cmbroot\samples\java\dl\images\*.*       -- input image files are here
 *
 *     In x:\cmbroot\samples\java\dl] type as:
 *
 *     java LoadFolderTSQBICDL folderQBIC.dat folder.log frnadmin password LIBSRVRN 
 *
 *     (note: in AIX system path is /usr/lpp/cmb/samples/java/dl/  )
 *
 *  6. If physical files are in different directories then file path in
 *     folderQBIC.dat should be modified.
 *
 *  7. This loader requires the DL indexClass("FOLDERDOC"), Text
 *     search server("TM"), Text search index("TMINDEX"), QBIC 
 *     database("SAMPLEDB"), QBIC catalog("SAMPLECAT") and QBIC server("qbicsrv")
 *     have been defined first.  The "FOLDERDOC" has attribute names defined as 4.,
 *     and all the field types are "variable character".
 *
 *     If you have different index class name, different
 *     text search server(search service name) or text search index ,
 *     different database name, catalog name and host alias name for QBIC,
 *     please modify the following variables:
 *
 *     indexClassName = "FOLDERDOC",   //a DL index class name
 *     searchEngineTS = "SM",          //Text search engine name - SM 
 *     searchServerTS = "TM",          //Text search server name    
 *     searchIndexTS = "TMINDEX",      //Text search index name
 *     searchInfo = "ENU",             //ENU for english, FRA for french... 
 *     searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
 *     QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
 *     QBICCatalog = "SAMPLECAT",      //QBIC catalog name
 *     searchServerQBIC = "qbicsrv";   //QBIC server name
 *
 *  8. If you want to use this loader to load data without part to be indexed	
 *     by Text Search, please comment out the lines indicated by the 
 *     |**** no text search by comment out following line ****| &  
 *     |**** end of comment out for no text search ***********|
 *     In this case, you don't need to define the TM search index name. 
 *
 *     Also, if you want to use this loader to load data without part to be 
 *     indexed by QBIC, please comment out the lines indicated by the 
 *     |**** no image search by comment out following line ****| &  
 *     |**** end of comment out for no image search ***********|
 *     In this case, you don't need to define the QBIC search index name.  
 * 
 *****************************************************************************/ 
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;
import java.lang.*;
import java.util.*;

public class LoadFolderTSQBICDL implements DKConstantDL
{
  static String
      indexClassName = "FOLDERDOC",   //a DL index class name
      searchEngineTS = "SM",          //TextMiner engine name - SM 
      searchServerTS = "TM",          //TextMiner search server name    
      searchIndexTS = "TMINDEX",      //TextMiner search index name
      searchInfo = "ENU",             //ENU for english, FRA for french... 
      searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
      QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
      QBICCatalog = "SAMPLECAT",      //QBIC catalog name
      searchServerQBIC = "qbicsrv";   //QBIC server name

  static boolean  
         smiSrv = false,               //flag for calling wakeUpService 
         qbicSrv = false;              //flag for calling invokeSearchEngine

  static BufferedReader  in;
  static BufferedWriter out;

  static DKDatastoreDL dsDL;
  final static int  maxAttr     = 50;
  final static int  maxItem     = 50;
  final static int  maxXDO      = 50;
  static String[]  attrName     = new  String[maxAttr];
  static short[]   attrId       = new   short[maxAttr];
  static short[]   attrType     = new   short[maxAttr];
  static boolean[] attrNullable = new boolean[maxAttr];
  static short attrCount = 0;

  public static void main(String args[])
  {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //sample loader always loading English data samples
    Locale origLocale = Locale.getDefault();
    boolean reSet = false;
    if (!(Locale.getDefault().getCountry().equals("US")))
    { 
      reSet = true;
      Locale.setDefault(new Locale("en", "US"));
    }

    try
    {
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (args.length < 5)
      {
       System.out.println("Invoke Syntax: " );
       System.out.println("  java LoadFolderTSQBICDL <inputDataFile> <outputLogFile> " +
        "<userid> <password> <libraryServer>" );
       return;
      }
      
      File inputFile = new File(args[0]);
      File outputFile = new File(args[1]);
      String userid = args[2];
      String password = args[3];
      String libSrv = args[4];
      
      out   = new BufferedWriter(new FileWriter(outputFile));
       out.write("You enter: java LoadFolderTSQBICDL " +
       args[0] + " " + args[1] + " " + args[2] + " " + args[3] + " " + args[4]);
       out.newLine();
      
      if (inputFile.exists() == false) 
      {
       System.out.println("Input data file does not exist: " + args[0]);
       return;
      }

      int i = 0;

      //-------------------------------------------------------------- 
      // verify the TextMiner "TMINDEX" and QBIC dataBase & catalog 
      // are exist
      //-------------------------------------------------------------- 
      verifySetUp(userid, password);

      //-------------------------------------------------------------- 
      // connecting to data store
      //-------------------------------------------------------------- 
      dsDL = new DKDatastoreDL();
      System.out.println("Connecting Datastore, Please Wait!!!");
      dsDL.connect(libSrv, userid, password,"");
      System.out.println("Datastore Connected");
      System.out.println("Loading " + args[0] + ",  please wait.... ");
      Integer opt = new Integer(DK_FALSE);
      dsDL.setOption(DK_DL_OPT_WAKEUPSRV, (Object)opt); 
      in = new BufferedReader(new FileReader(inputFile));
      String line;

      prepareAttrList();

      do
      {
          line = in.readLine();
          if ((line.toLowerCase()).startsWith("item"))
          {
              out.write("******** Begin Loading folder: " + line + " **********");
              out.newLine(); 

              DKDDO ddo = new DKDDO((short)DK_CM_DDO_INITIAL_SIZE);
              load(ddo);

              out.write("******** Finish Loading folder: " + line + " *********");
              out.newLine();
          }
          else if (  (!(line.startsWith("#")) &&
                     (!((line.toLowerCase()).equals("endofinput"))))  )
          {
              System.out.println("Folder or Document should begin with line:  item =");
              return;
          }
      } while ( !((line.toLowerCase()).equals("endofinput")) );
      out.write("******** Input Read: " + line + " **********");
      out.newLine();
      in.close();
      if (smiSrv == true)
      {
        //*** TextMiner search service name-search index name for API call      
        String  searchIndexTM = searchServerTS + "-" + searchIndexTS;
/**** no text search by comment out following line ****/
        System.out.println("about to call invokeSearchEngine for TM..... ");
        out.write("about to call invokeSearchEngine(" + searchEngineTS + " ..)");
        out.newLine();
        dsDL.invokeSearchEngine(searchEngineTS, searchIndexTM);
        out.write("invokeSearchEngine(..) done. ");
        out.newLine();
        System.out.println("invokeSearchEngine(..) done. ");
/**** end of comment out for no text search ***********/ 
      }
      if (qbicSrv == true)
      {
        //*** QBIC DataBase name-Catalog name-server Name for API call      
        String  searchIndexQBIC = QBICDataBase + "-" + 
                                QBICCatalog + "-" + searchServerQBIC;
/**** no image search by comment out following line ****/
        opt = new Integer(DK_DL_SRCHINDEX_ONLINE);
        dsDL.setOption(DK_DL_OPT_SEARCHINDEX, (Object)opt);
        System.out.println("about to call invokeSearchEngine for QBIC..... ");
        out.write("about to call invokeSearchEngine(" + searchEngineQBIC + " ..)");
        out.newLine();
        dsDL.invokeSearchEngine(searchEngineQBIC, searchIndexQBIC);
        out.write("invokeSearchEngine() done. ");
        out.newLine();
        System.out.println("invokeSearchEngine(..) done. ");
/**** end of comment out for no image search ***********/
      }
      out.close();
      System.out.println("Finish loading " + args[0] );
      dsDL.disconnect();
      System.out.println("datastore disconnected. Done....");
    } // end of try
    catch (DKException exc)
    {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (FileNotFoundException exc)
    {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (IOException exc)
    {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }  
    if (reSet) Locale.setDefault(origLocale);  //re-set back to original Locale
  }

  //**********************************************************
  // load - load the DDO, a recursive routine
  //********************************************************** 
  public static void load(DKDDO ddo) 
      throws DKException, IOException, Exception
  {
    Short itemtype = new Short((short)0);
    DKDDO   item[] = new DKDDO[maxItem];
    DKBlobDL xdo[] = new DKBlobDL[maxXDO];
    String xdoFiles[] = new String[maxXDO];
    String attrValue[] = new  String[maxAttr];
    DKParts parts = null;
    DKFolder folder = null;
    DKPid pid = new DKPid();
    String line, name = "", value = "";
    int i = 0, n = 0;
    short linecount = 0, itemcount = 0, xdocount = 0;
    short dataId;
    Short Dtype = new Short(DK_CM_DOCUMENT);
    int   smiPartId = 1,                //partId for TM search indexed part
          notePartId = 2,               //partId for notepart
          htmlPartId = 3,               //partId for html part 
          qbicPartId = 4,               //partId for QBIC search indexed part
          thmbPartId = 5,               //partId for thumbnail part
          partId = 10,                  //partId for regular xdo part
          CClass;
    boolean folderDDO_exist = false,
            find;

    //### for generated notepart file name
    String noteName = "notepart.txt";

    //*** TextMiner search service name-search index name for API call      
    String  searchIndexTM = searchServerTS + "-" + searchIndexTS;

    //*** QBIC DataBase name-Catalog name-server Name for API call      
    String  searchIndexQBIC = QBICDataBase + "-" + 
                                QBICCatalog + "-" + searchServerQBIC;
    //-------------------------------------------------------------- 
    // setup the general property for the ddo & set data to null
    //-------------------------------------------------------------- 
    pid.setObjectType(indexClassName);
    ddo.setPidObject(pid);
    ddo.setDatastore(dsDL);
    for (i = 0; i < attrCount; i++)
    {
       dataId = ddo.addData(attrName[i]);
       ddo.addDataProperty(dataId, DK_CM_PROPERTY_ATTRIBUTE_ID, new Short(attrId[i]));
       ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, new Short(attrType[i]));
       ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, new Boolean(attrNullable[i]));
       ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT,
                           new Integer((short)DK_CM_ATTRIBUTE_READWRITE));
       ddo.setNull(dataId); 
    }
      
    do
    {
       line = in.readLine();
       if ((line != null) &&
           (!(line.trim().startsWith("#"))))		   
       {
          n = 0; 
          n = line.indexOf("=");
          if (n < 0 ) continue;
          linecount++;
          name = value = "";
          name  = (line.substring(0,  n)).trim();
          value = (line.substring(n + 1)).trim();        
          if (linecount == 1)
          {
             if ( (name.toLowerCase()).equals("folder") )
                  itemtype = new Short(DK_CM_FOLDER);
             else if ( (name.toLowerCase()).equals("document") )
                  itemtype = new Short(DK_CM_DOCUMENT);
             else System.out.println("Input Error: " +
                  "Second line must start with Folder or Document");
          }
          if (value.equals("")) continue;    
          if ((!((name.toLowerCase()).equals("xdo"))) &&
              (!((name.toLowerCase()).equals("smi"))) &&
              (!((name.toLowerCase()).equals("html"))) &&
              (!((name.toLowerCase()).equals("thmb"))) &&   
              (!((name.toLowerCase()).equals("qbic"))))   
          {
             find = false;
             for (i = 0; i < attrCount; i++)
             {  
                if ((attrName[i].toUpperCase()).equals(name.toUpperCase()))
                {
                   dataId = ddo.dataId(attrName[i]);
                   ddo.setData(dataId, value);
                   find = true;
                   break;
                }                
             }
             if (find)
               continue;        
          }

          if ( (itemtype.intValue() == (new Short(DK_CM_FOLDER)).intValue())
                   && ((name.toLowerCase()).equals("item")) )  
          {  
             if (!(folderDDO_exist))
             {
               // ### create an folder ddo with an empty folder
               System.out.println("about do ddo->add() of folder....");
               ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, itemtype);
               folder = new DKFolder();
               dataId = ddo.addData(DKFOLDER);
               ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, new Short(DK_CM_DATAITEM_TYPE_COLLECTION_DDO));
               ddo.setData(dataId, folder);                             
               ddo.add(); 
               out.write("ddo.add() of folder successful...");
               out.newLine();
               System.out.println("ddo.add() of folder successful...");
               folderDDO_exist = true;  
             }          
             item[itemcount] = new DKDDO((short)DK_CM_DDO_INITIAL_SIZE);
             load(item[itemcount]);             
             itemcount++;
             out.write("===***=== Folder itemDDO #" + itemcount + " Created ===***===");
             out.newLine();
          }

          if ( (itemtype.intValue() == (new Short(DK_CM_DOCUMENT)).intValue())
                   && ( ((name.toLowerCase()).equals("xdo")) ||
                        ((name.toLowerCase()).equals("smi")) ||
                        ((name.toLowerCase()).equals("html")) ||
                        ((name.toLowerCase()).equals("thmb")) ||
                        ((name.toLowerCase()).equals("qbic"))  ) 
				   && (value != null) )
          {
             if ((name.toLowerCase()).equals("smi"))
             {
               xdoFiles[xdocount] = prepareLine(value, smiPartId);
               xdo[xdocount] = createXDO(value, DK_DL_CC_ASCII, (short)smiPartId);
  /**** no text search by comment out following line ****/  
               DKSearchEngineInfoDL aSrchEx = new DKSearchEngineInfoDL();
               aSrchEx.setSearchEngine(searchEngineTS);
               aSrchEx.setSearchIndex(searchIndexTM);
               aSrchEx.setSearchInfo(searchInfo);
               xdo[xdocount].setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx);                
  /**** no text search by comment out following line ****/  
               smiSrv = true;             
             } 
             else if ((name.toLowerCase()).equals("html"))
             {
               xdoFiles[xdocount] = prepareLine(value, htmlPartId);
               xdo[xdocount] = createXDO(value, DK_DL_CC_HTML, (short)htmlPartId);                 
             } 
             else if ((name.toLowerCase()).equals("qbic"))
             {
               CClass = findContentClass(value); 
               xdoFiles[xdocount] = prepareLine(value, qbicPartId);
               xdo[xdocount] = createXDO(value, CClass, (short)qbicPartId);
  /**** no image search by comment out following line ****/ 
               DKSearchEngineInfoDL aSrchEx2 = new DKSearchEngineInfoDL();
               aSrchEx2.setSearchEngine(searchEngineQBIC);
               aSrchEx2.setSearchIndex(searchIndexQBIC);
               aSrchEx2.setSearchInfo(searchInfo);
               xdo[xdocount].setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx2);          
  /**** no image search by comment out following line ****/ 
               qbicSrv = true;                  
             } 
             else if ((name.toLowerCase()).equals("thmb"))
             {
               CClass = findContentClass(value); 
               xdoFiles[xdocount] = prepareLine(value, thmbPartId);
               xdo[xdocount] = createXDO(value, CClass, (short)thmbPartId);                 
             } 
             else
             {
               CClass = findContentClass(value); 
               xdoFiles[xdocount] = prepareLine(value, partId);
               xdo[xdocount] = createXDO(value, CClass, (short)partId);
               partId++;  
             }         
             xdocount++;
          }
      }
    } while ( !((name.toLowerCase()).equals("end")) );
    out.write("  ****************The End Of " + value + " Reached ");
    out.newLine();

    if ( (itemtype.intValue() == (new Short(DK_CM_FOLDER)).intValue()) )
    {
       // ### add ddo to folder one by one 
       for (i = 0; i < itemcount; i++) 
       {
          folder.addMember(ddo, item[i]);
          folder.removeAllElements();
       } 
       out.write(" =========== No. of Items Added To The Folder: " + itemcount);
       out.newLine();
    }

    if ( (itemtype.intValue() == (new Short(DK_CM_DOCUMENT)).intValue()) )
    {
       // ### set ddo attribute property
       ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, itemtype);  

       BufferedWriter note = new BufferedWriter(
             new FileWriter(noteName));
       parts = new DKParts();
       for (i = 0; i < xdocount; i++)
       {
         //-------------------------------------
         // create the notepart file
         //-------------------------------------
         String tmp = xdoFiles[i];
         note.write(tmp, 0, tmp.length());
         note.newLine();                 
         parts.addElement(xdo[i]);
       }  
       note.close();
       parts.addElement(createXDO(noteName, DK_DL_CC_ASCII, notePartId));
       dataId = ddo.addData(DKPARTS);
       ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, new Short(DK_CM_DATAITEM_TYPE_COLLECTION_XDO));
       ddo.setData(dataId, parts);
       ddo.add();
       out.write("ddo.add() of document successful...");
       out.newLine();
       System.out.println("ddo.add() of document successful...");
       out.write("added itemId = " + ddo.getPidObject().getPrimaryId());
       out.newLine();
       parts.removeAllElements();
    }
    return;
  }

  //*****************************************************
  // createXDO - create an xdo
  //*****************************************************
  public static DKBlobDL createXDO(String file, int CClass, int partId)
            throws DKException, IOException, Exception
  {
       
     DKBlobDL xdo = new DKBlobDL(dsDL);
     xdo.setContentClass(CClass);
     if ((CClass == DK_DL_CC_ASCII) && (partId == 2))
         xdo.setAffiliatedType(DK_DL_NOTE); 
     else  
         xdo.setAffiliatedType(DK_DL_BASE);
     DKPidXDODL pidx = new DKPidXDODL();
     pidx.setObjectType("DKBlobDL");
     pidx.setPartId(partId);
     xdo.setPidObject(pidx);   
     xdo.setContentFromClientFile(file);

     out.write("  ##creat XDO part: " + file  +
                  " : " + partId);
     out.newLine();
     return xdo;
  }

  //************************************************************
  // prepareAttrList - prepare the attributes list for each item
  //************************************************************
  public static void  prepareAttrList()
            throws DKException, IOException, Exception
  {
     DKSequentialCollection  collection;
     dkIterator iterator;
     DKAttrDefDL attrDef; 
     collection = (DKSequentialCollection)
                   dsDL.listEntityAttrs(indexClassName);
     iterator = collection.createIterator();
     int i = 0;
     short sAttrType = 0;
     while (iterator.more())
     {
        attrDef = (DKAttrDefDL)iterator.next();
        attrName[i]      = attrDef.getName();
        sAttrType        = attrDef.getType();
        attrType[i]      = translateAttrTypeToDataItemType(sAttrType);
        attrId[i]        = attrDef.getId();
        attrNullable[i]  = attrDef.isNullable();
        i++;
        attrCount++;
     }
     out.write("Index Class: " + indexClassName + " with " + 
                attrCount + " Attributes Obtained ");
     out.newLine();
  }

  //**********************************************************
  // translateAttrTypeToDataItemType - change attribute type to
  //   ddo dataitem type 
  //**********************************************************
  public static short translateAttrTypeToDataItemType(short attrType)
  {
   short rAttrType = DK_CM_DATAITEM_TYPE_UNDEFINED;
   switch(attrType)
   {
    case DK_CM_VSTRING:
    case DK_CM_FSTRING:
    case DK_CM_DECIMAL:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_STRING;
    }
    break;
    case DK_CM_SHORT:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_SHORT;
    }
    break;
    case DK_CM_LONG:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_LONG;
    }
    break;
    case DK_CM_DATE:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_DATE;
    }
    break;
    case DK_CM_TIME:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIME;
    }
    break;
    case DK_CM_TIMESTAMP:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIMESTAMP;
    }
    break;
   }
   return rAttrType;
  }

  //**********************************************************
  // findContentClass - find content class from file extension
  //**********************************************************
  public static int findContentClass(String fullFileName)
            throws DKException, IOException
  {
     String suffix = "";
     int m, CClass;
     m = fullFileName.lastIndexOf(".");
     if (m != -1) suffix = fullFileName.substring(m + 1);
     if (suffix != "")
     {
       if ( (suffix.toLowerCase()).equals("gif") )
             CClass = DK_DL_CC_GIF;
       else if ( ((suffix.toLowerCase()).equals("html")) ||
                  ((suffix.toLowerCase()).equals("htm")) )
             CClass = DK_DL_CC_HTML;
       else if ( (suffix.toLowerCase()).equals("jpg") )
             CClass = DK_DL_CC_JPG;
       else if ( (suffix.toLowerCase()).equals("avi") )
             CClass = DK_DL_CC_AVI;
       else if ( (suffix.toLowerCase()).equals("wav") )
             CClass = DK_DL_CC_WAV;
       else if ( (suffix.toLowerCase()).equals("mid") )
             CClass = DK_DL_CC_MIDI;
       else if ( (suffix.toLowerCase()).equals("mov") )
            CClass = DK_DL_CC_MOV;
       else if ( (suffix.toLowerCase()).equals("wpd") )
             CClass = DK_DL_CC_WP;
       else if ( (suffix.toLowerCase()).equals("wpg") )
            CClass = DK_DL_CC_WP;
       else if ( (suffix.toLowerCase()).equals("wks") )
             CClass = DK_DL_CC_WKS;
       else if ( (suffix.toLowerCase()).equals("bmp") )
             CClass = DK_DL_CC_WINV3_BMP;
       else if ( (suffix.toLowerCase()).equals("mpg") )
             CClass = DK_DL_CC_MPEG;
       else CClass = DK_DL_CC_ASCII;   // Add more ContentClasses here!!!       
     }
     else
        CClass = DK_DL_CC_ASCII; 
     return CClass;
  }
  //**********************************************************
  // prepareLine - prepare a line record for the notepart file
  //**********************************************************
  public static String  prepareLine(String fullFileName, int partId)
  {
     String lineStr;
     int k, m;
     m = fullFileName.lastIndexOf("\\");
     k = fullFileName.lastIndexOf("/");
     if ((k == -1) && (m == -1))
     { 
        lineStr = fullFileName + ":" + String.valueOf(partId);                   
     }
     else if (k == -1)
        lineStr = (fullFileName.substring(m + 1)) + ":"
              + String.valueOf(partId);
     else 
        lineStr = (fullFileName.substring(k + 1)) + ":" 
              + String.valueOf(partId);
     return lineStr;
  }
  //**********************************************************
  // verifySetUp - verify the TextMiner index "TMINDEX" and
  // the QBIC Database and QBIC catalog are defined
  //**********************************************************
  public static void  verifySetUp(String userId, String passWord)
            throws DKException, IOException, Exception
  {
     System.out.println("verifying: " + searchIndexTS );
     DKDatastoreTS dsTS = new DKDatastoreTS();
     DKIndexInfoTS pIndxInfo = null;
     System.out.println("connecting to datastoreTS... " );
     dsTS.connect(searchServerTS, "", ' '); 
     System.out.println("datastoreTS connected. " );
     pIndxInfo = dsTS.getIndexInformation(searchIndexTS);
     System.out.println("Index Name = " + pIndxInfo.getIndexName());
     dsTS.disconnect();
     System.out.println("datastoreTS disconnected." );

     System.out.println("verifying QBIC database/catalog: " +
            QBICDataBase + " " + QBICCatalog);
     DKDatastoreQBIC dsQBIC = new DKDatastoreQBIC();
     System.out.println("connecting to datastoreQBIC... " );
     dsQBIC.connect(searchServerQBIC, userId, passWord, "");
     System.out.println("datastoreQBIC connected. " );
     System.out.println("open catalog " + QBICCatalog );
     dsQBIC.openCatalog(QBICDataBase, QBICCatalog);
     System.out.println("catalog opened successfully. " );
     dsQBIC.closeCatalog();
     dsQBIC.disconnect();
     System.out.println("datastoreQBIC disconnected. " );
     return;
  }

}






