/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    This sample pulls together all concepts learned so far, using the IBM COntent Management
    Data Modeling Concepts in a business model example.  The Insurance Model is defined and
    explained in SItemTypeCreationICM.
    
    Minimum Required Reading & Samples Run
        SConnectDisconnectICM                   SFolderICM
        SAttributeDefinitionCreationICM         SLinksICM
        SAttributeGroupDefinitionCreationICM    SResourceMgrDefCreationICM
        SReferenceAttrDefCreationICM            SResourceMgrSetDefaultICM
        SItemTypeCreationICM                    SSMSCollectionDefCreationICM
        SItemCreationICM                        SSMSCollectionDefSetDefaultICM
        SItemRetrievalICM                       SResourceItemCreaitonICM
        SItemUpdateICM                          SResourceItemRetrievalICM
        SItemDeletionICM

For a view of a real-world business model example, please refer to the specified section for
each funcitonality:
     
    Functionality   Section(s)
    --------------  -------------------------------
    Attributes      Auto Claim, All Sections
    Attr Groups     Adjuster Report 
    Child Component Police Report, Insurance Policy
    Folders         Adjuster Report
    Image XDO       Auto Photo
    Inbound Links   Police Report
    Item Creation   Auto Claim, All Sections
    Linking         Insurance Policy, Police Reports
    MIME Types      Auto Claim, All Sections
    Multivalue Attr Police Report, Insurance Policy
    Outbound Links  Insurance Policy
    Printing Items  Printing Section
    Reference Attr  Adjuster Report 
    Resource XDO    Auto Claim, All Sections
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SInsuranceModelItemCreaitonICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Putting everything together into a real-world example, the Sample 
 *                    Insurance Model.  Creates & Prints Items for Insurance Model.  
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Putting everything together into a real-world example
 *                    Creating Insurance Model's Auto Claims
 *                    Creating Insurance Model's Adjustment Reports
 *                    Creating Insurance Model's Police Reports
 *                    Creating Insurance Model's Insurance Policies
 *                    Creating Insurance Model's Auto Photos
 *                    Using Folders
 *                    Creating Links
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SInsuranceModelItemCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemCreationICM.java       (Needed by SItemRetrievalICM)
 *                    SItemRetrievalICM.java
 *                    SLinksICM.java
 *                    SInsuranceModelICM_AccidenReport_0120200130.tif
 *                    SInsuranceModelICM_AccidenReport_0506200110.tif
 *                    SInsuranceModelICM_AccidenReport_729200173.tif
 *                    SInsuranceModelICM_Adjustment_4-852369.tif
 *                    SInsuranceModelICM_Adjustment_5-654123.tif
 *                    SInsuranceModelICM_Adjustment_6-987654.tif
 *                    SInsuranceModelICM_Claim_4-852369.tif
 *                    SInsuranceModelICM_Claim_5-654123.tif
 *                    SInsuranceModelICM_Claim_6-987654.tif
 *                    SInsuranceModelICM_Photo_1ABC089.jpg
 *                    SInsuranceModelICM_Photo_4IBM134-1.tif
 *                    SInsuranceModelICM_Photo_4IBM134-2.tif
 *                    SInsuranceModelICM_Photo_6usa676-1.tif
 *                    SInsuranceModelICM_Photo_6usa676-2.tif
 *                    SInsuranceModelICM_Policy_10285265410.tif
 *                    SInsuranceModelICM_Policy_10315975308.tif
 *                    SInsuranceModelICM_Policy_10452136412.tif
 ************************************************************************************************/
public class SInsuranceModelItemCreationICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SInsuranceModelItemCreationICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===============================================");
        System.out.println("IBM Information Integrator for Content     v"+ver);
        System.out.println("Sample Program:  SInsuranceModelItemCreationICM");
        System.out.println("-----------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===============================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Uses Function provided below.
            System.out.println("Creating Insurance Model Items...");

                createInsuranceModelItems(dsICM);  // Use function provided below.

            System.out.println("Created Insurance Model Items.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Create Items for the Insurance Model.
    * @param dsICM  Connected DKDatastoreICM object.
    * @return  Returns a java.util.Vector containing all DDOs created.
    **/
    public static java.util.Vector createInsuranceModelItems(DKDatastoreICM dsICM) throws DKException, Exception{

            //------------------------------------------------------------------
            // Insurance Model
            //------------------------------------------------------------------
            // Please refer to the header documentation in SItemTypeCreationICM
            // for descriptions of each Item Type, example data, and the overall
            // Insurance Model.
            //
            // The foillowing sections will assume knowledge of all areas covered.
            // Each have been covered in the prerequisit Samples outlined in the
            // Getting Started section of the Java Sample Readme.
            
            //-------------------------------------------------------------
            // Auto Claim #6-987654 (Gregory Twain)
            //-------------------------------------------------------------
            // Features:  - Item Creation
            //            - Attributes
            //            - XDO
            System.out.println("Creating Auto Claim #6-987654...");

                // Create new DDO/XDO Object.
                DKLobICM autoClaim1 = (DKLobICM) dsICM.createDDO("S_autoClaim",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "6-987654"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  java.sql.Date.valueOf("2001-04-20")); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "Gregory"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Twain"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B12345696"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  "Gregory"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  "Twain"); 
                autoClaim1.setData(autoClaim1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10452136412"); 

                // Store the File
                autoClaim1.setContentFromClientFile("SInsuranceModelICM_Claim_6-987654.tif"); // Load the file into memory.
                autoClaim1.setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                autoClaim1.add();

            System.out.println("Created Auto Claim #6-987654.");

            //-------------------------------------------------------------
            // Auto Claim #5-654123 (Jessica Fitzgerald)
            //-------------------------------------------------------------
            System.out.println("Creating Auto Claim #5-654123...");

                // Create new DDO/XDO Object.
                DKLobICM autoClaim2 = (DKLobICM) dsICM.createDDO("S_autoClaim",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "5-654123"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  java.sql.Date.valueOf("2001-05-04")); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "Jessica"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Fitzgerald"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B98755678"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  "Alexander"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  "Churhill"); 
                autoClaim2.setData(autoClaim2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10315975308"); 

                // Store the File
                autoClaim2.setContentFromClientFile("SInsuranceModelICM_Claim_5-654123.tif"); // Load the file into memory.
                autoClaim2.setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                autoClaim2.add();

            System.out.println("Created Auto Claim #5-654123.");

            //-------------------------------------------------------------
            // Auto Claim #4-852369 (George Tolstoy)
            //-------------------------------------------------------------
            System.out.println("Creating Auto Claim #5-654123...");

                // Create new DDO/XDO Object.
                DKLobICM autoClaim3 = (DKLobICM) dsICM.createDDO("S_autoClaim",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "4-852369"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  java.sql.Date.valueOf("2001-07-29")); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "George"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Tolstoy"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B25878914"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  "Gabriela"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  "Neruda"); 
                autoClaim3.setData(autoClaim3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10285265410"); 

                // Store the File
                autoClaim3.setContentFromClientFile("SInsuranceModelICM_Claim_4-852369.tif"); // Load the file into memory.
                autoClaim3.setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                autoClaim3.add();

            System.out.println("Created Auto Claim #4-852369.");

            //-------------------------------------------------------------
            // Auto Photo 1 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            // Features:  - Image XDO
            System.out.println("Creating Auto Photo for Vin #1HOME10R4KL987258...");

                // Create new DDO/XDO Object.
                DKImageICM photo1a = (DKImageICM) dsICM.createDDO("S_autoPhoto",DKConstant.DK_CM_ITEM);
                
                // Set Attributes
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "6-987654"); 
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10452136412"); 
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"),           "1HOME10R4KL987258"); 
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Michael"); 
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Reed"); 
                photo1a.setData(photo1a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-02-01")); 

                // Store the File
                photo1a.setContentFromClientFile("SInsuranceModelICM_Photo_4IBM134-1.tif"); // Load the file into memory.
                photo1a.setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo1a.add();

            System.out.println("Created Auto Photo for Vin #1HOME10R4KL987258.");

            //-------------------------------------------------------------
            // Auto Photo 2 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            System.out.println("Creating Auto Photo for Vin #1HOME10R4KL987258...");

                // Create new DDO/XDO Object.
                DKImageICM photo1b = (DKImageICM) dsICM.createDDO("S_autoPhoto",DKConstant.DK_CM_ITEM);
                
                // Set Attributes
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "6-987654"); 
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10452136412"); 
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"),           "1HOME10R4KL987258"); 
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Michael"); 
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Reed"); 
                photo1b.setData(photo1b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-02-01")); 

                // Store the File
                photo1b.setContentFromClientFile("SInsuranceModelICM_Photo_4IBM134-2.tif"); // Load the file into memory.
                photo1b.setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo1b.add();

            System.out.println("Created Auto Photo for Vin #1HOME10R4KL987258.");

            //-------------------------------------------------------------
            // Auto Photo for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            System.out.println("Creating Auto Photo for Vin #1HOME10R4KL987258...");

                // Create new DDO/XDO Object.
                DKImageICM photo2 = (DKImageICM) dsICM.createDDO("S_autoPhoto",DKConstant.DK_CM_ITEM);
                
                // Set Attributes
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "5-654123"); 
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10315975308"); 
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"),           "5CALL78T6PU743358"); 
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "John"); 
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Johnson"); 
                photo2.setData(photo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-05-08")); 

                // Store the File
                photo2.setContentFromClientFile("SInsuranceModelICM_Photo_1ABC089.jpg");   // Load the file into memory.
                photo2.setMimeType("image/jpeg");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo2.add();

            System.out.println("Created Auto Photo for Vin #1HOME10R4KL987258.");

            //-------------------------------------------------------------
            // Auto Photo 1 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            System.out.println("Creating Auto Photo for Vin #1HOME10R4KL987258...");

                // Create new DDO/XDO Object.
                DKImageICM photo3a = (DKImageICM) dsICM.createDDO("S_autoPhoto",DKConstant.DK_CM_ITEM);
                
                // Set Attributes
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "4-852369"); 
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10285265410"); 
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"),           "9GOAL56H3LI552314"); 
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Catherine"); 
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Adjuster"); 
                photo3a.setData(photo3a.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-08-02")); 

                // Store the File
                photo3a.setContentFromClientFile("SInsuranceModelICM_Photo_6usa676-1.tif"); // Load the file into memory.
                photo3a.setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo3a.add();

            System.out.println("Created Auto Photo for Vin #1HOME10R4KL987258.");

            //-------------------------------------------------------------
            // Auto Photo 2 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            System.out.println("Creating Auto Photo for Vin #1HOME10R4KL987258...");

                // Create new DDO/XDO Object.
                DKImageICM photo3b = (DKImageICM) dsICM.createDDO("S_autoPhoto",DKConstant.DK_CM_ITEM);
                
                // Set Attributes
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "4-852369"); 
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10285265410"); 
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"),           "9GOAL56H3LI552314"); 
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Catherine"); 
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Adjuster"); 
                photo3b.setData(photo3b.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-08-02")); 

                // Store the File
                photo3b.setContentFromClientFile("SInsuranceModelICM_Photo_6usa676-2.tif"); // Load the file into memory.
                photo3b.setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo3b.add();

            System.out.println("Created Auto Photo for Vin #1HOME10R4KL987258.");

            //-------------------------------------------------------------
            // Adjuster Report for Claim #6-987654
            //-------------------------------------------------------------
            // Features:  - Attribute Group
            //            - Reference Attribute
            //            - Folder
            System.out.println("Creating Adjuster Report for Claim #6-987654...");

                // Create new DDO/XDO Object.
                DKLobICM adjusterReport1 = (DKLobICM) dsICM.createDDO("S_adjustrReport",DKConstant.DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "6-987654");               // Attributes
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10452136412"); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Michael"); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Reed"); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-02-01")); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "Gregory"); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Twain"); 
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     autoClaim1);                // Reference Attribute
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), "4IBM134");    // Attribute Group
                adjusterReport1.setData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), java.sql.Date.valueOf("2001-04-20"));
            
                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                DKFolder dkFolder = (DKFolder) adjusterReport1.getData(adjusterReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));
                dkFolder.addElement(photo1a); 
                dkFolder.addElement(photo1b); 

                // Store the File
                adjusterReport1.setContentFromClientFile("SInsuranceModelICM_Adjustment_6-987654.tif"); // Load the file into memory.
                adjusterReport1.setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport1.add();

            System.out.println("Created Adjuster Report for Claim #6-987654.");

            //-------------------------------------------------------------
            // Adjuster Report for Claim #5-654123
            //-------------------------------------------------------------
            System.out.println("Creating Adjuster Report for Claim #5-654123...");

                // Create new DDO/XDO Object.
                DKLobICM adjusterReport2 = (DKLobICM) dsICM.createDDO("S_adjustrReport",DKConstant.DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "5-654123");               // Attributes
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10315975308"); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "John"); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Johnson"); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-05-08")); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "Jessica"); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Fitsgerald"); 
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     autoClaim2);                // Reference Attribute
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), "1ABC089");    // Attribute Group
                adjusterReport2.setData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), java.sql.Date.valueOf("2001-05-04"));

                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                dkFolder = (DKFolder) adjusterReport2.getData(adjusterReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));
                dkFolder.addElement(photo2); 

                // Store the File
                adjusterReport2.setContentFromClientFile("SInsuranceModelICM_Adjustment_5-654123.tif"); // Load the file into memory.
                adjusterReport2.setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport2.add();

            System.out.println("Created Adjuster Report for Claim #5-654123.");

            //-------------------------------------------------------------
            // Adjuster Report for Claim #4-852369
            //-------------------------------------------------------------
            System.out.println("Creating Adjuster Report for Claim #4-852369...");

                // Create new DDO/XDO Object.
                DKLobICM adjusterReport3 = (DKLobICM) dsICM.createDDO("S_adjustrReport",DKConstant.DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   "4-852369");                // Attrib8utes
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  "10285265410"); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), "Catherine"); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), "Green"); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), java.sql.Date.valueOf("2001-08-02")); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantFName"), "George"); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_claimantLName"), "Tolstoy"); 
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     autoClaim3);                // Reference Attribute
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), "6USA676");    // Attribute Group
                adjusterReport3.setData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), java.sql.Date.valueOf("2001-07-29"));

                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                dkFolder = (DKFolder) adjusterReport3.getData(adjusterReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,DKConstant.DK_CM_DKFOLDER));
                dkFolder.addElement(photo3a); 
                dkFolder.addElement(photo3b); 

                // Store the File
                adjusterReport3.setContentFromClientFile("SInsuranceModelICM_Adjustment_4-852369.tif"); // Load the file into memory.
                adjusterReport3.setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport3.add();

            System.out.println("Created Adjuster Report for Claim #4-852369.");

            //-------------------------------------------------------------
            // Police Report #01202001-30
            //-------------------------------------------------------------
            // Features:  - Multivalue Attributes
            //            - Child Components
            //            - Inbound Links
            System.out.println("Creating Police Report #01202001-30...");

                // Create new DDO/XDO Object.
                DKLobICM policeReport1 = (DKLobICM) dsICM.createDDO("S_policeReport",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport1.setData(policeReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_reportNumber"), "01202001-30"); 
                policeReport1.setData(policeReport1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"), java.sql.Date.valueOf("2001-01-20")); 
                
                // Set Multi-Value Attributes / Child Components
                DKDDO victimInfo = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo.setData(victimInfo.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "4IBM134"); 
                victimInfo.setData(victimInfo.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B12345696"); 

                // Get Child Component Collection
                short dataid = policeReport1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                DKChildCollection children = (DKChildCollection) policeReport1.getData(dataid);
               
                // If none exists, create One
                if(children==null){
                    children = new DKChildCollection();
                    policeReport1.setData(policeReport1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_victimInfo"), children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children.addElement(victimInfo); 

                // Link Adjuster Report to Police Report (Inbound Link)
                DKLink dkLink = new DKLink("S_policeReports",adjusterReport1,policeReport1);
                SLinksICM.addLinkToDDO(dkLink,policeReport1);

                // Store the File
                policeReport1.setContentFromClientFile("SInsuranceModelICM_AccidenReport_0120200130.tif"); // Load the file into memory.
                policeReport1.setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policeReport1.add();
                
            System.out.println("Created Police Report #01202001-30.");

            //-------------------------------------------------------------
            // Police Report #05062001-10
            //-------------------------------------------------------------
            System.out.println("Creating Police Report #05062001-10...");

                // Create new DDO/XDO Object.
                DKLobICM policeReport2 = (DKLobICM) dsICM.createDDO("S_policeReport",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport2.setData(policeReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_reportNumber"), "05062001-10"); 
                policeReport2.setData(policeReport2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"), java.sql.Date.valueOf("2001-05-04")); 
                
                // Set Multi-Value Attributes / Child Components
                DKDDO victimInfo1 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "1ABC089"); 
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B98755678"); 
                DKDDO victimInfo2 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo2.setData(victimInfo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "5JJJ064"); 
                victimInfo2.setData(victimInfo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "C99851876"); 

                // Get Child Component Collection
                dataid   = policeReport2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection) policeReport2.getData(dataid);
               
                // If none exists, create One
                if(children==null){
                    children = new DKChildCollection();
                    policeReport2.setData(policeReport2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_victimInfo"), children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children.addElement(victimInfo1); 
                children.addElement(victimInfo2); 

                // Link Adjuster Report to Police Report (Inbound Link)
                dkLink = new DKLink("S_policeReports",adjusterReport2,policeReport2);
                SLinksICM.addLinkToDDO(dkLink,policeReport2);

                // Store the File
                policeReport2.setContentFromClientFile("SInsuranceModelICM_AccidenReport_0506200110.tif"); // Load the file into memory.
                policeReport2.setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policeReport2.add();
                
            System.out.println("Created Police Report #05062001-10.");

            //-------------------------------------------------------------
            // Police Report #07292001-00
            //-------------------------------------------------------------
            System.out.println("Creating Police Report #07292001-00...");

                // Create new DDO/XDO Object.
                DKLobICM policeReport3 = (DKLobICM) dsICM.createDDO("S_policeReport",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport3.setData(policeReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_reportNumber"), "07292001-00"); 
                policeReport3.setData(policeReport3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"), java.sql.Date.valueOf("2001-07-29")); 
                
                // Set Multi-Value Attributes / Child Components
                victimInfo1 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "1ZZZ687"); 
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "G88468714"); 
                victimInfo2 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo2.setData(victimInfo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "5SEG684"); 
                victimInfo2.setData(victimInfo2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "G58166871"); 
                DKDDO victimInfo3 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo3.setData(victimInfo3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "5CCC876"); 
                victimInfo3.setData(victimInfo3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "A87163587"); 

                // Get Child Component Collection
                dataid   = policeReport3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection) policeReport3.getData(dataid);
               
                // If none exists, create One
                if(children==null){
                    children = new DKChildCollection();
                    policeReport3.setData(policeReport3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_victimInfo"), children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children.addElement(victimInfo1); 
                children.addElement(victimInfo2); 
                children.addElement(victimInfo3); 

                // Link Adjuster Report to Police Report (Inbound Link)
                // No Adjustment Reports Exist for this report.

                // Store the File
                // No File Available for this report.

                // Save New Item In Datastore
                policeReport3.add();
                
            System.out.println("Created Police Report #07292001-00.");

            //-------------------------------------------------------------
            // Police Report #07292001-73
            //-------------------------------------------------------------
            System.out.println("Creating Police Report #07292001-73...");

                // Create new DDO/XDO Object.
                DKLobICM policeReport4 = (DKLobICM) dsICM.createDDO("S_policeReport",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport4.setData(policeReport4.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_reportNumber"), "07292001-73"); 
                policeReport4.setData(policeReport4.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_accidentDate"), java.sql.Date.valueOf("2001-07-29")); 
                
                // Set Multi-Value Attributes / Child Components
                victimInfo1 = dsICM.createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  "6USA676"); 
                victimInfo1.setData(victimInfo1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_driverLicense"), "B25878914"); 

                // Get Child Component Collection
                dataid   = policeReport4.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection) policeReport4.getData(dataid);
               
                // If none exists, create One
                if(children==null){
                    children = new DKChildCollection();
                    policeReport4.setData(policeReport4.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_victimInfo"), children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children.addElement(victimInfo1); 

                // Link Adjuster Report to Police Report (Inbound Link)
                dkLink = new DKLink("S_policeReports",adjusterReport3,policeReport4);
                SLinksICM.addLinkToDDO(dkLink,policeReport4);

                // Store the File
                policeReport4.setContentFromClientFile("SInsuranceModelICM_AccidenReport_729200173.tif"); // Load the file into memory.
                policeReport4.setMimeType("image/tiff");                                                  // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policeReport4.add();
                
            System.out.println("Created Police Report #07292001-73.");

            //-------------------------------------------------------------
            // Insurance Policy #10452136412
            //-------------------------------------------------------------
            // Features:  - Linking
            System.out.println("Creating Insurance Policy #10452136412...");

                // Create new DDO/XDO Object.
                DKLobICM policy1 = (DKLobICM) dsICM.createDDO("S_insurePolicy",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_street"),       "258 Green Rd."); 
                policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_city"),         "Ocean"); 
                policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_state"),        "CA"); 
                policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_zipCode"),      "90060"); 
                policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"), "10452136412"); 
                
                // Set Multi-Value Attributes / Child Components
                DKDDO insured1 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Gregory"); 
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Twain"); 
                DKDDO vin1 = dsICM.createChildDDO("S_insurePolicy", "S_vin");
                vin1.setData(vin1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"), "1HOME10R4KL987258"); 
                DKDDO vin2 = dsICM.createChildDDO("S_insurePolicy", "S_vin");
                vin2.setData(vin2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"), "3CASA55R4KL987500"); 

                // Get Insured Child Component Collection
                dataid  = policy1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_insured");
                DKChildCollection insured = (DKChildCollection) policy1.getData(dataid);
               
                // If none exists, create One
                if(insured==null){
                    insured = new DKChildCollection();
                    policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_insured"), insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_vin");
                DKChildCollection vin = (DKChildCollection) policy1.getData(dataid);
               
                // If none exists, create One
                if(vin==null){
                    vin = new DKChildCollection();
                    policy1.setData(policy1.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_vin"), vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured.addElement(insured1); 
                vin.addElement(vin1); 
                vin.addElement(vin2); 

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                dkLink = new DKLink("S_policeReports",policy1,policeReport1);
                SLinksICM.addLinkToDDO(dkLink,policy1);
                dkLink = new DKLink("S_autoClaims",policy1,autoClaim1);
                SLinksICM.addLinkToDDO(dkLink,policy1);
                dkLink = new DKLink("S_adjustments",policy1,adjusterReport1);
                SLinksICM.addLinkToDDO(dkLink,policy1);

                // Store the File
                policy1.setContentFromClientFile("SInsuranceModelICM_Policy_10452136412.tif"); // Load the file into memory.
                policy1.setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policy1.add();

            System.out.println("Created Insurance Policy #10452136412.");

            //-------------------------------------------------------------
            // Insurance Policy #10315975308
            //-------------------------------------------------------------
            System.out.println("Creating Insurance Policy #10315975308...");

                // Create new DDO/XDO Object.
                DKLobICM policy2 = (DKLobICM) dsICM.createDDO("S_insurePolicy",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_street"),       "456 First St."); 
                policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_city"),         "Santa"); 
                policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_state"),        "CA"); 
                policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_zipCode"),      "90123"); 
                policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"), "10315975308"); 

                // Set Multi-Value Attributes / Child Components
                insured1 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Alexand"); 
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Churchill"); 
                DKDDO insured2 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured2.setData(insured2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Joan"); 
                insured2.setData(insured2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Churchill"); 
                DKDDO insured3 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured3.setData(insured3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Marty"); 
                insured3.setData(insured3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Churchill"); 
                vin1 = dsICM.createChildDDO("S_insurePolicy", "S_vin");
                vin1.setData(vin1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"), "5CALL78T6PU743358"); 

                // Get Insured Child Component Collection
                dataid  = policy2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_insured");
                insured = (DKChildCollection) policy2.getData(dataid);
               
                // If none exists, create One
                if(insured==null){
                    insured = new DKChildCollection();
                    policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_insured"), insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_vin");
                vin     = (DKChildCollection) policy2.getData(dataid);
               
                // If none exists, create One
                if(vin==null){
                    vin = new DKChildCollection();
                    policy2.setData(policy2.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_vin"), vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured.addElement(insured1); 
                insured.addElement(insured2); 
                insured.addElement(insured3); 
                vin.addElement(vin1); 

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                dkLink = new DKLink("S_policeReports",policy2,policeReport2);
                SLinksICM.addLinkToDDO(dkLink,policy2);
                dkLink = new DKLink("S_autoClaims",policy2,autoClaim2);
                SLinksICM.addLinkToDDO(dkLink,policy2);
                dkLink = new DKLink("S_adjustments",policy2,adjusterReport2);
                SLinksICM.addLinkToDDO(dkLink,policy2);

                // Store the File
                policy2.setContentFromClientFile("SInsuranceModelICM_Policy_10315975308.tif"); // Load the file into memory.
                policy2.setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policy2.add();

            System.out.println("Created Insurance Policy #10315975308.");

            //-------------------------------------------------------------
            // Insurance Policy #10285265410
            //-------------------------------------------------------------
            System.out.println("Creating Insurance Policy #10285265410...");

                // Create new DDO/XDO Object.
                DKLobICM policy3 = (DKLobICM) dsICM.createDDO("S_insurePolicy",DKConstant.DK_CM_DOCUMENT);
                
                // Set Attributes
                policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_street"),       "333 Karenin"); 
                policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_city"),         "Marina"); 
                policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_state"),        "CA"); 
                policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_zipCode"),      "90546"); 
                policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_policyNumber"), "10285265410"); 

                // Set Multi-Value Attributes / Child Components
                insured1 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Gabriela"); 
                insured1.setData(insured1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Neruda"); 
                insured2 = dsICM.createChildDDO("S_insurePolicy", "S_insured");
                insured2.setData(insured2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_firstName"), "Francisco"); 
                insured2.setData(insured2.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_lastName"),  "Neruda"); 
                vin1 = dsICM.createChildDDO("S_insurePolicy", "S_vin");
                vin1.setData(vin1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_vin"), "9GOAL56H3LI552314"); 

                // Get Insured Child Component Collection
                dataid  = policy3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_insured");
                insured = (DKChildCollection) policy3.getData(dataid);
               
                // If none exists, create One
                if(insured==null){
                    insured = new DKChildCollection();
                    policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_insured"), insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_vin");
                vin     = (DKChildCollection) policy3.getData(dataid);
               
                // If none exists, create One
                if(vin==null){
                    vin = new DKChildCollection();
                    policy3.setData(policy3.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_vin"), vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured.addElement(insured1); 
                insured.addElement(insured2); 
                vin.addElement(vin1); 

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                dkLink = new DKLink("S_policeReports",policy3,policeReport4);
                SLinksICM.addLinkToDDO(dkLink,policy3);
                dkLink = new DKLink("S_autoClaims",policy3,autoClaim3);
                SLinksICM.addLinkToDDO(dkLink,policy3);
                dkLink = new DKLink("S_adjustments",policy3,adjusterReport3);
                SLinksICM.addLinkToDDO(dkLink,policy3);

                // Store the File
                policy3.setContentFromClientFile("SInsuranceModelICM_Policy_10285265410.tif"); // Load the file into memory.
                policy3.setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                policy3.add();

            System.out.println("Created Insurance Policy #10285265410.");

            //-------------------------------------------------------------
            // Print Each
            //-------------------------------------------------------------
            // Using the SItemRetrievalICM's printDDO funciton, this will
            // demonstrate what each DDO looks like now.
            System.out.println("Printing Items...");

                SItemRetrievalICM.printDDO(autoClaim2);
                SItemRetrievalICM.printDDO(photo2);
                SItemRetrievalICM.printDDO(adjusterReport2);
                SItemRetrievalICM.printDDO(policeReport2);
                SItemRetrievalICM.printDDO(policy2);

            System.out.println("Printed Items.");
            
            //-------------------------------------------------------------
            // Package up DDOs 
            //-------------------------------------------------------------
            java.util.Vector ddoVector = new java.util.Vector();
            ddoVector.add(autoClaim1);
            ddoVector.add(autoClaim2);
            ddoVector.add(autoClaim3);
            ddoVector.add(photo1a);
            ddoVector.add(photo1b);
            ddoVector.add(photo2);
            ddoVector.add(photo3a);
            ddoVector.add(photo3b);
            ddoVector.add(adjusterReport1);
            ddoVector.add(adjusterReport2);
            ddoVector.add(adjusterReport3);
            ddoVector.add(policeReport1);
            ddoVector.add(policeReport2);
            ddoVector.add(policeReport3);
            ddoVector.add(policeReport4);
            ddoVector.add(policy1);
            ddoVector.add(policy2);
            ddoVector.add(policy3);
            
            return(ddoVector);
            
    }//end createInsuranceModelItems

}//end class SInsuranceModelItemCreationICM
              
