/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating Items / Check Out / Locking
    In order to update an item, it must be checked out.  As mentioned in SItemRetrievalICM,
    an option can be specified during item retrieval to check out the item.  If it is not
    check out during retrieval, it simply needs to be explicitly checked out using the 
    DKDatastoreICM.checkOut(ddo) method.
    
    Child Components / Multivalue Attributes
        In order to update a child component or multivalue attribute, the child must be
        accessed through the root node and the update() method called on the root.
        Although child component DDOs may be retrieved, they can not be updated individually.

    Once the item is checked out, any DDO in the hierarchial tree structure may be modified
    the same way they were read in SItemRetrievalICM and set in SItemCreationICM.

    Update Options:

    Option       Contant                       Description
    -----------  ----------------------------  ------------------------------------------------
    New Version  DKConstant.DK_CM_VERSION_NEW  If Version Control is set to "By Application",
                                               for this Item Type, this option explicitly 
                                               specifies that it should create a new version 
                                               instead of updating the specific version checked
                                               out.  For more informaiton in Version Control
                                               options, refer to SItemTypeCreationICM Sample.
    ...........................................................................................
    Check In     DKConstant.DK_CM_CHECKIN      The Item will be checked in / unlocked.
    ...........................................................................................
    Check Out    DKConstant.DK_CM_CHECKOUT     The Item will be checked out / locked.
    (Default)
    -------------------------------------------------------------------------------------------    

Checking In / Unlocking
    An Item will remain checked out / locked until it is explicitly checked
    in either through Update Options or through the DKDatstoreICM.checkIn()
    function.  Please refer to the Update Options for more information.

Updating Child Components / Multivalue Attributes
    Multivalue attributes must be modified, added, and removed/deleted through
    the parent.  All operations must be made persistent through the parent DDO's
    DKDDO.add() or DKDDO.update() functions.
    
    Incremental Changes Tracked
        All changes to each individual DKChildCollection are tracked internally.  
        Each incremental change will be packaged up and sent to the Library Server
        in single call when the root component's DKDDO.update() function is called.
        It is important to note that the incremental changes are tracked and it is
        not the end result that is made persistent.  Optimal modifications rely on
        the fewest changes that result in the same end result.
    
    Updating a Child Component
        Updating a child component is performed simply by finding the desired
        DDO within the parent's DKChildCollection, modifying it, and then call
        DKDDO.update() on the root component.  
        
        The desired DDO must be located in the parent's DKChildCollection by
        creating a dkIterator from the collection, and searching for it based
        on PID String, Component ID, or based on attributes.
    
    Adding a Child Component
        As shown in SItemCreationICM, additin a child coimponent is performed
        simply adding a new child DDO to the parent's DKChildCollection, and 
        then calling DKDDO.update() on the root component.
        
    Removing/Deleting a Child Component
        Deleting a child component is performed through simply removing the
        undesired DKDDO object from the parent's DKChildCollection, and then
        calling DKDDO.update() on the root component.
        
        The desired DDO must be located in the parent's DKChildCollection by
        creating a dkIterator from the collection, and searching for it based
        on PID String, Component ID, or based on attributes.  The 
        DKChildCollection.removeElementAt(dkIterator) function may be used
        to remove the undesired DDO.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, Checking Out / Locking, Checking In / Unlocking Items,
 *                    Update Options
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Check Out / Locking
 *                    Check In / Unlocking
 *                    Modifying Items
 *                    Adding values to Multivalue Attributes / Child Components
 *                    Modifying Existing Values of Multivalue Attributes / Child Components
 *                    Removing values from Multivalue Atttributes / Child Components
 *                    Updating / Saving Items
 *                    Update Options
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemCreationICM.java
 *                    SItemRetrievalICM.java
 ************************************************************************************************/
public class SItemUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemUpdateICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of semantic type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            System.out.println("Creating & Saving Sample Items...");

	            DKDDO ddoDocument = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_DOCUMENT);
	            DKDDO ddoFolder   = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_FOLDER);   
	            DKDDO ddoItem     = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_ITEM);     

            System.out.println("Created & Saved Sample Items.");

            //-------------------------------------------------------------
            // Check Out / Lock
            //-------------------------------------------------------------
            // Items may be checked out / locked explicitly using the 
            // DKDatastoreICM.checkOut() method, or by specifying the 
            // Check Out option during retrieval.
            System.out.println("Checking Out Items...");

                dsICM.checkOut(ddoDocument);
                dsICM.checkOut(ddoFolder);
                dsICM.checkOut(ddoItem);

            System.out.println("Checked Out Items.");

            //-------------------------------------------------------------
            // Modify Item
            //-------------------------------------------------------------
            // Values may be read as explained in SItemRetrievalICM and may
            // be set as explained in SItemCreationICM.
            System.out.println("Modifying Item...");

                // Modify Attribute Values
                ddoDocument.setData(ddoDocument.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_varchar"), "My modified string value"); 
                ddoDocument.setData(ddoDocument.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_double"),  new Double("999")); 
                ddoDocument.setData(ddoDocument.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_integer"), new Integer("888")); 

                ddoFolder.setData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_varchar"), "My modified string value"); 
                ddoFolder.setData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_double"),  new Double("999")); 
                ddoFolder.setData(ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_integer"), new Integer("888")); 

                ddoItem.setData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_varchar"), "My modified string value"); 
                ddoItem.setData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_double"),  new Double("999")); 
                ddoItem.setData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_integer"), new Integer("888")); 

            System.out.println("Modified Item.");

            //-------------------------------------------------------------
            // Updating item in persistent datastore / Saving changes
            //-------------------------------------------------------------
            // Item will remain checked out / locked until it is explicitly
            // checked in either through Update Options or through the checkIn()
            // function.  Please refer to the Update Options specified in the
            // header documentation of this sample.
            System.out.println("Updating / Saving Item...");

                ddoDocument.update();                        // Item will remain checked out.
                ddoFolder.update(DKConstant.DK_CM_CHECKIN);  // May specify Update Options
                ddoItem.update(DKConstant.DK_CM_VERSION_NEW + DKConstant.DK_CM_CHECKIN); // Save as new version.

            System.out.println("Updated / Saved Item.");

            //-------------------------------------------------------------
            // Explicit Check In / Unlocking
            //-------------------------------------------------------------
            System.out.println("Explicitly Checking an item Out & In, Locking & Unlocking...");

                dsICM.checkOut(ddoFolder);
                
                System.out.println("Item Checked Out / Locked.");
                
                dsICM.checkIn(ddoFolder);

            System.out.println("Item Checked In / Unlocked");

            //---------------------------------------------------------------
            // Determining if an Item is Locked / Checked Out & By Which User
            //---------------------------------------------------------------
            System.out.println("Determining if an Item is Locked / Checked Out & By Which User...");

                // Check Out Item
                System.out.println("Checking out item...");
                dsICM.checkOut(ddoFolder);
                
                // Create a new Datastore Extension object.
                DKDatastoreExtICM dsExtICM = (DKDatastoreExtICM) dsICM.getExtension(DKConstant.DK_CM_DATASTORE_EXT); // Get the Datastore Extension Object

                if(dsExtICM.isCheckedOut(ddoFolder)){
                    System.out.println("Item is Checked Out / Locked by Some User.");
                    String checkedOutByStr = dsExtICM.checkedOutUserid(ddoFolder);
                    System.out.println("Item is Checked Out by '"+checkedOutByStr+"'.");
                } else
                    System.out.println("Item is not Checked Out.  It is unlocked.");
                
                System.out.println("Checking out item back in...");
                dsICM.checkIn(ddoFolder);

            System.out.println("Determined if an Item is Locked & By Which User.");
            
            //-------------------------------------------------------------
            // Print Updated DDO 
            //-------------------------------------------------------------
            // Print a DDO using the wrapper from SItemRetrievalICM.            
            System.out.println("Printing DDO...");
                SItemRetrievalICM.printDDO(ddoDocument); // Use wrapper defined in SItemRetrievalICM.
            System.out.println("Printed DDO.");

            //--------------------------------------------------------------
            // Add another value to a Multivalue Attribute / Child Component
            //--------------------------------------------------------------
            System.out.println("Adding another value to a Multivalue Attribute / Child Component...");

                System.out.println("Checking Out Item.");  // Must check it out.
                dsICM.checkOut(ddoItem);
                
                System.out.println("Accessing Multivalue Attribute Named 'S_simpleChild' / Child Collection.");
                // The name of the multivalue attribute is the object type of the child component defined in the Item Type.
                // SItemTypeCreationICM created the S_withChild Item Type with a child type / multivalue attribute of "S_simpleChild".
                short dataid = ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_simpleChild");  // If the multivalue attribute doesn't exist, '0' will be returned.
                DKChildCollection children = (DKChildCollection) ddoItem.getData(dataid);
                if(children==null){ // if no collection exists, create one.
                    System.out.println("Creating New Child Collection...");
                    children = new DKChildCollection();
                    ddoItem.setData(ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_CHILD, "S_simpleChild"), children);
                }
    
                System.out.println("Creating a new value / child ddo.");
                DKDDO child = dsICM.createChildDDO("S_withChild","S_simpleChild"); // Create a new child DDO.
                // ... Set any attributes or children of its own here.
                
                System.out.println("Adding the new value / child ddo to the parent's collection.");
                children.addElement(child); // add children to the collection.
            
                System.out.println("Making changes persistent.");
                ddoItem.update(DKConstant.DK_CM_CHECKIN);  // Explicitly unlocking item.

            System.out.println("Added another value to a Multivalue Attribute / Child Component.");

            //--------------------------------------------------------------
            // Remove a value of a Multivalue Attribute / Child Component
            //--------------------------------------------------------------
            System.out.println("Removing a value of a Multivalue Attribute / Child Component...");

                System.out.println("Checking Out Item.");  // Must check it out.
                dsICM.checkOut(ddoItem);
                
                System.out.println("Accessing Multivalue Attribute Named 'S_simpleChild' / Child Collection.");
                // The name of the multivalue attribute is the object type of the child component defined in the Item Type.
                // SItemTypeCreationICM created the S_withChild Item Type with a child type / multivalue attribute of "S_simpleChild".
                dataid = ddoItem.dataId(DKConstant.DK_CM_NAMESPACE_CHILD,"S_simpleChild");  // If the multivalue attribute doesn't exist, '0' will be returned.
                children = (DKChildCollection) ddoItem.getData(dataid);
    
                System.out.println("Searching for undesired child/value.");
                // Let's say the component id is known.  We will then find it based on Component ID.
                String desiredComponentId = ((DKPidICM)child.getPidObject()).getComponentId();
                dkIterator iter = children.createIterator();
                boolean found = false;  // Search until it is found.
                while(!found){ 
                    DKDDO curChild = (DKDDO) iter.next(); // Get the current element and move pointer to next.
                    if(curChild==null) // if at end of collection, it was not found.
                        throw new Exception("This child should exist in this sample since it was just added above.");
                    String foundComponentId = ((DKPidICM)curChild.getPidObject()).getComponentId();
                    if(foundComponentId.compareTo(desiredComponentId)==0)
                        found = true;
                }
                
                System.out.println("Removing the value / child ddo from the parent's collection.");
                children.removeElementAt(iter); // Remove value / ddo.  Will remove last item returned by iter.next().
            
                System.out.println("Making changes persistent.");
                ddoItem.update(DKConstant.DK_CM_CHECKIN);  // Explicitly unlocking item.

            System.out.println("Removed a value of a Multivalue Attribute / Child Component.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
       
}//end class SItemUpdateICM
              
