/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

When Retrieving Reference Attributes, use it as an Attribute Group.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving & Printing Reference Attribute Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Reference Attribute (Group) Definitions
 *                    Printing Reference Attribute (Group) Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SReferenceAttrDefRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here must already exist.  If not, please
 *                    run the SReferenceAttrDefCreationICM Sample first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SReferenceAttrDefRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SReferenceAttrDefRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("==============================================");
        System.out.println("IBM Information Integrator for Content    v"+ver);
        System.out.println("Sample Program:  SReferenceAttrDefRetrievalICM");
        System.out.println("----------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("==============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Retrieve a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Treat the Reference Attribute (Group) as an actual
            // Attribute Group, since it was created as such.
            //
            // Retrieving the "S_autoClaim" Reference Attribute (Group) Definition.
            System.out.println("Retrieving Reference Atttribute 'S_autoClaim'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_autoClaim"); // Retrieve the definition.

            System.out.println("Retrieved Reference Attribute 'S_autoClaim'.");

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            System.out.println("Printing the Reference Attr Definition of 'S_autoClaim'...");
                printReferenceAttrDef(attrGroupDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Reference Attr Definition of 'S_autoClaim'.");

            //--------------------------------------------------------------
            // Retrieve another Reference Attribute (Group)
            //--------------------------------------------------------------
            // Retrieving the "S_myRef" attribute Group.
            System.out.println("Retrieving Reference Atttribute 'S_myRef'...");

                attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_myRef"); // Retrieve the definition.

            System.out.println("Retrieved Reference Attribute 'S_myRef'.");

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            System.out.println("Printing the Reference Attr Definition of 'S_myRef'...");
                printReferenceAttrDef(attrGroupDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Reference Attr Definition of 'S_myRef'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print an Reference Attribute Definition
    * @param referenceAttrDef  Reference Attribute (Group) Definition, DKAttrGroupDefICM.
    **/
    public static void printReferenceAttrDef(DKAttrGroupDefICM referenceAttrDef) throws DKException, Exception{
        System.out.println("---------------------------------------------");
        System.out.println("       Reference Attribute Definition");
        System.out.println("---------------------------------------------");
        System.out.println("       Name:  "+referenceAttrDef.getName());
        System.out.println("Description:  "+referenceAttrDef.getDescription());
        switch(referenceAttrDef.getRefDeleteRule()){
            case DKConstantICM.DK_ICM_DELETE_RULE_RESTRICT:
                System.out.println("Delete Rule:  Restrict (Default)");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_CASCADE:
                System.out.println("Delete Rule:  Cascade");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_SET_NULL:
                System.out.println("Delete Rule:  Set Null");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_NO_ACTION:
                System.out.println("Delete Rule:  No Action");
                break;
            default:
                System.out.println("Delete Rule:  Other ("+referenceAttrDef.getRefDeleteRule()+")");
                break;
        }
        System.out.println("---------------------------------------------");
    }

   /**
    * Print an Reference Attribute Definition
    * @param dsICM              Connected DKDatastoreICM object.
    * @param referenceAttrName  Reference Attribute Name.
    **/
    public static void printReferenceAttrDef(DKDatastoreICM dsICM, String referenceAttrName) throws DKException, Exception{
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup(referenceAttrName); // Retrieve the definition.
        printReferenceAttrDef(attrGroupDef);   
    }
   
}//end class SReferenceAttrDefRetrievalICM
              
