
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
// sample connect/map/execute/disconnect to IP/390

/*
 * Filename       : TQueryExprIP.java
 * Description    : Execute query expression with schema-mapping on IP Datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TQueryExprIP class
public class TQueryExprIP implements DKConstantIP, DKConstantFed
{
   // Main method
   public static void main(String argv[])
   {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
         pw = new String(""); // pw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TQueryExprIP " +
        "<libSrv> <userid> <pw>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      }

      DKDatastoreIP dsIP = null;
      try
      {
         // NOTE:
         // Fed. Entity, fed. Attr, schema mapping, and search template information
         // could have been obtained from the Federated Datastore.
         // For clarity of this example and for most practical purposes,
         // they are created here in-memory.

         // create federated attributes :
         dkAttrDef fDate = new DKAttrDefFed();
           fDate.setEntityName("FedSample");
           fDate.setName("FedDate");
           fDate.setType(DK_CM_VSTRING);
           fDate.setId((short)95);
           fDate.setSize(50);
           fDate.setScale(1);
           fDate.setPrecision(50);
           fDate.setNullable(true);
           fDate.setStringType(DK_ATTR_ALPHA);

         dkAttrDef fsubject = new DKAttrDefFed();
           fsubject.setEntityName("FedSample");
           fsubject.setName("FedSubject");
           fsubject.setType(DK_CM_VSTRING);
           fsubject.setId((short)99);
           fsubject.setSize(100);
           fsubject.setScale(1);
           fsubject.setPrecision(100);
           fsubject.setNullable(true);
           fsubject.setStringType(DK_ATTR_ALPHA);

         System.out.println(" *** federated attributes created ***");

         // create native attributes
         dkAttrDef nbDate = new DKAttrDefIP();
           nbDate.setEntityName("DOCUMENT");
           nbDate.setName("ReceiveDate");
           nbDate.setType(DK_CM_VSTRING);
           nbDate.setId((short)95);
           nbDate.setSize(50);
           nbDate.setScale(1);
           nbDate.setPrecision(50);
           nbDate.setNullable(true);
           nbDate.setStringType(DK_ATTR_ALPHA);

         dkAttrDef nbsubject = new DKAttrDefIP();
           nbsubject.setEntityName("DOCUMENT");
           nbsubject.setName("FolderId");
           nbsubject.setType(DK_CM_VSTRING);
           nbsubject.setId((short)99);
           nbsubject.setSize(100);
           nbsubject.setScale(1);
           nbsubject.setPrecision(100);
           nbsubject.setNullable(true);
           nbsubject.setStringType(DK_ATTR_ALPHA);

         System.out.println(" *** native attributes created ***");

         //  create schema mapping : entity and attr mappings

         // maps "FedDate" to "FileTab"
         dkAttrMapping mapDate = new DKAttrMappingFed();
           mapDate.addMappedAttr(fDate);
           mapDate.addMapToAttr(nbDate);

         // maps "FedSubject" to "FolderId"
         dkAttrMapping mapSubject = new DKAttrMappingFed();
           mapSubject.addMappedAttr(fsubject);
           mapSubject.addMapToAttr(nbsubject);

         System.out.println(" *** attribute mapping created *** ");

         dkSchemaMapping sm = new DKSchemaMappingFed();
           sm.setServerName(libSrv);
           sm.setServerType(DK_DSTYPE_IP);
           sm.setEntityName("DOCUMENT");               // Native entity
           sm.setMappedEntityName("FedSample");        // Mapped entity

         // add attribute mapping to this schema
         sm.addAttrMapping(mapDate);
         sm.addAttrMapping(mapSubject);

         System.out.println(" ** schema mapping created ***");

         //  Create a compound query expression:
         //  String values must be enclosed within a pair of single-quotes or
         //    backslah-double-quote (\"). See examples below.
         DKQTerm sTerm = new DKQTerm("FedSubject", DKOpCode.EQ, "'IPSAMP'");
         System.out.println(" Search Term = " + sTerm.toString());

         // Create a query sub-expression
         DKQSubExpr qse = new DKQSubExpr();
         qse.setTranslation(true);
         qse.setMappingName(sm.getName());
         qse.setQueryTerm(sTerm);
         qse.setEntityName("FedSample");
         qse.setDisplayList(new String[]{"FedDate","FedSubject"});

         // Generic Options : MAX_RESULTS=6,CONTENT=YES
         DKNVPair[] opt = new DKNVPair[2];
         opt[0] = new DKNVPair("MAX_RESULTS", "5");
         opt[1] = new DKNVPair("CONTENT","YES");
         qse.setOptionList(opt);
         for (int i = 0; i < opt.length; i++)
           System.out.println(" option[" + i + "] : " + opt[i].getName() +
                                                  "=" + opt[i].getValue().toString());

         // Create a query expression
         DKQExpr qe = new DKQExpr();
         qe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
         qe.addSubQuery(qse);

         // Create a compound query expression
         DKCQExpr cqe = new DKCQExpr(qe);
         cqe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);

         // All set; Connect and execute the compund query expression
         dsIP = new DKDatastoreIP();
         System.out.println(" *** connecting to IP/390 datastore ***");
         String s = new String("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
         dsIP.connect(libSrv, userid, pw, s);
         System.out.println(" \t -----> connected ");

         // Do not forget to register the schema mapping
         dsIP.registerMapping(new DKNVPair("SCHEMA", sm));
         dkResultSetCursor rsc = dsIP.execute(cqe);

         // Process the result set cursor :
         if (rsc == null)
            System.out.println(" *** cursor is null");
         else {
              DKDDO ddo;
              while (rsc.isValid())  {
                ddo = rsc.fetchNext();
                if (ddo != null) {
                  System.out.println(" PrimaryId : " + ddo.getPid().getPrimaryId());
                  displayDDO(ddo);
                }
              }
         }

         // disconnects
         System.out.println("\n\n *** disconnect from the datastore IP ... ***");
         dsIP.disconnect();
         System.out.println(" *** datastore IP disconnected ***");
         dsIP.destroy();
      }
      catch (DKException exc)
      {
         try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         System.out.println("Exception error code " + exc.errorCode());
         System.out.println("Exception error state " + exc.errorState());
         exc.printStackTrace();
      }
      catch (Exception exc)
      {
         try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
   }  // end main...

  // simple method to show DDO hit
  public static void displayDDO(DKDDO ddo)
                throws DKException, Exception
  {
    String   dataName;
    short    cnt;
    Object a = null;
    dkDataObjectBase pDO = null;
    dkXDO pXDO = null;
    DKPidXDO pidXDO = null;
    String pidString;
    DKPidXDOIP pidXDOIP = null;

    if (ddo != null) {
       cnt = ddo.dataCount();
       for (short i = 1; i <= cnt; i++) {
           a = ddo.getData(i);
           dataName = ddo.getDataName(i);
           System.out.print(i + ": Attribute Name :" + dataName );
           if (a instanceof dkCollection)
           {
              System.out.println("   Attribute Value is a collection");
              dkCollection pCol = (dkCollection)a;
              dkIterator pIter = pCol.createIterator();
              short j = 0;
              while (pIter.more() == true)
              {
                 j++;
                 a = pIter.next();
                 pDO = (dkDataObjectBase)a;
                 if (pDO.protocol() == DK_PDDO)
                 {
                    System.out.println("    DKDDO object " + j + " in collection");
                    System.out.println("      object's pid is " +
                          ((DKDDO)pDO).getPid().pidString());
                 }
                 else if (pDO.protocol() == DK_XDO)
                 {
                    pXDO = (dkXDO)pDO;

                    pidXDOIP = (DKPidXDOIP)pXDO.getPidObject();
                    pidString = pidXDOIP.getPrimaryId();
                    System.out.println( "    " + j + ": DKXDOIP primary id = "
                                      + pidString );

                    pidXDOIP = (DKPidXDOIP) pXDO.getPidObject();
                    System.out.println( "                   Part # = "
                                      + pidXDOIP.getPartId() );

                    pidXDOIP = (DKPidXDOIP) pXDO.getPidObject();
                    System.out.println( "                Part Type = "
                                      + pXDO.getAffiliatedType() );
                 }
              } // end while
           } // end if collection
           else
           {
              System.out.println(",\t   Value :" + a.toString());
           }
       }
    }
  }

}

